!-------------------------------------------------------------------------------
! LABREN: Laboratório de Modelagem e Estudos de Recursos Renováveis de Energia
!-------------------------------------------------------------------------------
! SUBROUTINE: ASTRO
!
!> Calculates eccentricity correction factor, solar declination, and equation
!! of time.
!!
!! @param[in]  JDAY CHARACTER*3 Julian day.
!! @param[out] E0   DOUBLE      Eccentricity correction factor of the Earth's
!!                              orbit.
!! @param[out] DEC  DOUBLE      Solar declination [degrees].
!! @param[out] ET   DOUBLE      Equation of time [minutes].
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE ASTRO (JDAY,E0,DEC,ET)
   use BRconstants
!     calculate eccentricity correction, declination e equation of
!     time
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!     PI    : pi
!     CDR   : change degree to radians
!     DANG  : day angle (radians)
!     DEC   : declination (o)
!     E0    : eccentricity correction factor of earth
!     ET    : equation of time (minutes)
!     JDAY  : julian day
!     COnDA : cossine of n*day angle
!     SInDA : sine of n*day angle
!......................................................................
      ! IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER, INTENT(IN) :: JDAY
      DOUBLE PRECISION, INTENT(OUT)::E0,DEC,ET

      DOUBLE PRECISION :: PI,CDR,DANG,CO1DA,CO2DA,CO3DA,SI1DA,SI2DA,SI3DA
      DOUBLE PRECISION, PARAMETER ::  D0 = 0.006918_dp, DC1 = 0.399912_dp, DC2 = 0.006758_dp, DC3 = 0.002697_dp
      DOUBLE PRECISION, PARAMETER ::  DS1 = 0.070257_dp, DS2 = 0.000907_dp, DS3 = 0.001480_dp
      DOUBLE PRECISION, PARAMETER ::  E00 = 1.000110_dp, EC1 = 0.034221_dp, EC2 = 0.000719_dp
      DOUBLE PRECISION, PARAMETER ::  ES1 = 0.001280_dp, ES2 = 0.000077_dp, ET0 = 0.000075_dp
      DOUBLE PRECISION, PARAMETER ::  TC1    = 0.001868_dp, TC2 = 0.014615_dp, TS1 = 0.032077_dp, TS2 = 0.040890_dp
!     definition of constants bellow
      PI    = DACOS(-1.0_dp)
      CDR  = PI/180.0_dp

!     calculate day angle
!      DANG  = 2.0*PI*(FLOAT(JDAY) - 1.0)/365.0 Função com erro qndo calculado com o dia 1
!      Alterando a equação acima para:
       DANG = 0.017201916_dp*DBLE(JDAY)

!     calculate sines and cossines of the day angle
      CO1DA = DCOS(DANG)
      CO2DA = DCOS(2*DANG)
      CO3DA = DCOS(3*DANG)
      SI1DA = DSIN(DANG)
      SI2DA = DSIN(2*DANG)
      SI3DA = DSIN(3*DANG)
!     calculate declination

      DEC   = (D0 - DC1*CO1DA + DS1*SI1DA - DC2*CO2DA + DS2*SI2DA - DC3*CO3DA + DS3*SI3DA)/CDR
!     calculate eccentricity correction

      E0    = E00 + EC1*CO1DA + ES1*SI1DA + EC2*CO2DA + ES2*SI2DA
!     calculate equation of time

      ET     = (ET0 + TC1*CO1DA - TS1*SI1DA - TC2*CO2DA - TS2*SI2DA)*229.18
      RETURN
END SUBROUTINE ASTRO

!-------------------------------------------------------------------------------
! SUBROUTINE: MONTH
!
!> Returns the month number based on its name, represented by a three character
!! string in English.
!!
!! @param[in]     CMON   CHARACTER*3 First three characters of the month name
!!                                   in English.
!! @param[out]    IMON   INTEGER     Month [1-12].
!!
!! @todo       Convert string to lower case before comparison to cover all
!!             cases.
!! @todo       Replace 'if' statements with a select case.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
SUBROUTINE MONTH(CMON,IMON)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*3 ,intent(in)::CMON
      INTEGER,intent(out):: IMON

      IF((CMON.EQ.'JAN').OR.(CMON.EQ.'Jan').OR.(CMON.EQ.'jan')) IMON=1
      IF((CMON.EQ.'FEB').OR.(CMON.EQ.'Feb').OR.(CMON.EQ.'feb')) IMON=2
      IF((CMON.EQ.'MAR').OR.(CMON.EQ.'Mar').OR.(CMON.EQ.'mar')) IMON=3
      IF((CMON.EQ.'APR').OR.(CMON.EQ.'Apr').OR.(CMON.EQ.'apr')) IMON=4
      IF((CMON.EQ.'MAY').OR.(CMON.EQ.'May').OR.(CMON.EQ.'may')) IMON=5
      IF((CMON.EQ.'JUN').OR.(CMON.EQ.'Jun').OR.(CMON.EQ.'jun')) IMON=6
      IF((CMON.EQ.'JUL').OR.(CMON.EQ.'Jul').OR.(CMON.EQ.'jul')) IMON=7
      IF((CMON.EQ.'AUG').OR.(CMON.EQ.'Aug').OR.(CMON.EQ.'aug')) IMON=8
      IF((CMON.EQ.'SEP').OR.(CMON.EQ.'Sep').OR.(CMON.EQ.'sep')) IMON=9
      IF((CMON.EQ.'OCT').OR.(CMON.EQ.'Oct').OR.(CMON.EQ.'oct')) IMON=10
      IF((CMON.EQ.'NOV').OR.(CMON.EQ.'Nov').OR.(CMON.EQ.'nov')) IMON=11
      IF((CMON.EQ.'DEC').OR.(CMON.EQ.'Dec').OR.(CMON.EQ.'dec')) IMON=12

      RETURN
END SUBROUTINE MONTH
