!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: GLOD8A
!
!> Calls SWTR8A routine that calculates global horizontal irradiation
!!! @param[in]  XLON   DOUBLE     Longitude of the ground station [degrees].
!! @param[in]  YLAT   DOUBLE      Latitude of the ground station [degrees].
!! @param[in]  NDIA  INTEGER      data variable / data /y of year
!! @param[in]  XHOR   DOUBLE      Time the sattelite image was generated [h].
!! @param[in]  TCLEAR  DOUBLE     Parameterized sky transmittance for broadband
!!                                solar radiation in the clear atmosphere.
!! @param[in]  TCLOUD  DOUBLE     Parameterized sky transmittance for broadband
!!                                solar radiation in the extreemly cloudy atmosphere.
!! @param[out]  GLOBAL  DOUBLE    Global horizontal irradiance.
!! @param[out] TRANS  DOUBLE  Parameterized sky transmittance for broadband
!!                            solar radiation in the atmosphere.
!! @param[out] TDIR   DOUBLE  Downward direct flux.
!! @param[in] CEFF   DOUBLE, optional  Cloud effective index between clear and cloudy conditions (maximum optical depth)(USEL0=FALSE)
!!                               or between clear and minimum overcast (DNI=0)  (USEL0=TRUE)
!! @param[in] PEFF   DOUBLE, optional  Used with USEL0=TRUE, cloud optical depth index, used to interpolate
!!                               between minimum overcast (DNI=0) and cloudy conditions with maximum optical depth
!! @param[in] TCDL0   DOUBLE, optional  Used with USEL0=TRUE, atmospheric global transmittance ar minimum overcast (DNI=0) conditions
!! @param[out] GLOBAL   DOUBLE   Global horizontal irratiation
!! @param[out] GLOBAL   XXKT   Clearness Index Kt
!! @see        BRASILSR_irradiances
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
 SUBROUTINE GLOD8A(NDIA,XLON,YLAT,XHOR,TCLEAR,TCLOUD,GLOBAL,XXKT,COSZEN,CEFF,PEFF,TCDL0)
   use BRconstants
   use BRinterface

!      estimates global radiation in cloudy and clear-sky atmospheres
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!     IMON   : month
!     IDAY   : day of month
!     XLON   : longitude of the ground station (o)
!     YLAT   : latitude of
!     RMGN2  : normalized cloud transmittance 0.0 <= RMGN2 <= 1.0
!              (RMGN2 = 1 - REF)
!     RADGLO  : matrix of hourly values of important variables
!             IT = 1 - cossine of zenith angle
!             IT = 2 - normalized cloud transmittance
!             IT = 3 - hourly global radiation
!             IT = 4 - hourly extraterrestrial radiation
!             IT = 5 - zenith angle
!     JDAY   : julian day
!     JDAY   : julian day
!     TRANS   : parameterized sky transmittance for broadband solar radiation
!              in the atmosphere
!......................................................................
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER, INTENT(IN)::NDIA
      DOUBLE PRECISION, INTENT(IN)::XLON,YLAT, XHOR
      DOUBLE PRECISION, intent(in) :: TCLEAR,TCLOUD
      DOUBLE PRECISION, intent(out) :: GLOBAL,XXKT,COSZEN
      DOUBLE PRECISION, intent(in), optional :: CEFF,PEFF,TCDL0

      DOUBLE PRECISION, DIMENSION(5) :: RADGLO
!......................................................................
!     subroutine SWTR8A - calculation of hourly radiation during a day

      CALL SWTR8A(XLON,YLAT,NDIA,XHOR,TCLEAR,TCLOUD,RADGLO,CEFF=CEFF,PEFF=PEFF,TCDL0=TCDL0)
      GLOBAL=RADGLO(3)
      if((radglo(3).ge.0.0).and.(radglo(4).gt.0.0)) then
      XXKT=RADGLO(3)/RADGLO(4)
      else
      XXKT=-2.0
      endif
      COSZEN=RADGLO(1)
!......................................................................
      RETURN
 1002 FORMAT(I3,',',I3,',',F7.2,',',F7.2,5(',',F8.2))
      END
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
