!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: SWTR8A
!
!> Calculates global horizontal irradiation
!!! @param[in]  XLON   DOUBLE     Longitude of the ground station [degrees].
!! @param[in]  YLAT   DOUBLE      Latitude of the ground station [degrees].
!! @param[in]  NDIA  INTEGER      data variable / data /y of year
!! @param[in]  XHOR   DOUBLE      Time the sattelite image was generated [h].
!! @param[in]  TCLEAR  DOUBLE     Parameterized sky transmittance for broadband
!!                                solar radiation in the clear atmosphere.
!! @param[in]  TCLOUD  DOUBLE     Parameterized sky transmittance for broadband
!!                                solar radiation in the extreemly cloudy atmosphere.
!! @param[in]  CDR    DOUBLE  Conversion factor: degrees to radians.
!! @param[out] TRANS  DOUBLE  Parameterized sky transmittance for broadband
!!                            solar radiation in the atmosphere.
!! @param[out] TDIR   DOUBLE  Downward direct flux.
!! @param[in] CEFF   DOUBLE, optional  Cloud effective index between clear and cloudy conditions (maximum optical depth)(USEL0=FALSE)
!!                               or between clear and minimum overcast (DNI=0)  (USEL0=TRUE)
!! @param[in] PEFF   DOUBLE, optional  Used with USEL0=TRUE, cloud optical depth index, used to interpolate
!!                               between minimum overcast (DNI=0) and cloudy conditions with maximum optical depth
!! @param[in] TCDL0   DOUBLE, optional  Used with USEL0=TRUE, atmospheric global transmittance ar minimum overcast (DNI=0) conditions
!! @param[out] RADGLO   DOUBLE,DIMENSION(5) matrix of hourly values of important variables
!             IT = 1 - cossine of zenith angle
!             IT = 2 - normalized cloud transmittance
!             IT = 3 - hourly global radiation
!             IT = 4 - hourly extraterrestrial radiation
!             IT = 5 - zenith angle
!! @see        BRASILSR_irradiances
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
 SUBROUTINE SWTR8A (XLON,YLAT,NDIA,XHOR,TCLEAR, TCLOUD,RADGLO,CEFF,PEFF,TCDL0)
      use BRconstants
      use gradevars
!     calculate hourly radiation during a day
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!     RMGN2   : normalized cloud transmittance 0.0 <= RMGN2 <= 1.0
!              (RMGN2 = 1 - REF)
!     XLON    : longitude of the ground station (o)
!     YLAT    : latitude of the ground station (o)
!     NDIA    : day of year
!     PI      : pi
!     CDR     : change degree to radians
!     SC0     : solar constant (W/m2)
!     JDAY    : julian day
!     E0      : eccentricity correction factor of the earth's orbit
!     DEC     : solar declination (degrees)
!     DECR    : solar declination (radians)
!     YLATR   : latitude of the ground station (radians)
!     CODEC   : cossine of solar declination
!     SIDEC   : sine of solar declination
!     SILAT   : sine of latitude of ground station
!     ET      : equation of time (minutes)
!     XFRARS  : decimal fraction of the hour of sunrise solar
!     XFRASS  : decimal fraction of the hour of sunset solar
!     RADGLO  : matrix of hourly values of important variables
!             IT = 1 - cossine of zenith angle
!             IT = 2 - normalized cloud transmittance
!             IT = 3 - hourly global radiation
!             IT = 4 - hourly extraterrestrial radiation
!             IT = 5 - zenith angle
!     ZN      : time zone number (3.0 - Florianopolis) (hours)
!     TIMCOR  : time correction (hours)
!     TSRA    : apparent time for sunrise (hours)
!     TSSA    : apparent local time for sunset (hours)
!     TSRS    : standard local time for sunrise (hours)
!     TSSS    : standard local time for sunrset (hours)
!     ISTART  : start hour for radiation estimation (hours)
!     IEND    : end hour for radiation estimation (hours)
!     T1S     : standard time at begin of integration interval (hours)
!     T2S     : standard time at end of integration interval (hours)
!     T1A     : apparent time at begin of integration interval (hours)
!     T2A     : apparent time at end of integration interval (hours)
!     DT      : time interval (hours)
!     IT      : hour of sattelite image
!     SIW1    : sine of hour angle at begin of integration interval
!     SIW2    : sine of hour angle at end of integration interval
!     COWI    : cossine of hour angle at middle of integration interval
!     DSIW    : sine of hour angle interval
!     COSZEN  : cossine of zenith angle
!     THETA   : zenith angle (degrees)
!     XI0     : extraterrestrial hourly radiation (Wh/m2)
!     W1      : hour angle at begin of interval (degrees)
!     W2      : hour angle at end of interval (degrees)
!     WI      : characteristic hour angle (degrees)
!     TRANS   : parameterized sky transmittance for broadband solar radiation
!              in the atmosphere
!     RSFCN   : global transmittance of atmosphere
!     RADSFC  : hourly radiation at surface estimated (Wh/m2)
!     COWSR   : cossine of sunrise hour angle
!     WSR     : sunrise hour angle (degrees)
!......................................................................
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      INTEGER, INTENT(IN)::NDIA
      DOUBLE PRECISION, INTENT(IN)::XLON,YLAT, XHOR
      DOUBLE PRECISION, intent(in) :: TCLEAR,TCLOUD
      DOUBLE PRECISION,DIMENSION(5), intent(out) ::RADGLO
      DOUBLE PRECISION, intent(in), optional :: CEFF,PEFF,TCDL0

!     definition of constants bellow
      PI    = ACOS(-1.0_dp)
      CDR  = PI/180.0_dp
      SC0  = 1368.00_dp
!......................................................................
!     subroutine ASTRO - calculation of eccentricity correction,
!     declination an equation of time
!     input  - JDAY
!     output - E0,DEC,ET
      CALL ASTRO(JDAY,E0,DEC,ET)
!......................................................................
!     inicialization of variables
!      ISTART = 9
!      IEND = 21
      DO 10  L    = 1 , 5
   10 RADGLO(L) = -999999.9_dp
!     change degrees to radians and calculate sines and cossines
      DECR = DEC*CDR
      YLATR= YLAT*CDR
      CODEC = DCOS(DECR)
      COLAT = DCOS(YLATR)
      SIDEC = DSIN(DECR)
      SILAT = DSIN(YLATR)
!     calculate time correction
      ZN = 0.0_dp
      TIMCOR = (4.0_dp*(15.0_dp*ZN+XLON)+ET)/60.0_dp
!......................................................................
!     subroutine SUNRISE - calculation of sunrise apparent hour
!     input  - DECR,YLATR,CDR
!     output - TSRA
      CALL SUNRISE(DECR,YLATR,TSRA)


!......................................................................
!     calculate hours of start and end of day
      TSSA  = 24.0_dp-TSRA
      TSRS  = TSRA - TIMCOR
      TSSS  = TSSA - TIMCOR
!......................................................................
!     construction of matrix RADGLOB
      TSOLAR = XHOR+TIMCOR
      WSOLAR    = (12.00_dp - TSOLAR)*15._dp
      COWI  = DCOS(WSOLAR*CDR)
!     characteristic solar zenith angle THETA (o)
      COSZEN = SIDEC*SILAT + CODEC*COLAT*COWI
      THETA  = DACOS(COSZEN)/CDR
!     mean hourly insolation top of atmosphere:
      IF((THETA .LE. 90.0).AND.(THETA .GT. 0.0)) THEN
         XI0   = SC0*E0*COSZEN
      ELSE
         XI0=-1.0_dp
      ENDIF
      RADGLO(1)  =  COSZEN
      if (present(CEFF)) then
        RADGLO(2)  =  CEFF
      ELSE
        RADGLO(2)  =  -1.0_dp
      ENDIF
      RADGLO(4)  =  XI0
      RADGLO(5)  =  THETA

!......................................................................
!     calculate radiation during the day
!......................................................................
       IF (RUN_CS_ONLY) THEN !Running clear sky mode
          RSFCN = TCLEAR 
       ELSE
          IF((CEFF.LT.0.0).OR.(CEFF.GT.1.0_dp).OR.(XI0.LT.0.0)) THEN
             RADSFC=-2000.0_dp
          ELSE
            IF (PRESENT(PEFF).AND.PRESENT(TCDL0)) THEN
               IF((CEFF.LE. 0.99999_dp).AND.(PEFF.LE.0.0001_dp))THEN
                 RSFCN  = (1.0_dp - CEFF) * (TCLEAR - TCDL0) + TCDL0
               ELSEIF((CEFF .GT. 0.99999_dp).AND.(PEFF.GT.0.0001_dp))THEN
                 RSFCN  = (1.0_dp - PEFF) * (TCDL0 - TCLOUD) + TCLOUD
               ENDIF
            ELSE
               RSFCN  = (1.0_dp - CEFF) * (TCLEAR - TCLOUD) + TCLOUD
            ENDIF
          ENDIF
      END IF

      RADSFC = XI0 * RSFCN
       IF(((THETA .LE. 90.0).AND.(THETA .GT. 0.0)).AND.(RADSFC.LT.0.0_dp)) THEN
        WRITE(*,*) 'RADIACAO MENOR QUE ZERO'
        STOP
       ENDIF
      RADGLO(3)=RADSFC

   30 CONTINUE
!......................................................................
      RETURN
      END
