!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: DIFFUSE1
!
!> Calculates the Direct and diffuse horizontal irradiation
!! @param[in] GLOBAL   DOUBLE   Global horizontal irratiation
!! @param[in] GLOBAL   XXKT   Clearness Index Kt
!! @param[in] XIMG   DOUBLE, optional  Cloud effective index between clear and cloudy conditions (maximum optical depth)
!! @param[in] TDIRCLEAR   DOUBLE atmospheric transmittance to direct irradiation
!! @param[out] DIF   DOUBLE   Diffuse horizontal irratiation
!! @param[out] DIR   DOUBLE   Direct horizontal irratiation
!! @see        BRASILSR_irradiances
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
 SUBROUTINE DIFFUSE1(GLOBAL, XXKT,COSZEN,TDIRCLEAR, DIF, DNI, CEFF,PEFF)
   use gradevars
   use BRconstants
!     calculate hourly diffuse radiation during a day using
!        * cloudless direct transmittance
!        * clearness index
!        * global and extraterrestrial radiation
!     Main ideia:   get the direct radiation considering the cloud
!                   contribution and diffuse will be global minus direct
! Aded: 14/05/2002
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      DOUBLE PRECISION, INTENT(IN)::	GLOBAL		! global radiation
      DOUBLE PRECISION, INTENT(IN)::	XXKT  !Clearness Index Kt
      DOUBLE PRECISION, INTENT(IN)::	COSZEN		! solar zenith angle
      DOUBLE PRECISION, INTENT(IN)::	TDIRCLEAR		! solar zenith angle
      DOUBLE PRECISION, INTENT(OUT)::	DIF		! diffuse radiation
      DOUBLE PRECISION, INTENT(OUT)::	DNI		! direct radiation
      DOUBLE PRECISION, intent(in), optional :: CEFF,PEFF
      DOUBLE PRECISION ::	CEFFDIR		! cloud deep coefficient to be used in DIR estimation
      DOUBLE PRECISION :: EXTRATER	! TOA radiation
      DOUBLE PRECISION :: ICEFF   ! inverse cloud cover coefficient from satellite data
      DOUBLE PRECISION :: BETA		! cloud backscatter coefficient
      DOUBLE PRECISION :: TAUW		! cloud transmitance
      DOUBLE PRECISION :: DELTAW		! cloud optical thickness
      DOUBLE PRECISION :: DTAUW		! direct transmittance with clouds
      DOUBLE PRECISION :: G		! cloud asymmetry parameter
      DOUBLE PRECISION :: GLINHA		! corrected G
      DOUBLE PRECISION :: PI, CDR, EQVI, LMIN, INF, SUP, DIR,THETA

!......................................................................
!     definition of constants bellow
      EXTRATER=GLOBAL/XXKT
      PI = ACOS(-1.0_dp)
      CDR = PI/180.0_dp
      G = 0.85_dp
      GLINHA = (G - G*G)/(1 - G*G)
      BETA = 0.5_dp - 3._dp*GLINHA/8._dp - 7._dp*GLINHA**3/128._dp - 9._dp*GLINHA**5/128._dp
      DIF=0.0_dp


      IF (RUN_CS_ONLY) THEN
         TAUW = 1.0
         DELTAW = (1 - TAUW)/(BETA * TAUW)
         DTAUW = DEXP(-DELTAW)
      ELSE
        IF (PRESENT(CEFF)) THEN
          IF (PRESENT(PEFF)) THEN !Original Ceff formulation
            IF((CEFF.GE. 1.99999).OR.(PEFF.GE. 1.99999))THEN
              CEFFDIR=2.0_dp
            ELSE !Recovering Ceff original formulation
              INF=LMAX*COSZEN-LMIN
              IF(INF.LE.0.0)THEN
                 CEFFDIR=2.0_dp
              ELSE
                 IF((CEFF.LE. 0.99999).AND.(PEFF.EQ.0))THEN
                     SUP=L0CEFF*COSZEN-LMIN
                     CEFFDIR=CEFF*SUP/INF
                 ELSEIF((CEFF .GE. 0.99999).AND.(PEFF.GE.0.99999))THEN
                      CEFFDIR=1.0_dp
                 ELSEIF((CEFF .GT. 0.99999).AND.(PEFF.GE.0).AND.(PEFF.LT.0.99999))THEN
                     EQVI=(PEFF*(LMAX-L0CEFF)+L0CEFF)*COSZEN
                     SUP=EQVI-LMIN
                     INF=LMAX*COSZEN-LMIN
                     if(SUP.LE. 0)then
                       CEFFDIR= 0.0_dp
                     else
                       CEFFDIR = (sup/INF)
                     endif
                 ENDIF
                 CEFFDIR = 1.0_dp - CEFFDIR
              ENDIF
            ENDIF
          ELSE
            CEFFDIR = 1.0_dp - CEFF
          ENDIF

          IF ((CEFFDIR >= 0.95_dp).AND.(CEFFDIR < 1.01_dp)) THEN
             TAUW = 1.0_dp
             DELTAW = (1.0_dp - TAUW)/(BETA * TAUW)
             DTAUW = DEXP(-DELTAW)
          ELSEIF((CEFFDIR < 0.95_dp).AND.(CEFFDIR >= 0.0_dp)) THEN !Alterado para teste... padrao 0.95 e 0.05
             TAUW = CEFFDIR + 0.05_dp
             DELTAW = (1.0_dp - TAUW)/(BETA * TAUW)
             DTAUW = DEXP(-DELTAW)
          ELSE
             DTAUW = -1.0_dp
          END IF
        ELSE
           WRITE(*,*)'Effective cloud cover information (CEFF) required when not running in clear sky mode. \n Turn RUN_CS_ONLY to .TRUE. or prepare CEFF with programs in utils directory.'
           STOP
        END IF
      ENDIF

! teoricamente esta express�o � multiplicada por coszen
! teoricamente DELTAW � dividido por coszen
      IF(DTAUW > 0.0) THEN
  ! radia��o direta � calculada a partir do produto da direta de c�u claro por DTAUW
         DIR = DTAUW * TDIRCLEAR * EXTRATER
  ! difusa � determinada pela subt. da global da direta com nuvens
         DIF = GLOBAL - DIR
      ELSE
         DIR=-2000.0
         DIF=-2000.0
      ENDIF
      THETA  = ACOS(COSZEN)/CDR
      IF((THETA.LE.85.0).AND.(THETA.GT.0.0)) THEN
         DNI=DIR/COSZEN
      ELSE
         DNI=-2000.0_dp
      ENDIF
      RETURN

      END SUBROUTINE DIFFUSE1
