!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: AerExtinction
!> Calculates extinction coefficients for the first 5 kilometers according to profile 1
! Maximum extinction coefficient is at the surface
!!!!
!! @param[in]  aod550   DOUBLE     Aerosol optical depth at 550 nm
!! @param[in]  altitude   DOUBLE     Surface elevation [m].
!! @param[out]  Kext   DOUBLE, DIMENSION(:)   extinction coefficients for the first 5 kilometers
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
SUBROUTINE AerExtinction(aod550, altitude, Kext)
     use Brasilsr_interface
     use Brasilsr_constants, only:dp
     implicit none
DOUBLE PRECISION, INTENT(IN) ::aod550, altitude
DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) ::Kext
DOUBLE PRECISION, DIMENSION(51) ::SCAL,SCALX
DOUBLE PRECISION ::FITTOP, FITBOT, H, DLAY, SCATOP, aodBot,  aodC, ZJ, CSCAL,SCABOT
DOUBLE PRECISION, PARAMETER ::  difaod=0.00001_dp, aodTopo=0.02164265_dp, aodClpInit=0.11962_dp
INTEGER::JTOP, JBOT , i,J
DOUBLE PRECISION::layerfrac
integer:: LAYER
SCALX =(/2.10E-6, 2.39E-6,&
     & 2.72E-6, 3.10E-6, 3.53E-6, 4.02E-6, 4.61E-6, 5.29E-6, 6.07E-6,&
     & 6.97E-6, 8.00E-6, 9.25E-6, 1.07E-5, 1.24E-5, 1.43E-5, 1.65E-5,&
     & 1.90E-5, 2.18E-5, 2.51E-5, 2.89E-5, 3.32E-5, 4.49E-5, 6.07E-5,&
     & 8.21E-5, 1.11E-4, 1.50E-4, 1.90E-4, 2.42E-4, 2.96E-4, 3.52E-4,&
     & 4.23E-4, 4.92E-4, 5.63E-4, 6.01E-4, 6.41E-4, 6.43E-4, 6.45E-4,&
     & 6.22E-4, 6.63E-4, 7.14E-4, 7.87E-4, 9.80E-4, 1.41E-3, 2.30E-3,&
     & 3.54E-3, 4.85E-3, 6.43E-3, 8.19E-3, 9.70E-3, 2.58E-2, 6.95E-2/)
  LAYER=int(altitude/1000.0_dp)
  layerfrac=DMOD(altitude,1000.0_dp)

  FITTOP  = 5._dp
  IF ((LAYER>5) .AND. aod550>0.00005) then
    WRITE(*,*) 'Model not prepared for distributing AOD at heights above 5000 m, very low AOD will be assumed'
    Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
  ELSE
    IF (LAYER==0) THEN
      FITBOT  = 0.
      JTOP    = 51 - INT(FITTOP)
      JBOT    = 51 - INT(FITBOT)
      H  = FITTOP - FITBOT
      DLAY    = 1.
      SCATOP = SCALX(JTOP)
      aodBot = aod550 - aodTopo !Optical depth between 0 and 5 km
      IF (aodBot.LE.difaod) THEN  ! aod550 is very low or zero after removing the contribution of the top layers
        Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
      ELSE
       ! WRITE(*,*) 'inic', aod550, aodBot, aodC
        aodC=aodClpInit !Initial value before iterarion, equivalent to AOD from 0 to 5 km, before assimilation of external data
        SCABOT = aodBot*0.6D0 !Initial value for SCABOT before iteration is 0.6*AOD
        CSCAL   = DLOG(SCATOP/SCABOT)/H !Initial value for CSCAL before iterarion using estimated SCABOT
        i=0
        DO WHILE (ABS(aod550-aodC).GT.difaod)
          i=i+1
          IF (i>100) THEN
             WRITE(*,*) 'More than 100 iterations for AOD value of ', aod550, aodBot, aodC
             print*, SCALX(45:51)
             STOP
          END IF
          ZJ = FITTOP + DLAY
          ! aodC=0.D0
          DO 20 J = JTOP,JBOT
             ZJ = ZJ - DLAY
             SCALX(J) = SCABOT*EXP(CSCAL*(ZJ-FITBOT))
          20 CONTINUE
          aodC=SUM(SCALX)
          ! WRITE(*,*)i, ABS(aodBottom-aodC), aodBottom,aodC, SCABOT
          SCABOT= aodBot * (1-EXP(CSCAL))/ (1-(EXP(CSCAL))**5)
          CSCAL   = LOG(SCATOP/SCABOT)/H
        END DO
        SCALX(51)=SCALX(51)*DEXP(-CSCAL*layerfrac/1000.)
        Kext=SCALX(47:51)
      ENDIF
    ELSE
      FITBOT  = DBLE(LAYER)
      JTOP    = 51 - INT(FITTOP)
      JBOT    = 51 - INT(FITBOT)
      H  = FITTOP - FITBOT
      DLAY    = 1.
      SCATOP = SCALX(JTOP)
      aodC=aodClpInit
      aodBot = aod550 - aodTopo !Optical depth between 0 and 5 km
      IF (aodBot.LE.difaod) THEN  ! aod550 is very low or zero after removing the contribution of the top layers
        Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
      ELSE
        SCABOT = aodBot*0.6D0 !Initial value for SCABOT before iteration is 0.6*AOD
        CSCAL   = DLOG(SCATOP/SCABOT)/H !Initial value for CSCAL before iterarion using estimated SCABOT
        DO I=51-LAYER+1,51
          SCALX(I)=0.00001_dp
        END DO
        i=0
        DO WHILE (ABS(aod550-aodC).GT.difaod)
          i=i+1
          IF (i>100) THEN
             WRITE(*,*) 'More than 100 iterations for AOD value of ', aod550, aodBot, aodC
             print*, SCALX(45:51)
             STOP
          END IF
          ZJ = FITTOP + DLAY
          ! aodC=0.D0
          DO 30 J = JTOP,JBOT
             ZJ = ZJ - DLAY
             SCALX(J) = SCABOT*EXP(CSCAL*(ZJ-FITBOT))
          30 CONTINUE
          aodC=SUM(SCALX)
          ! WRITE(*,*)i, ABS(aodBottom-aodC), aodBottom,aodC, SCABOT
          SCABOT= (aodBot-sum(SCALX(51-LAYER+1:51))) * (1-EXP(CSCAL))/ (1-(EXP(CSCAL))**(5-LAYER))
          CSCAL   = LOG(SCATOP/SCABOT)/H
        END DO
        SCALX(JBOT)=SCALX(JBOT)*DEXP(-CSCAL*layerfrac/1000.)
        Kext=SCALX(47:51)
      ENDIF
    ENDIF
  ENDIF
 RETURN
END SUBROUTINE AerExtinction

!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: AerExtinction
!> Calculates extinction coefficients for the first 5 kilometers according to profile 2
! Maximum extinction coefficient is at the second layer
!!!!
!! @param[in]  aod550   DOUBLE     Aerosol optical depth at 550 nm
!! @param[in]  altitude   DOUBLE     Surface elevation [m].
!! @param[out]  Kext   DOUBLE, DIMENSION(:)   extinction coefficients for the first 5 kilometers
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019

SUBROUTINE AerExtinction2(aod550, altitude, Kext)
  use Brasilsr_interface
  use Brasilsr_constants, only:dp
   implicit none
DOUBLE PRECISION, INTENT(IN) ::aod550, altitude
DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) ::Kext
DOUBLE PRECISION, DIMENSION(51) ::SCAL,SCALX
DOUBLE PRECISION ::FITTOP, FITBOT, H, DLAY, SCATOP, aodBot,  aodC, ZJ, CSCAL,SCABOT
DOUBLE PRECISION, PARAMETER ::  difaod=0.00001_dp, aodTopo=0.02164265_dp, aodClpInit=0.11962_dp
INTEGER::JTOP, JBOT , i,J
DOUBLE PRECISION::layerfrac
integer:: LAYER
SCALX =(/2.10E-6, 2.39E-6,&
     & 2.72E-6, 3.10E-6, 3.53E-6, 4.02E-6, 4.61E-6, 5.29E-6, 6.07E-6,&
     & 6.97E-6, 8.00E-6, 9.25E-6, 1.07E-5, 1.24E-5, 1.43E-5, 1.65E-5,&
     & 1.90E-5, 2.18E-5, 2.51E-5, 2.89E-5, 3.32E-5, 4.49E-5, 6.07E-5,&
     & 8.21E-5, 1.11E-4, 1.50E-4, 1.90E-4, 2.42E-4, 2.96E-4, 3.52E-4,&
     & 4.23E-4, 4.92E-4, 5.63E-4, 6.01E-4, 6.41E-4, 6.43E-4, 6.45E-4,&
     & 6.22E-4, 6.63E-4, 7.14E-4, 7.87E-4, 9.80E-4, 1.41E-3, 2.30E-3,&
     & 3.54E-3, 4.85E-3, 6.43E-3, 8.19E-3, 9.70E-3, 2.58E-2, 6.95E-2/)

  LAYER=int(altitude/1000.0_dp)
  layerfrac=DMOD(altitude,1000.0_dp)

  FITTOP  = 5._dp
  IF ((LAYER>5) .AND. aod550>0.00005) then
    WRITE(*,*) 'Model not prepared for distributing AOD at heights above 5000 m, very low AOD will be assumed'
    Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
  ELSE
    IF (LAYER==0) THEN
      FITBOT  = 1._dp
      JTOP    = 51 - INT(FITTOP)
      JBOT    = 51 - INT(FITBOT)
      H  = FITTOP - FITBOT
      DLAY    = 1._dp
      SCATOP = SCALX(JTOP)
      aodC=aodClpInit
      aodBot = aod550 - aodTopo !Optical depth between 0 and 5 km
      IF (aodBot.LE.difaod) THEN  ! aod550 is very low or zero after removing the contribution of the top layers
        Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
      ELSE
        !Initial value before iterarion, equivalent to AOD from 0 to 5 km, before assimilation of external data
        SCABOT = aodBot*0.6D0 !Initial value for SCABOT before iteration is 0.6*AOD
        CSCAL   = DLOG(SCATOP/SCABOT)/H !Initial value for CSCAL before iterarion using estimated SCABOT
        i=0
        DO WHILE (ABS(aod550-aodC).GT.difaod)
          i=i+1
          IF (i>100) THEN
             WRITE(*,*) 'More than 100 iterations for AOD value of ', aod550, aodBot, aodC
             print*, SCALX(45:51)
             STOP
          END IF
          ZJ = FITTOP + DLAY
          DO 20 J = JTOP,JBOT
             ZJ = ZJ - DLAY
             SCALX(J) = SCABOT*EXP(CSCAL*(ZJ-FITBOT))
             ! aodC=aodC+SCAL(J)
          20 CONTINUE
          SCALX(51)=SCALX(49)
          aodC=SUM(SCALX)
          SCABOT= (aodBot- SCALX(51)) * (1-EXP(CSCAL))/ (1-(EXP(CSCAL))**4)
          CSCAL   = LOG(SCATOP/SCABOT)/H
        END DO
        SCALX(51)=SCALX(51)*DEXP(-CSCAL*layerfrac/1000.)
        Kext=SCALX(47:51)
      ENDIF
    ELSE
      FITBOT  = DBLE(LAYER)
      JTOP    = 51 - INT(FITTOP)
      JBOT    = 51 - INT(FITBOT)
      H  = FITTOP - FITBOT
      DLAY    = 1.
      SCATOP = SCALX(JTOP)
      aodC=aodClpInit
      aodBot = aod550 - aodTopo !Optical depth between 0 and 5 km
      IF (aodBot.LE.difaod) THEN  ! aod550 is very low or zero after removing the contribution of the top layers
        Kext=(/0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp,0.00001_dp/)
      ELSE
        SCABOT = aodBot*0.6D0 !Initial value for SCABOT before iteration is 0.6*AOD
        CSCAL   = DLOG(SCATOP/SCABOT)/H !Initial value for CSCAL before iterarion using estimated SCABOT
        DO I=51-LAYER+1,51
          SCALX(I)=0.00001_dp
        END DO
        i=0
        DO 30 WHILE (ABS(aod550-aodC).GT.difaod)
          i=i+1
          IF (i>100) THEN
             WRITE(*,*) 'More than 100 iterations for AOD value of ', aod550, aodBot, aodC
             print*, SCALX(45:51)
             STOP
          END IF
          ZJ = FITTOP + DLAY
          DO J = JTOP,JBOT
             ZJ = ZJ - DLAY
             SCALX(J) = SCABOT*EXP(CSCAL*(ZJ-FITBOT))
             ! aodC=aodC+SCAL(J)
          END DO
          aodC=SUM(SCALX)
          SCABOT= (aodBot- sum(SCALX(51-LAYER+1:51))) * (1-EXP(CSCAL))/ (1-(EXP(CSCAL))**(5-LAYER))
          CSCAL   = LOG(SCATOP/SCABOT)/H
        30 CONTINUE
        SCALX(JBOT)=SCALX(JBOT)*DEXP(-CSCAL*layerfrac/1000.)
        Kext=SCALX(47:51)
      ENDIF
    ENDIF
  ENDIF
 RETURN
END SUBROUTINE AerExtinction2
