!-------------------------------------------------------------------------------
! LABREN: Laboratório de Modelagem e Estudos de Recursos Renováveis de Energia
!-------------------------------------------------------------------------------
! SUBROUTINE: ATMOSPHERE
!
!> Chooses the atmosphere type index based on the temperature at the ground
!! station.
!!
!! @param[in]  TSTAT  REAL*8  Temperature at the ground station [K].
!! @param[out] LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                            1. tropical <br>
!!                            2. midlatitude summer <br>
!!                            3. midlatitude winter <br>
!!                            4. subartic summer <br>
!!                            5. subartic winter
!! @param[out] TS     DOUBLE  The difference between the surface and
!!                            characteristical temperatures [K].
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
      SUBROUTINE ATMOSPHERE(TSTAT, ZSTAT,LATMOS,TS)
       use Brasilsr_interface
       use Brasilsr_constants, only:dp
!     choose atmosphere index (standard atmosphere as function of temperature)

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     TSTAT   : temperature at ground station (K)
!     LATMOS  : atmosphere index
!               1 - tropical
!               2 - midlatitude summer
!               3 - midlatitude winter
!               4 - subartic summer
!               5 - subartic winter
!     TS      : difference between surface and characteristical temperatures
!     TnSF    : temperature for atmosphere type 'n' (K)
!......................................................................

      ! IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT NONE
      DOUBLE PRECISION, INTENT(IN)::TSTAT, ZSTAT
      INTEGER, INTENT(OUT)::LATMOS
      DOUBLE PRECISION, INTENT(OUT)::TS

      DOUBLE PRECISION, PARAMETER::LAPSE_RATE=6.5_DP

      DOUBLE PRECISION::T1SFC,T2SFC,T4SFC,T3SFC,T5SFC
      DOUBLE PRECISION:: SEALEVEL_TEMP


      SEALEVEL_TEMP=TSTAT-(ZSTAT/1000.0_DP)*LAPSE_RATE

!     determine characteristical temperatures for each type of atmosphere
 !> Data from Table 1 @cite mcclatchey1972optical, temperature [K], surface level 0 km
      T1SFC = 300.0_dp
      T2SFC = 294.0_dp
      T4SFC = 287.0_dp
      T3SFC = 272.2_dp
      T5SFC = 257.1_dp

!     choose atmosphere index and calculate difference between surface
!     and characteristical temperatures

      LATMOS = 1
      TS     = SEALEVEL_TEMP - T1SFC
      IF(SEALEVEL_TEMP.LT.294.0_dp .AND. SEALEVEL_TEMP.GE.288.5_dp) THEN
        LATMOS = 2
        TS     = SEALEVEL_TEMP - T2SFC
      ENDIF
      IF(SEALEVEL_TEMP.LT.288.5_dp .AND. SEALEVEL_TEMP.GE.277.1_dp) THEN
        LATMOS = 4
        TS     = SEALEVEL_TEMP - T4SFC
      ENDIF
      IF(SEALEVEL_TEMP.LT.277.1_dp .AND. SEALEVEL_TEMP.GE.262.9_dp) THEN
        LATMOS = 3
        TS     = SEALEVEL_TEMP - T3SFC
      ENDIF
      IF(SEALEVEL_TEMP.LT.262.9_dp) THEN
        LATMOS = 5
        TS     = SEALEVEL_TEMP - T5SFC
      ENDIF

      !    LATMOS = 1
      !    TS     = TSTAT - T1SFC
      ! IF(TSTAT.LT.297.0_dp .AND. TSTAT.GE.290.5_dp) THEN
      !    LATMOS = 2
      !    TS     = TSTAT - T2SFC
      ! ENDIF
      ! IF(TSTAT.LT.290.5_dp .AND. TSTAT.GE.279.5_dp) THEN
      !    LATMOS = 4
      !    TS     = TSTAT - T4SFC
      ! ENDIF
      ! IF(TSTAT.LT.279.5_dp .AND. TSTAT.GE.264.5_dp) THEN
      !    LATMOS = 3
      !    TS     = TSTAT - T3SFC
      ! ENDIF
      ! IF(TSTAT.LT.264.5_dp) THEN
      !    LATMOS = 5
      !    TS     = TSTAT - T5SFC
      ! ENDIF

      RETURN
      END

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
