!> @brief Module for specific grid point calculations.
!! @details Includes custom types to introduce local data
!! and to save spectral data at specific grid points
!! @author     LABREN
!! @version    1.0
!! @date       2020
!-------------------------------------------------------------------------------

MODULE Brasilsr_spectral
  use Brasilsr_constants
  use gradevars
  TYPE :: point_withdata_defs
      CHARACTER(len=30) :: NOME
      DOUBLE PRECISION :: LATI
      DOUBLE PRECISION :: LONGI
      DOUBLE PRECISION :: ALTI
      INTEGER :: GRADI
      INTEGER :: GRADJ
      DOUBLE PRECISION, DIMENSION(Ntimes) :: AOD550
      DOUBLE PRECISION, DIMENSION(Ntimes) :: EA
      DOUBLE PRECISION, DIMENSION(Ntimes) :: PWV
      DOUBLE PRECISION, DIMENSION(Ntimes) :: O3
  END TYPE point_withdata_defs
  TYPE :: point_2save_defs
      CHARACTER(len=30) :: NOME
      DOUBLE PRECISION :: LATI
      DOUBLE PRECISION :: LONGI
      DOUBLE PRECISION :: ALTI
      INTEGER :: GRADI
      INTEGER :: GRADJ
  END TYPE point_2save_defs

  DOUBLE PRECISION, dimension(MAXWEL,run_points):: spectral_dir_surf, spectral_dif_surf, spectral_dir_scaled_surf
  DOUBLE PRECISION, dimension(MAXWEL,run_points):: spectral_dir_top, spectral_dif_top
END MODULE Brasilsr_spectral

!-------------------------------------------------------------------------------
!> @brief Module for spectral definitions
!! @details Defines weights based in the Angstrom exponent if present as input,
!! otherwise uses default values.
!! @author     LABREN
!! @version    1.0
!! @date       2020
!-------------------------------------------------------------------------------
MODULE Brasilsr_commonspectral
  use Brasilsr_constants

 contains
 function init_exaer(i,EA) RESULT(EXAER_RESULT)
   integer, intent(IN)::i
   double precision, intent(in), optional:: EA
   double precision, dimension(MAXWEL)::EXAER_RESULT
   DOUBLE PRECISION, DIMENSION(MAXWEL,5)::EXAER_ORIG
   !Old values of mean EA
   ! DOUBLE PRECISION, PARAMETER::EA_AM= 1.862_dp, EA_CE=1.772_dp
   DOUBLE PRECISION, PARAMETER::EA_AM= 1.819_dp, EA_CE=1.668_dp

   ! Brasil-SR ln(lambdax/lambda550) for the 37 spectral intervals
   DOUBLE PRECISION, PARAMETER, DIMENSION(MAXWEL):: LN_LAMBDA_RATIO = &
       &(/0.996712_dp, 0.962811_dp, 0.916291_dp, 0.871839_dp, 0.829279_dp, &
       &  0.788457_dp, 0.749237_dp, 0.711496_dp, 0.675129_dp, 0.640037_dp, &
       &  0.599491_dp, 0.541597_dp, 0.480973_dp, 0.423814_dp, 0.369747_dp, &
       &  0.301105_dp, 0.200671_dp, 0.095310_dp, 0.000000_dp, -0.087011_dp,&
       &  -0.167054_dp, -0.231112_dp, -0.289952_dp, -0.329958_dp, -0.345522_dp, &
       &  -0.406676_dp, -0.466589_dp, -0.543381_dp, -0.639938_dp, -0.733242_dp, &
       &  -0.816169_dp, -0.952710_dp, -1.082113_dp, -1.234414_dp, -1.365628_dp, &
       &  -1.551424_dp, -1.787204_dp/)

   EXAER_ORIG(:,1)  = (/1.493_dp, 1.502_dp, 1.514_dp, 1.526_dp, 1.538_dp, 1.550_dp, 1.550_dp, 1.550_dp, 1.550_dp, &
                  1.550_dp, 1.548_dp, 1.534_dp, 1.513_dp, 1.469_dp, 1.424_dp, 1.360_dp, 1.249_dp, 1.123_dp, &
                  1.000_dp, 0.893_dp, 0.789_dp, 0.705_dp, 0.644_dp, 0.605_dp, 0.587_dp, 0.518_dp, 0.458_dp, &
                  0.306_dp, 0.304_dp, 0.245_dp, 0.193_dp, 0.130_dp, 0.086_dp, 0.051_dp, 0.033_dp, 0.016_dp, &
                  0.086_dp/)
  ! Continental aerosol @cite Schmetz1993 Table1
   EXAER_ORIG(:,2)  = (/2.414_dp, 2.366_dp, 2.302_dp, 2.238_dp, 2.174_dp, 2.110_dp, 2.054_dp, 1.998_dp, 1.942_dp, &
                   1.886_dp, 1.818_dp, 1.733_dp, 1.638_dp, 1.559_dp, 1.479_dp, 1.378_dp, 1.252_dp, 1.113_dp, &
                   1.000_dp, 0.909_dp, 0.824_dp, 0.759_dp, 0.712_dp, 0.682_dp, 0.668_dp, 0.614_dp, 0.569_dp, &
                   0.524_dp, 0.459_dp, 0.411_dp, 0.368_dp, 0.311_dp, 0.260_dp, 0.194_dp, 0.158_dp, 0.144_dp, &
                   0.110_dp/)
   ! Marine aerosol @cite Schmetz1993 Table1
    EXAER_ORIG(:,3)  = (/1.316_dp, 1.304_dp, 1.288_dp, 1.272_dp, 1.256_dp, 1.240_dp, 1.226_dp, 1.212_dp, 1.198_dp, &
                    1.184_dp, 1.167_dp, 1.148_dp, 1.128_dp, 1.112_dp, 1.096_dp, 1.076_dp, 1.052_dp, 1.021_dp, &
                    1.000_dp, 0.980_dp, 0.961_dp, 0.949_dp, 0.939_dp, 0.933_dp, 0.930_dp, 0.920_dp, 0.910_dp, &
                    0.899_dp, 0.883_dp, 0.866_dp, 0.850_dp, 0.823_dp, 0.793_dp, 0.748_dp, 0.704_dp, 0.584_dp, &
                    0.665_dp/)
   ! Amazonia
    EXAER_ORIG(:,4)=DEXP(EA_AM*LN_LAMBDA_RATIO)
   ! ! Cerrado
    EXAER_ORIG(:,5)=DEXP(EA_CE*LN_LAMBDA_RATIO)

    IF (PRESENT(EA)) THEN
      EXAER_RESULT=DEXP(EA*LN_LAMBDA_RATIO)
    ELSE
      EXAER_RESULT=EXAER_ORIG(:,i)
    END IF
 end function init_exaer

 END MODULE Brasilsr_commonspectral

 !-------------------------------------------------------------------------------
 !> @brief Module for interfaces
 !! @author     LABREN
 !! @version    1.0
 !! @date       2020
 !-------------------------------------------------------------------------------
MODULE Brasilsr_interface

INTERFACE
  DOUBLE PRECISION function BSA(fiso, fgeo,fvol,Z)
   use Brasilsr_constants
    DOUBLE PRECISION, intent (in) ::fiso, fgeo,fvol,Z
  end function BSA
END INTERFACE

INTERFACE
 DOUBLE PRECISION function WSA(fiso, fgeo,fvol)
   use Brasilsr_constants
   DOUBLE PRECISION, intent (in) ::fiso, fgeo,fvol
 end function WSA
END INTERFACE


INTERFACE
  SUBROUTINE ASTRO (JDAY,E0,DEC,ET)
    INTEGER, INTENT(IN)::JDAY
    DOUBLE PRECISION, INTENT(OUT)::E0,DEC,ET
  END SUBROUTINE ASTRO
END INTERFACE

INTERFACE
  SUBROUTINE ATMOSPHERE(TSTAT,ZSTAT,LATMOS,TS)
    DOUBLE PRECISION, INTENT(IN)::TSTAT,ZSTAT
    INTEGER, INTENT(OUT)::LATMOS
    DOUBLE PRECISION, INTENT(OUT)::TS
  END SUBROUTINE ATMOSPHERE
END INTERFACE

INTERFACE
  SUBROUTINE AerExtinction(aod550, altitude, Kext)
    DOUBLE PRECISION, INTENT(IN) ::aod550, altitude
    DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) ::Kext
  END SUBROUTINE AerExtinction
END INTERFACE

INTERFACE
  SUBROUTINE AerExtinction2(aod550, altitude, Kext)
    DOUBLE PRECISION, INTENT(IN) ::aod550, altitude
    DOUBLE PRECISION, DIMENSION(:), INTENT(OUT) ::Kext
  END SUBROUTINE AerExtinction2
END INTERFACE

INTERFACE
  SUBROUTINE LESEN(LATMOS,Kext,ZSTAT,E0,SOL,DZ,PRESS,ALT,TEMP,ROL,ROH,RO3,SCA,RAY)
    use Brasilsr_constants
    use Brasilsr_spectral_grid, only: MAXIND, WEIGHT,SPECTRAL_INDEX
    INTEGER, INTENT(IN):: LATMOS
    DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: Kext
    DOUBLE PRECISION, INTENT(IN) :: ZSTAT,E0
    DOUBLE PRECISION, DIMENSION(MAXIND), INTENT(OUT) :: SOL
    DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT) :: DZ
    DOUBLE PRECISION, DIMENSION(NLEV), INTENT(OUT)   :: PRESS
    DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(OUT) :: ALT, TEMP, ROL, ROH, RO3, SCA,RAY
  END SUBROUTINE LESEN
END INTERFACE

INTERFACE
  SUBROUTINE AERO(BIOMA,SCA,DZ,ALT,INAERO,ASIG,EAngstrom)
    use Brasilsr_constants
    use Brasilsr_commonspectral
    INTEGER, INTENT(in)::BIOMA
    DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(in)::DZ
    DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(in)::SCA,ALT
    INTEGER, DIMENSION(LAYMAX),INTENT(OUT)::INAERO
    DOUBLE PRECISION, DIMENSION(LEVMAX,MAXWEL), INTENT(OUT)::ASIG
    DOUBLE PRECISION, INTENT(IN),OPTIONAL::EAngstrom
  END SUBROUTINE AERO
END INTERFACE

INTERFACE
   SUBROUTINE BETAS(NCL,IALL,COSZEN,TW,K1,IWP,LCLOUD,INAERO,ASIG,U,V,BET,BET0,OM,TAU,TOHNW,scaling)
      use Brasilsr_constants
      use Brasilsr_spectral_grid
     LOGICAL, INTENT(IN),OPTIONAL::scaling
     LOGICAL*1, INTENT(IN):: LCLOUD
     INTEGER, INTENT(IN)::NCL, IALL, K1,IWP
     INTEGER, DIMENSION(LAYMAX), INTENT(IN)::INAERO
     DOUBLE PRECISION, INTENT(IN)::COSZEN
     DOUBLE PRECISION, DIMENSION(LEVMAX,MAXWEL), INTENT(IN)::ASIG
     DOUBLE PRECISION, DIMENSION(LAYMAX,MAXIND), INTENT(IN)::U,V
     DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(IN)::TW
     DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT):: BET,BET0,OM,TAU,TOHNW
  END SUBROUTINE BETAS
END INTERFACE

INTERFACE
  SUBROUTINE D2STR(JDAY,XLON,YLAT,BIOMA,TSTAT,fiso,fgeo,fvol,RFSURF,PWV,O3,ZSTAT,Kext,XHOR,ICLOUD,TAUW,TRANS,TDIR,COSZ,EAngstrom,scaling,ip)
    use Brasilsr_constants
    use gradevars
    INTEGER, INTENT(IN)::JDAY,BIOMA
    DOUBLE PRECISION, INTENT(IN)::XLON,YLAT,TSTAT,RFSURF,ZSTAT,PWV,O3
    DOUBLE PRECISION, dimension(:), INTENT(IN) :: fiso,fgeo,fvol
    DOUBLE PRECISION, dimension(:), intent(in) :: Kext
    DOUBLE PRECISION, intent(in) :: XHOR
    DOUBLE PRECISION, intent(in), optional :: EAngstrom
    INTEGER, intent(in) :: ICLOUD
    LOGICAL, intent(in),OPTIONAL:: scaling
    DOUBLE PRECISION, intent(in) ::TAUW
    DOUBLE PRECISION, intent(out) ::TRANS,TDIR,COSZ
    INTEGER, INTENT(IN),OPTIONAL::ip
  END SUBROUTINE D2STR
END INTERFACE

INTERFACE
  SUBROUTINE STRPSRB(BIOMA,LATMOS,TS,ROFF,ALDIF_MAXWEL,ALDR_MAXWEL,PWV,O3,Kext,THETA,E0,ICLOUD,IWP,ISUB,TAUW,TOP,NCL,INTVAL,tstat,rf,zstat,CLOLWC,CDR,TRANS,TDIR,EAngstrom,scaling,ip)
    use Brasilsr_constants
    INTEGER, INTENT(IN)::LATMOS, ICLOUD, IWP, ISUB, NCL, INTVAL, BIOMA
    DOUBLE PRECISION, INTENT(IN):: TS, ROFF,PWV,O3
    DOUBLE PRECISION, dimension(:), INTENT(IN):: ALDIF_MAXWEL,ALDR_MAXWEL
    DOUBLE PRECISION, INTENT(IN):: TSTAT, RF, ZSTAT
    DOUBLE PRECISION, dimension(:), intent(in) :: Kext
    LOGICAL, intent(in),OPTIONAL:: scaling
    DOUBLE PRECISION, intent(in), optional :: EAngstrom
    DOUBLE PRECISION, INTENT(IN)::THETA,E0, TAUW, TOP, CLOLWC, CDR
    DOUBLE PRECISION, INTENT(OUT)::TRANS, TDIR
    INTEGER, INTENT(IN),OPTIONAL::ip
  END SUBROUTINE STRPSRB
END INTERFACE
!--------------------------------------------
contains
subroutine handler_alloc(IERR,my_errmsg)
  integer, intent (in) :: IERR
  CHARACTER(256), intent (in)  :: my_errmsg
  write(*,*) 'Failed to allocate with stat = ', my_stat, ' and errmsg '//trim(my_errmsg)
end subroutine handler_alloc
!--------------------------------------------
subroutine handler_dealloc(IERR,my_errmsg)
  integer, intent (in) :: IERR
  CHARACTER(256), intent (in)  :: my_errmsg
  write(*,*) 'Failed to allocate with stat = ', my_stat, ' and errmsg '//trim(my_errmsg)
end subroutine handler_dealloc
END MODULE Brasilsr_interface
