!-------------------------------------------------------------------------------
! LABREN:  Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: STRPSRB
!
!> Calculates transmittance for clear and cloudy sky.
!! Adapted from IGMK model as described in @cite stuhlmann/1990 and @cite Schmetz1993.
!!
!! @param[in]  BIOMA INTEGER Brazilian biome:<br>
!!                            1: Amazonia -> aerosol type 4
!!                            3: Cerrado -> aerosol type 5
!!                            all other values -> aerosol type 3 (continental polluted)
!! @param[in]  LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                            1. tropical <br>
!!                            2. midlatitude summer <br>
!!                            3. midlatitude winter <br>
!!                            4. subartic summer <br>
!!                            5. subartic winter
!! @param[in]  TS     DOUBLE  The difference between the surface and
!!                            characteristical temperatures [K].
!! @param[in]  ROFF   DOUBLE  Difference in water vapor.
!! @param[in]  ALDIF_MAXWEL DOUBLE, DIMENSION(:)  Surface albedo to diffuse irradiation (broadband or spectral, according to USE_SPECTRAL_ALB parameter)
!! @param[in]  ALDIR_MAXWEL DOUBLE, DIMENSION(:)  Surface albedo to direct irradiation (broadband or spectral, according to USE_SPECTRAL_ALB parameter)
!! @param[in]  PWV   DOUBLE   Precipitable water vapor (cm)
!! @param[in]  O3   DOUBLE   Column ozone (Dobson)
!! @param[in]  Kext   DOUBLE, DIMENSION(5) Extinction coeficient from 0 to 5 km.
!! @param[in]  THETA  DOUBLE  Zenith angle [degrees].
!! @param[in]  ICLOUD INTEGER Cloud parameter. <br>
!!                            0: no clouds <br>
!!                            1: calculation with clouds
!! @param[in]  IWP    INTEGER Cloud droplet size distribution.
!! @param[in]  ISUB   INTEGER Defines which WOLKE subroutine will be used: <br>
!!                            1. WOLKE1 <br>
!!                            2. WOLKE2 <br>
!!                            ISUB=2 (value was set in D2STR), so only WOLKE2
!!                            is called
!! @param[in]  TAUW   DOUBLE  Cloud optical thickness. [0-100] <br>
!!                            0.0:   no clouds <br>
!!                            100.0: completely cloudy
!! @param[in]  TOP    DOUBLE  Cloud top height [mbar].
!! @param[in]  NCL    INTEGER Number of cloud layers.
!! @param[in]  INTVAL INTEGER Spectral interval.
!! @param[in]  TSTAT  REAL*8  Temperature at the surface [K].
!! @param[in]  RF     REAL*8  Relative humidity [0-1].
!! @param[in]  ZSTAT  REAL*8  Surface elevation [m].
!! @param[in]  CLOLWC DOUBLE  Cloud liquid water content.
!! @param[in]  CDR    DOUBLE  Conversion factor: degrees to radians.
!! @param[out] TRANS  DOUBLE  Parameterized sky transmittance for broadband
!!                            solar radiation in the atmosphere.
!! @param[out] TDIR   DOUBLE  Downward direct flux.
!! @param[in] EAngstrom   DOUBLE, optional  Angstrom exponent. If not entered, a climatological value is used.
!! @param[in] scaling LOGICAL, optional whether use delta-Eddington scaling or not.
!! @param[in] IP integer, index representing grid point in spectral variables.
!!
!! @see        D2STR
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
 SUBROUTINE STRPSRB(BIOMA,LATMOS,TS,ROFF,ALDIF_MAXWEL,ALDR_MAXWEL,PWV,O3,Kext,THETA,E0,ICLOUD,IWP,ISUB,TAUW,TOP,NCL,INTVAL,tstat,rf,zstat,CLOLWC,CDR,TRANS,TDIR,EAngstrom,scaling,ip)
 ! use omp_lib
 ! use gradevars
 use Brasilsr_interface
 use Brasilsr_constants
 use Brasilsr_spectral
 use Brasilsr_spectral_grid, only: MAXIND, SPECTRAL_INDEX
!     calculate transmittance for clear and cloudy sky

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     LATMOS  : atmosphere type
!     1 - tropical
!     2 - midlatitude summer
!     3 - midlatitude winter
!     4 - subartic summer
!     5 - subartic winter
!     TS : difference between surface and characteristical temperatures
!     ROFF    : difference in water vapor
!     SFCALB  : surface albedo (0/1)
!     Kext(5) : Extinction coeficient from 0 to 5 km
!     THETA   : zenith angle (o)
!     ICLOUD  : cloud parameter
!    0 - no clouds
!    1 - calculation with clouds
!     IWP     : cloud droplet size distribution
!     ISUB    : use subroutine WOLKE1 (ISUB=1) or WOLKE2 (ISUB=2)
!     TAUW    : cloud optical thickness
!    0.0 - no clouds
!    100.0 - with clouds
!     TOP     : cloud top (mbar)
!     NCL     : number of cloud layers
!     INTVAL  : spectral interval
!     INTVA   : value of the begin of spectral interval
!     INTVE   : value of the end of spectral interval
!     WH2O    : precipitable water [cm]
!     CLOLWC  : cloud liquid water content
!     CDR     : change degrees to radians
!     TRANS   : parameterized sky transmittance for broadband solar radiation
!    in the atmosphere
!     SOL     : solar constant for each spectral interval
!     DZ      : thickness of each atmospheric layer
!     ROL     : air density
!     ROH     : water content (humidity)
!     RO3     : ozone content
!     IH : data for index of spectral intervals
!     LCLOUD  : logical variable to calculate with or without clouds
!     PRESS   : atmospheric pressure (mbar)
!     ALT     : altitude (m)
!     TEMPE    : Temperature (oC)
!     SCA     :
!     RAY     :
!     IGO     : flag for CLOLWC greater than limits
!     TW :
!     K1 :
!     DIR     : direct irradiation for each spectral interval (Wh/m2)
!     DIF     : diffuse irradiation for each spectral interval(Wh/m2)
!     DOWN    : global irradiation (Wh/m2)
!     DIRS    : direct irradiation (Wh/m2)
!     DIFS    : diffuse irradiation (Wh/m2)
!     IBEG    : begin of spectral interval
!     ISTO    : end of spectral interval
!     IALL    : counter of spectral interval
!     IDO     : flag to execute the loop of spectral intervals
!     NLA     : NLEV-1
!     NLY     : NLAY-1
!     ALDIF   : surface albedo for cloudy sky
!     ALDR    : surface albedo for clear sky
!     COSZEN  : cossine of zenith angle
!     LEVMAX  : maximum number of atmopheric levels
!     LAYMAX  : maximum number of atmopheric layers
!     MAXWEL  : number of spectral intervals
!     MAXIND  : number of spectral intervals including their subdivisions
!     IDO     :
!     BET     :
!     BET0    :
!     OM :
!     TAU     :
!     TOHNW   :
!     A  :
!     AA :
!     DW :
!     INAERO  :
!     ASIG    :
!     U  :
!     V  :
!     WH2O1   :
!     WCO2SW  :
!     WO3SW   :
!     WO2SW   :
!     RAYM    :
!......................................................................

 ! IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 IMPLICIT NONE

 INTEGER, INTENT(IN)::LATMOS, ICLOUD, IWP, ISUB, NCL, INTVAL, BIOMA
 DOUBLE PRECISION, INTENT(IN):: TS, ROFF,PWV,O3
 DOUBLE PRECISION, dimension(:), INTENT(IN):: ALDIF_MAXWEL,ALDR_MAXWEL
 DOUBLE PRECISION, INTENT(IN):: TSTAT, RF, ZSTAT
 DOUBLE PRECISION, dimension(:), intent(in) :: Kext
 LOGICAL, intent(in),optional:: scaling
 DOUBLE PRECISION, INTENT(IN),OPTIONAL::EAngstrom
 DOUBLE PRECISION, INTENT(IN)::THETA, E0,TAUW, TOP, CLOLWC, CDR
 DOUBLE PRECISION, INTENT(OUT)::TRANS, TDIR
 INTEGER, INTENT(IN),OPTIONAL::ip
 DOUBLE PRECISION:: ALDIF,ALDR

 DOUBLE PRECISION, DIMENSION(LAYMAX,MAXIND) :: U,V
 DOUBLE PRECISION, DIMENSION(LEVMAX,MAXWEL) :: ASIG
 DOUBLE PRECISION, DIMENSION(LEVMAX) :: SCA, RAY,TEMPE,ALT,ROL,ROH,RO3
 DOUBLE PRECISION, DIMENSION(LEVMAX) :: DOWN,DIR,DIF,DIRS,DIFS
 DOUBLE PRECISION, DIMENSION(NLEV) :: PRESS
 DOUBLE PRECISION, DIMENSION(LAYMAX) :: DZ,TW,TOHNW,BET,BET0,TAU, OM
 DOUBLE PRECISION, DIMENSION(LAYMAX) :: WH2O1,WCO2SW,WO3SW,WO2SW,RAYM
 DOUBLE PRECISION, DIMENSION(MAXIND) :: SOL
 INTEGER, DIMENSION(LAYMAX)::INAERO
 INTEGER, DIMENSION(8)::INTVA,INTVE
 LOGICAL* 1 :: LCLOUD
 DOUBLE PRECISION :: WH2O,COSZEN,A,AA,DW,layerfrac,XDIR
 INTEGER :: IGO,IBEG,ISTO,IALL,IDO,NLA,NLY,I,K1,LAYER


  INTVA = (/20, 16,  1,  0,  0,  0,  0,  1/)

  INTVE = (/29, 30, 37,  0,  0,  0,  0, 37/)

 ! DO 10   I =  1 ,  22
 !    10 INDEX(I) = I
 ! DO 20   I = 23 , MAXIND
 !    20 INDEX(I) = IH(I-22)

 LCLOUD = .FALSE.
 IF (ICLOUD .GT. 0) THEN
 LCLOUD = .TRUE.
 ENDIF
 LAYER=floor(ZSTAT/1000.0_dp)
 layerfrac=DMOD(ZSTAT,1000.0_dp)/1000.0_dp

 if (PRESENT(ip)) then
   if (PRESENT(scaling)) then
      spectral_dir_surf(:,ip)=0.00_dp
   else
     spectral_dir_scaled_surf(:,ip)=0.00_dp
     spectral_dif_surf(:,ip)=0.00_dp
   end if
   spectral_dir_top(:,ip)=0.00_dp
   spectral_dif_top(:,ip)=0.00_dp
 endif
!******************************************************************************

!     subroutine LESEN - determination of atmospheric profiles
!     input  - LATMOS,VIS
!     output - SOL,DZ,PRESS,ALT,TEMPE,ROL,ROH,RO3,SCA,RAY

 CALL LESEN(LATMOS,Kext,ZSTAT,E0,SOL,DZ,PRESS,ALT,TEMPE,ROL,ROH, RO3,SCA,RAY)


!!******************************************************************************

!     MODIFICACOES SUGERIDAS PELO FERNANDO PARA PREVER A AGUA PRECIPITAVEL
!     A DIFERENTES ALTITUDES

!     calculation of the precitable water as function of relative
!     humidity, partial pressure of water vapor and temperature

!  XD=287.05_dp/1005.0_dp
!  PSTAT = 1013.25_dp*EXP(-0.0001184_dp*ZSTAT)
!  PVSAT = EXP(26.23_dp - 5416._dp/TSTAT)/100._dp
!  WMIXRAT=0.622_dp*RF*PVSAT/1013.25_dp
!  XM=(1.0_dp-0.26_dp*WMIXRAT)*XD
!  TNEW=TSTAT*((PSTAT/1013.25_dp)**XM)
!  PVSAT = EXP(26.23_dp - 5416._dp/TNEW)
!  WH2O  = 0.493_dp*RF*PVSAT/TNEW
!
! !     test for precitable water less than 0.0000001 cm
!
!  IF (WH2O .LE.0.0000001_dp) THEN
!     WRITE (*,*)'NEGATIVE PRECIPITABLE WATER VAPOR',WH2O,' IS SET TO 0.000001 (cm)',TSTAT,ZSTAT
!     READ(*,*)
!     WH2O  = 0.0000001_dp
!  ENDIF
WH2O=PWV
!     TERMINO DAS MODIFICACOES

!
!......................................................................

 IGO = 1
 IF(ISUB.EQ.1) THEN
!......................................................................

!     subroutine WOLKE1 - determination of clouds properties, input is
!     CLOLWC
!     input  - NCL,TOP,IWP,LCLOUD,CLOLWC,DZ,TEMPE,PRESS
!     output - NCL,IGO,TAUW,TW,K1
      CALL WOLKE1(NCL,IGO,TOP,TAUW,IWP,LCLOUD,CLOLWC,DZ,TEMPE,PRESS,TW,K1)
!......................................................................
 ELSE
!......................................................................
!     subroutine WOLKE2 - determination of clouds properties, input is
!     TAUW
!     input  - NCL,TOP,TAUW,IWP,LCLOUD,DZ,TEMPE,PRESS
!     output - NCL,CLOLWC,TW,K1
      CALL WOLKE2(NCL,TOP,TAUW,IWP,LCLOUD,CLOLWC,DZ,TEMPE,PRESS,TW,K1)
!......................................................................
 END IF

 IF(IGO.NE.1) RETURN

!......................................................................
!     subroutine VARIA - changes atmospheric profiles
!     input  - RAY,NCL,TS,ROFF,WH2O,K1,LCLOUD,DZ,TEMPE,PRESS,ROL,ROH,RO3
!     output - TEMPE,ROH,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM
 CALL VARIA(ZSTAT,RAY,NCL,TS,ROFF,WH2O,O3,K1,LCLOUD,DZ,TEMPE,PRESS,ROL,ROH,RO3,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM)

!......................................................................
!     subroutine AERO - determination of aerosol profiles
!     input  - SCA,DZ,ALT
!     output - INAERO,ASIG
 CALL AERO(BIOMA,SCA,DZ,ALT,INAERO,ASIG,EAngstrom=EAngstrom)

!......................................................................
!     subroutine ATMOS - determination of absorption and scattering
!     input  - DZ,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM
!     output - U,V
 CALL ATMOS(DZ,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM,U,V)
!......................................................................

 COSZEN= DCOS(CDR*THETA)

!     condition to avoid that some parts of the map be in the night
 IF(COSZEN.LT.0.01_dp) COSZEN=0.01_dp

 DO I=1,LEVMAX
    DIR(I)    = 0.0_dp
    DIF(I)    = 0.0_dp
    DOWN(I)   = 0.0_dp
    DIRS(I)   = 0.0_dp
    DIFS(I)   = 0.0_dp
 enddo
!.....loop spectral intervals..........................................

 IBEG = INTVA(INTVAL)
 ISTO = INTVE(INTVAL)

 IF(IBEG.EQ.0.AND.ISTO.EQ.0) GOTO 100

    DO 40 IALL = 1,MAXIND
       IDO = SPECTRAL_INDEX(IALL)
       IF(IDO.LT.IBEG.OR.IDO.GT.ISTO) GOTO 40

       NLA = NLEV
       NLY = NLAY

       ALDIF = ALDIF_MAXWEL(IDO)
       ALDR  = ALDIF_MAXWEL(IDO)

!......................................................................

!     subroutine BETAS - determination of spectral absorption, extinction
!     and scattering for each atmospheric layer
!     input  - NCL,IALL,COSZEN,TW,K1,IWP,LCLOUD,INAERO,ASIG,U,V
!     output - BET,BET0,OM,TAU,TOHNW
       CALL BETAS (NCL,IALL,COSZEN,TW,K1,IWP,LCLOUD,INAERO,ASIG,U,V,BET,BET0,OM,TAU,TOHNW,scaling=scaling)

!......................................................................
       DIR(1)  = COSZEN
       A  = 0.0_dp
       AA = 0.0_dp


       DO I = 2 , NLEV
          A  =  A + TOHNW(I-1)
          AA =  AA + TAU(I-1)
          DW =  AA/COSZEN
          DW =  DMIN1(100.0_dp,DW)
          DIR(I)  =  COSZEN*DEXP(-DW)
       enddo

       ! IF (A .GT. 25.0_dp)  GOTO 200
       IF (A .GT. 25.0_dp) THEN
           NLA = I-1
           NLY     = NLA-1
           ALDIF   = 0.
           ALDR    = 0.
        ENDIF
       IF (I .EQ. NLEV)  GOTO 300

       ! 200 NLA = I-1
       ! NLY     = NLA-1
       ! ALDIF   = 0.
       ! ALDR    = 0.
       300 CONTINUE

       IF (NLY.GT.0) THEN
!......................................................................
!     subroutine MATBAU - calculation of diffuse irradiation
!     for each spectral interval
!     input  - NLY,ALDIF,ALDR,COSZEN,BET,BET0,OM,TAU,DIR
!     output - DIF
          CALL MATBAU(NLY,ALDIF,ALDR,COSZEN,BET,BET0,OM,TAU,DIR,DIF)
!......................................................................
       ENDIF
!......................................................................

!   upward (UP), downward diffuse (DIF)
!   and downward direct (DIR) fluxes
!   have been computed for unity input fluxes for each
!   layer boundary for the wave-length interval IALL.
!   The true fluxes are obtained by multiplication with the
!   spectral solar constant (input from <LESEN>)  SOL .
!.......................................................................

!   for each boundary NLA spectral integrated quantities

!     DIF  : downward diffuse flux
!     DIR  : downward direct flux
!
!   are computed
!.......................................................................
       IF (isnan(DIF(1))) THEN
         DIF(1)=10E-20
       ENDIF
       DO I = 1,NLA
          DIF(I)   = DIF(I)*SOL(IALL)
          DIR(I)   = DIR(I)*SOL(IALL)
          DIRS(I)  = DIRS(I)+DIR(I)
          DIFS(I)  = DIFS(I)+DIF(I)
       enddo

       XDIR=DLOG(DIR(NLEV-LAYER)/DIR(NLEV-LAYER-1))

       if (PRESENT(ip)) then
         if (present(scaling)) then
           ! spectral_dir_surf(IDO,ip)=spectral_dir_surf(IDO,ip)+DIR(NLEV-LAYER)
           spectral_dir_surf(IDO,ip)=spectral_dir_surf(IDO,ip)+DIR(NLEV-LAYER)*DEXP(-layerfrac*XDIR)
           ! spectral_dir_surf(IDO,ip)=spectral_dir_surf(IDO,ip)+(1-layerfrac)*DIR(NLEV-LAYER)+layerfrac*DIR(NLEV-LAYER-1)
         else
           ! spectral_dir_scaled_surf(IDO,ip)=spectral_dir_scaled_surf(IDO,ip)+DIR(NLEV-LAYER)
           spectral_dir_scaled_surf(IDO,ip)=spectral_dir_scaled_surf(IDO,ip)+DIR(NLEV-LAYER)*DEXP(-layerfrac*XDIR)
           ! spectral_dir_scaled_surf(IDO,ip)=spectral_dir_scaled_surf(IDO,ip)+(1-layerfrac)*DIR(NLEV-LAYER)+layerfrac*DIR(NLEV-LAYER-1)
           ! spectral_dif_surf(IDO,ip)=spectral_dif_surf(IDO,ip)+DIF(NLEV-LAYER)
           spectral_dif_surf(IDO,ip)=spectral_dif_surf(IDO,ip)+(1-layerfrac)*DIF(NLEV-LAYER)+layerfrac*DIF(NLEV-LAYER-1)
         endif
         spectral_dir_top(IDO,ip)=spectral_dir_top(IDO,ip)+DIR(1)
         spectral_dif_top(IDO,ip)=spectral_dif_top(IDO,ip)+DIF(1)
       end if

    40 CONTINUE    !end of loop - 40 -  spectral intervals

    DO I = 1 , NLEV
       DOWN(I) = DIRS(I) + DIFS(I)
    enddo

   100 CONTINUE

!     calculation of atmospheric transmittance
 TRANS=DOWN(NLEV)/DOWN(1)
 TDIR=DIRS(NLEV)/DOWN(1)

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: LESEN
!
!> Determination of atmospheric profiles.
!!
!! @param[in]  LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                            1. tropical <br>
!!                            2. midlatitude summer <br>
!!                            3. midlatitude winter <br>
!!                            4. subartic summer <br>
!!                            5. subartic winter
!! @param[in]  Kext  DOUBLE, DIMENSION(5) Extinction coeficient from 0 to 5 km.
!! @param[out] SOL    DOUBLE  Solar constant for each spectral interval. <br>
!!                    1D ARRAY with dimension MAXIND, a parameter specified in
!!                    STRPSRB.
!! @param[out] DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                    1D ARRAY with dimension LAYMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] PRESS  DOUBLE  Atmospheric pressure [mbar]. <br>
!!                    1D ARRAY with dimension 31 specified in STRPSRB.
!! @param[out] ALT    DOUBLE  Altitude [m]. <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] TEMPE   DOUBLE  Temperature [C]. <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] ROL    DOUBLE  Air density. <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] ROH    DOUBLE  Water content (humidity). <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] RO3    DOUBLE  Ozone content. <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] SCA    DOUBLE  ? <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!! @param[out] RAY    DOUBLE  ? <br>
!!                    1D ARRAY with dimension LEVMAX, a parameter specified in
!!                    STRPSRB.
!!
!! @warning    Multiple variables with fixed sizes defined in STRPSRB.
!!
!! @see        STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE LESEN(LATMOS,Kext,ZSTAT,E0,SOL,DZ,PRESS,ALT,TEMPE,ROL,ROH,RO3,SCA,RAY)
      ! use gradevars
       use Brasilsr_constants
       use Brasilsr_spectral_grid, only: MAXIND, WEIGHT,SPECTRAL_INDEX
!  determination of atmospheric profiles

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     LATMOS  : atmosphere type
!     1 - tropical
!     2 - midlatitude summer
!     3 - midlatitude winter
!     4 - subartic summer
!     5 - subartic winter
!     Kext(5) : Extinction coeficient from 0 to 5 km.
!     SOL     : solar constant for each spectral interval
!     DZ : thickness of each atmospheric layer
!     ROL     : air density
!     ROH     : water content (humidity)
!     RO3     : ozone content
!     PRESS   : atmospheric pressure (mbar)
!     ALT     : altitude (m)
!     TEMPE    : temperature (oC)
!     SCA     :
!     RAY     :
!     RAYL    :
!     SCAL    :
!     ISOL    :
!     WEIGHT  :
!     FITTOP  : top of aerosol profile (km)
!     FITBOT  : bottom of aerosol profile (km)
!     JTOP    :
!     JBOT    :
!     H  : altitude of aerosol profile (km)
!     DLAY    : delta of atmospheric layer (km)
!     SCATOP :
!     SCABOT :
!     CSCAL   :
!     ZJ : altitude (km)
!     M1 :
!     M2 :
!     CH :
!     CRAY    :
!     CS!     :
!     X  :
!......................................................................

 ! PARAMETER (LEVMAX = 51, LAYMAX = 50, MAXWEL= 37, MAXIND= 135)
 ! IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 IMPLICIT NONE
 INTEGER, INTENT(IN):: LATMOS
 DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: Kext
 DOUBLE PRECISION, INTENT(IN) :: ZSTAT,E0
 DOUBLE PRECISION, DIMENSION(MAXIND), INTENT(OUT) :: SOL
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT) :: DZ
 DOUBLE PRECISION, DIMENSION(NLEV), INTENT(OUT)   :: PRESS
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(OUT) :: ALT, TEMPE, ROL, ROH, RO3, SCA,RAY

 ! DIMENSION RAYL(LEVMAX),SCAL(LEVMAX)
 ! DIMENSION RAY(LEVMAX),SCA(LEVMAX)
 ! DIMENSION TEMPE(LEVMAX),PRESS(NLEV),ALT(LEVMAX)
 ! DIMENSION ROL(LEVMAX),ROH(LEVMAX),RO3(LEVMAX)
 ! DIMENSION DZ(LAYMAX),TPRESS(NLEV)
 ! DIMENSION SOL(MAXIND)
 DOUBLE PRECISION, DIMENSION(NLEV,5)::TPRESS
 DOUBLE PRECISION, DIMENSION(MAXWEL)::ISOL
 DOUBLE PRECISION, DIMENSION(LEVMAX)::RAYL,SCAL
 DOUBLE PRECISION :: FITTOP, FITBOT, CH, CRAY, CSC,X

 INTEGER ::ZJ, DLAY, JTOP, JBOT, I, J, M1, M2, ZEROLAYR

 TPRESS(:,1) = (/ 0.854_dp,  1.59_dp,  3.05_dp,  6.0_dp ,  12.2_dp,  25.7_dp, &
          &  30.0_dp,  35.0_dp,  40.9_dp,  48.0_dp,  56.5_dp,  66.6_dp, &
          &  78.9_dp,  93.7_dp, 111.0_dp, 132.0_dp, 156.0_dp, 182.0_dp, &
          & 213.0_dp, 247.0_dp, 286.0_dp, 329.0_dp, 378.0_dp, 432.0_dp, &
          & 492.0_dp, 559.0_dp, 633.0_dp, 715.0_dp, 805.0_dp, 904.0_dp, &
          & 1025.0_dp/)

 TPRESS(:,2) = (/0.951_dp, 1.760_dp, 3.330_dp, 6.520_dp, 13.200_dp, 27.700_dp, &
          & 32.200_dp, 37.600_dp, 43.700_dp, 51.000_dp, 59.500_dp, 69.500_dp, &
          & 81.200_dp, 95.000_dp, 111.000_dp, 130.000_dp, 153.000_dp, 179.000_dp, &
          & 209.000_dp, 243.000_dp, 281.000_dp, 324.000_dp, 372.000_dp, 426.000_dp, &
          & 487.000_dp, 554.000_dp, 628.000_dp, 710.000_dp, 802.000_dp, 902.000_dp, &
          & 1013.000_dp/)

TPRESS(:,3) = (/0.682_dp, 1.290_dp, 2.530_dp, 5.180_dp, 11.100_dp, 24.300_dp, &
          & 28.600_dp, 33.400_dp, 39.100_dp, 45.800_dp, 53.700_dp, 62.800_dp, &
          & 73.500_dp, 86.100_dp, 100.700_dp, 117.800_dp, 137.800_dp, 161.000_dp, &
          & 188.200_dp, 219.900_dp, 256.800_dp, 299.200_dp, 347.300_dp, 401.600_dp, &
          & 462.700_dp, 531.300_dp, 608.100_dp, 693.800_dp, 789.700_dp, 897.300_dp, &
          & 1018.000_dp/)

TPRESS(:,4) = (/0.987_dp, 1.810_dp, 3.400_dp, 6.610_dp, 13.400_dp, 27.800_dp, &
          & 32.270_dp, 37.500_dp, 43.600_dp, 50.700_dp, 58.900_dp, 68.600_dp, &
          & 79.800_dp, 92.800_dp, 108.000_dp, 125.000_dp, 146.000_dp, 170.000_dp, &
          & 197.700_dp, 230.000_dp, 267.700_dp, 310.700_dp, 359.000_dp, 413.000_dp, &
          & 473.000_dp, 541.000_dp, 616.000_dp, 700.000_dp, 792.900_dp, 896.000_dp, &
          & 1010.000_dp/)

TPRESS(:,5) = (/0.572_dp, 1.113_dp, 2.243_dp, 4.701_dp, 10.200_dp, 22.560_dp, &
          & 26.490_dp, 31.090_dp, 36.470_dp, 42.770_dp, 50.140_dp, 58.750_dp, &
          & 68.820_dp, 80.580_dp, 94.310_dp, 110.300_dp, 129.100_dp, 151.000_dp, &
          & 176.600_dp, 206.700_dp, 241.800_dp, 282.900_dp, 330.800_dp, 385.300_dp, &
          & 446.700_dp, 515.800_dp, 593.200_dp, 679.800_dp, 777.500_dp, 887.800_dp,&
          & 1013.000_dp/)

 RAYL = (/ 2.1352E+1,2.4178E+1,2.7378E+1,&
     & 3.1117E+1,3.5642E+1,4.0885E+1,4.6968E+1,5.4039E+1,6.2270E+1,&
     & 7.1869E+1,8.3082E+1,9.6204E+1,1.1159E+2,1.2965E+2,1.5091E+2,&
     & 1.7598E+2,2.0559E+2,2.4064E+2,2.8185E+2,3.2835E+2,3.8280E+2,&
     & 4.4660E+2,5.2141E+2,6.0920E+2,7.1230E+2,8.3346E+2,9.7598E+2,&
     & 1.1437E+3,1.3414E+3,1.5743E+3,1.8487E+3,2.1624E+3,2.5294E+3,&
     & 2.9589E+3,3.4614E+3,4.0495E+3,4.7378E+3,5.5433E+3,6.4861E+3,&
     & 7.5853E+3,8.5981E+3,9.7116E+3,1.0933E+4,1.2268E+4,1.3726E+4,&
     & 1.5313E+4,1.7037E+4,1.8906E+4,2.0929E+4,2.3115E+4,2.5471E+4/)

 SCAL = (/2.10E-6, 2.39E-6,&
     & 2.72E-6, 3.10E-6, 3.53E-6, 4.02E-6, 4.61E-6, 5.29E-6, 6.07E-6,&
     & 6.97E-6, 8.00E-6, 9.25E-6, 1.07E-5, 1.24E-5, 1.43E-5, 1.65E-5,&
     & 1.90E-5, 2.18E-5, 2.51E-5, 2.89E-5, 3.32E-5, 4.49E-5, 6.07E-5,&
     & 8.21E-5, 1.11E-4, 1.50E-4, 1.90E-4, 2.42E-4, 2.96E-4, 3.52E-4,&
     & 4.23E-4, 4.92E-4, 5.63E-4, 6.01E-4, 6.41E-4, 6.43E-4, 6.45E-4,&
     & 6.22E-4, 6.63E-4, 7.14E-4, 7.87E-4, 9.80E-4, 1.41E-3, 2.30E-3,&
     & 3.54E-3, 4.85E-3, 6.43E-3, 8.19E-3, 9.70E-3, 2.58E-2, 6.95E-2/)

!Original from model
 ! ISOL = (/0.4_dp ,   16.0_dp,   37.0_dp,    55.0_dp,   54.0_dp,   60.0_dp,    86.0_dp,&
 !     &   160.0_dp,  177.0_dp,  273.0_dp,   757.5_dp, 1476.0_dp, 1845.4_dp,  2107.7_dp,&
 !     &  2102.5_dp, 5425.0_dp, 9476.2_dp,  9601.2_dp, 9320.5_dp, 8836.5_dp,  7960.5_dp,&
 !     &  5066.2_dp, 6551.2_dp, 1220.4_dp,  1791.7_dp, 8769.6_dp, 1958.5_dp, 10184.4_dp,&
 !     &  5094.8_dp, 7307.3_dp, 2996.9_dp,10003.79_dp, 2591.6_dp, 6498.2_dp,   796.5_dp,&
 !     &  3813.85_dp, 1128.3_dp /)

 !Gueymard 2004
 ISOL = (/ 4.6815_dp,   25.2700_dp,   47.6260_dp,   48.3990_dp,   55.1720_dp,   52.9500_dp,  128.9350_dp, &
      &  240.1900_dp,  216.3487_dp,  446.3200_dp,  838.8100_dp, 1569.2050_dp, 1911.4250_dp, 2111.0900_dp, &
      & 2171.5150_dp, 5565.6400_dp, 9794.3000_dp, 9767.8000_dp, 9340.4000_dp, 8707.1000_dp, 7793.3000_dp,&
      & 5043.0000_dp, 6574.9000_dp, 1225.0000_dp, 1779.9000_dp, 8700.9000_dp, 1903.5300_dp, 9728.7800_dp,&
      & 4936.4800_dp, 7022.1400_dp, 3030.9400_dp, 10003.3600_dp, 2241.9700_dp, 6396.2100_dp, 800.2450_dp,&
      & 3728.5900_dp, 1102.0400_dp/)


 DO 10 I=1,MAXIND
   10 SOL(I)=ISOL(SPECTRAL_INDEX(I))*0.01_dp*WEIGHT(I)*E0

!.....................................................................
! calculate aerosol-profiles for a given surface visibility
! Load Aerosol-Profile (Mc Clatchey with surface visibility=50km)
! fit new aerosol profile between fittop(5km) and fitbot(0km) for
! a surface visibility VIS = 5 - 50km taken from the input data
!.....................................................................

 FITTOP  = 5._dp
 FITBOT  = 0._dp
 JTOP    = 51 - INT(FITTOP)+1
 JBOT    = 51 - INT(FITBOT)
 ! H  = FITTOP - FITBOT
 DLAY    = 1
 ! SCATOP = SCAL(JTOP)
 ! SCABOT = 3.912/VIS
 ! CSCAL   = DLOG(SCATOP/SCABOT)/H
 !
 ZJ = 0
 DO 20 J = JTOP,JBOT
   ZJ = ZJ + DLAY
   ! WRITE(*,*)J,ZJ
   SCAL(J) = Kext(ZJ)
   20 CONTINUE
! WRITE(*,*) 'SCAL',SCAL
! WRITE(*,*) 'Kext',Kext
! STOP
! ....................................................................
!     *** model atmosphere ***
 do 25 I=1,NLEV
       PRESS(I) = TPRESS(I,LATMOS)
 25 continue

!......................................................................

!     subroutine PTZQD - calculation of atmospheric properties for each
!    level
!     input  - LATMOS,PRESS
!     output - PRESS,ALT,TEMPE,ROL,ROH,RO3

 CALL PTZQD(LATMOS,PRESS,ALT,TEMPE,ROL,ROH,RO3)

!......................................................................

 DO 30  I = 1 , NLEV
    ROL(I) = ROL(I)*1000._dp
    ROH(I) = ROH(I)*1000._dp
    RO3(I) = RO3(I)*1000._dp
    PRESS(I) = PRESS(I)*0.001_dp
   30 CONTINUE

 DO 40 I=1,NLAY
 M1 = 50 - INT(MIN(ALT(I),50.0_dp)-0.001_dp)
 M2 = M1 + 1
 CH = ALT(I) - AINT(MIN(ALT(I),50.0_dp)-0.0001_dp)
 CRAY= DLOG (RAYL(M1)/RAYL(M2))
 CSC = DLOG (SCAL(M1)/SCAL(M2))
 X = CH - 1.
 RAY(I) = RAYL(M1) * DEXP(X*CRAY)
 SCA(I) = SCAL(M1) * DEXP(X*CSC)
   40 CONTINUE

 SCA(NLEV) = SCAL(51)
 RAY(NLEV) = RAYL(51)

 !DO I=1,NLAYZEROLAYR
 !     DZ(I)=0.
 !ENDDO
 !
 DO I=1,NLAY
      DZ(I)=DABS(ALT(I)-ALT(I+1))
 ENDDO
 ! ZSTAT2=ZSTAT+1000._DP
 !
 ! ZEROLAYR=int(ZSTAT2/1000._DP)
 ! print*,ZEROLAYR,ZSTAT2/1000._DP
 ! DO i=1,ZEROLAYR
 !   DZ(NLAY-i+1)=0._DP
 ! ENDDO
 ! DZ(NLAY-ZEROLAYR)=DZ(NLAY-ZEROLAYR)-(ZSTAT2-ZEROLAYR*1000)/1000._DP
 !
 ! print*, DZ

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: PTZQD
!
!> Calculation of atmospheric properties for each level.
!!
!! @param[in]     LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                               1. tropical <br>
!!                               2. midlatitude summer <br>
!!                               3. midlatitude winter <br>
!!                               4. subartic summer <br>
!!                               5. subartic winter
!! @param[in,out] PRESS  DOUBLE  Atmospheric pressure [mbar]. <br>
!!                       1D ARRAY with dimension 31 specified in STRPSRB.
!! @param[out]    ALT    DOUBLE  Altitude [m]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    TEMPE   DOUBLE  Temperature [C]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    ROL    DOUBLE  Air density. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    ROH    DOUBLE  Water content (humidity). <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    RO3    DOUBLE  Ozone content. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!!
!! @warning       Multiple variables with fixed sizes defined in STRPSRB.
!!
!! @see        STRPSRB LESEN
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE PTZQD(LATMOS,PRESS,ALT,TEMPE,ROL,ROH,RO3)
      ! use gradevars
       use Brasilsr_constants
!     calculation of atmospheric properties for each level

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     LATMOS  : atmosphere type
!     1 - tropical
!     2 - midlatitude summer
!     3 - midlatitude winter
!     4 - subartic summer
!     5 - subartic winter
!     ROL     : air density
!     ROH     : water content (humidity)
!     RO3     : ozone content
!     PRESS   : atmospheric pressure (mbar)
!     ALT     : altitude (m)
!     TEMPE    : temperature (oC)
!     NLEVD2  :
!     UMSP    :
!     QL :
!     RM :
!     PTZQ1   :
!     PTZQ2   :
!     PTZQ3   :
!     DLOGP   :
!     DELZAP  :
!     NINT    :
!     ZNINT   :
!     G0 :
!     AA :
!     G  :
!     ZMASS   :
!     R  :
!     DZZ :
!     HT :
!     RKn     :
!......................................................................
!
 ! PARAMETER(LEVMAX=51)
 IMPLICIT DOUBLE PRECISION (A-H,O-Z)

 INTEGER, INTENT(IN) :: LATMOS
 DOUBLE PRECISION, DIMENSION(NLEV), INTENT(INOUT) :: PRESS
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(OUT) :: ALT,TEMPE,ROL,ROH,RO3

 DOUBLE PRECISION :: RM,DELZAP,R,G0,ZMASS,AA,UMSP,QL,DLOGP,ZNINT,G,DZZ,HT,RK1,RK2,RK3,RK4
 INTEGER :: NLEVD2,I,J,M,NINT
 DOUBLE PRECISION :: Rd

 ! DIMENSION PRESS(31),ALT(LEVMAX),TEMPE(LEVMAX),ROL(LEVMAX)
 ! DIMENSION ROH(LEVMAX),RO3(LEVMAX)
!
!DATA RM,DELZAP,R,G0,ZMASS,AA /348.36764,0.5,8.314,9.81,28.9,6400./
! RM     = 348.36764_dp
Rd     = 287.0_dp   !Ideal gas constant Rd
DELZAP = 0.5_dp
R      = 8.314_dp
G0     = 9.81_dp
ZMASS  = 28.9_dp
AA     = 6400.0_dp
!

 NLEVD2=INT(NLEV*0.5_dp)
 DO I=1,NLEVD2
    J=NLEV-I+1
    UMSP=PRESS(J)
    PRESS(J)=PRESS(I)
    PRESS(I)=UMSP
 enddo

 ALT(1)=0._dp
 TEMPE(1)=PTZQ1(ALT(1),LATMOS)
 ! QL: Air density
 QL=100000.0_dp*PRESS(1)/(Rd*TEMPE(1))
 ! QL=PRESS(1)*RM/TEMPE(1)
 ROH(1)=PTZQ2(PRESS(1),LATMOS,QL)
 ROL(1)=QL/(1.+0.608_dp*ROH(1)/QL)
 RO3(1)=PTZQ3(PRESS(1),LATMOS)

 DO N=1,NLEV-1
    DLOGP=7.*DLOG(PRESS(N)/PRESS(N+1))
    NINT=INT(DLOGP/DELZAP)+1
    ZNINT=DBLE(NINT)
    G=G0*(1.0_dp-2.0_dp*ALT(N)/AA)
    DZZ=R*DLOGP/(7._dp*ZMASS*G*ZNINT)
    HT=ALT(N)
    DO M=1,NINT
       RK1=PTZQ1(HT,LATMOS)*DZZ
       RK2=PTZQ1(HT+0.5_dp*RK1,LATMOS)*DZZ
       RK3=PTZQ1(HT+0.5_dp*RK2,LATMOS)*DZZ
       RK4=PTZQ1(HT+RK3,LATMOS)*DZZ
       HT=HT+0.16666667_dp*(RK1+RK2+RK2+RK3+RK3+RK4)
    enddo
    ALT(N+1)=HT
    TEMPE(N+1)=PTZQ1(HT,LATMOS)
    QL=100000.0_dp*PRESS(N+1)/(TEMPE(N+1)*Rd)
    ! QL=PRESS(N+1)*RM/TEMPE(N+1)

    ROH(N+1)=PTZQ2(PRESS(N+1),LATMOS,QL)
    ROL(N+1)=QL/(1.+0.608_dp*ROH(N+1.0_dp)/QL)
    RO3(N+1)=PTZQ3(PRESS(N+1),LATMOS)
 enddo

 DO N=1,NLEV
    ROH(N)=ROH(N)*0.001_dp
    ROL(N)=ROL(N)*0.001_dp
 enddo

 DO I=1,NLEVD2
    J=NLEV-I+1
    UMSP=PRESS(J)
    PRESS(J)=PRESS(I)
    PRESS(I)=UMSP
    UMSP=ALT(J)
    ALT(J)=ALT(I)
    ALT(I)=UMSP
    UMSP=TEMPE(J)
    TEMPE(J)=TEMPE(I)
    TEMPE(I)=UMSP
    UMSP=ROL(J)
    ROL(J)=ROL(I)
    ROL(I)=UMSP
    UMSP=ROH(J)
    ROH(J)=ROH(I)
    ROH(I)=UMSP
    UMSP=RO3(J)
    RO3(J)=RO3(I)
    RO3(I)=UMSP
 enddo
 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: WOLKE1
!
!> Determination of cloud properties.
!!
!! @param[in,out] NCL    INTEGER Number of cloud layers.
!! @param[in,out] IGO    INTEGER Flag for CLOLWC greater than limits.
!! @param[in]     TOP    DOUBLE  Cloud top height [mbar].
!! @param[in,out] TAUW   DOUBLE  Cloud optical thickness. [0-100] <br>
!!                               0.0:   no clouds <br>
!!                               100.0: completely cloudy
!! @param[in]     IWP    INTEGER Cloud droplet size distribution.
!! @param[in]     LCLOUD LOGICAL Calculate with or without clouds.
!! @param[in]     CLOLWC DOUBLE  Cloud liquid water content.
!! @param[in]     DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     TEMPE   DOUBLE  Temperature [C]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     PRESS  DOUBLE  Atmospheric pressure [mbar]. <br>
!!                       1D ARRAY with dimension 31 specified in STRPSRB.
!! @param[out]    TW     DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    K1     INTEGER ?
!!
!! @warning    Variable NCL is implicitly defined as a constant in D2STR and
!!             then changed in this subroutine.
!!
!! @see        D2STR STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE WOLKE1(NCL,IGO,TOP,TAUW,IWP,LCLOUD,CLOLWC,DZ,TEMPE,PRESS,TW,K1)
 use Brasilsr_constants
!     determination of clouds properties, input is CLOLWC
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     IWP     : cloud droplet size distribution
!     TAUW    : cloud optical thickness
!    0.0 - no clouds
!    100.0 - with clouds
!     TOP     : cloud top (mbar)
!     NCL     : number of cloud layers
!     CLOLWC  : cloud liquid water content
!     DZ : thickness of each atmospheric layer
!     LCLOUD  : logical variable to calculate with or without clouds
!     PRESS   : atmospheric pressure (mbar)
!     TEMPE    : temperature (oC)
!     IGO     : flag for CLOLWC greater than limits
!     K1 :
!     ALW     :
!     FLW     :
!     TW :
!     PWZW    :
!     ICLT    :
!     ICLB    :
!     TTOP    :
!     TBOT    :
!     EXPON   :
!     ESAT    :
!     ROBOT   :
!     ROTOP   :
!     CLIM    :
!     EXMASS  :
!     EWOL    :
!     FIXLWC  :
!     IWO     :
!     KWO     :
!......................................................................

 IMPLICIT NONE

 LOGICAL*1 , INTENT(IN) :: LCLOUD
 INTEGER, INTENT(IN) :: IWP
 INTEGER, INTENT(INOUT) :: NCL,IGO
 DOUBLE PRECISION, INTENT(IN) :: CLOLWC
 DOUBLE PRECISION, INTENT(INOUT) :: TAUW
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(IN) :: DZ
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(IN) :: TEMPE
 DOUBLE PRECISION, DIMENSION(NLEV), INTENT(IN) :: PRESS
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT) :: TW
 INTEGER, INTENT(OUT) :: K1
 !
 DOUBLE PRECISION , DIMENSION(16,8) :: EWOL
 DOUBLE PRECISION , DIMENSION(LAYMAX) :: FLW
 DOUBLE PRECISION , DIMENSION(8)::FIXLWC
 INTEGER :: I,ICLT,ICLB,KWO,IWO
 DOUBLE PRECISION ::  PWZW,EXMASS,ZTOTAL,ALW,TOP,TTOP,TBOT,EXPON,ESAT,ROTOP,ROBOT,CLIM
!
FIXLWC = (/0.05_dp, 0.14_dp, 0.28_dp, 0.47_dp, 1.00_dp, 2.50_dp, 0.13_dp, 0.44_dp/)

EWOL(:,1)  = (/15.2_dp,  15.4_dp,  15.7_dp,  15.3_dp,  15.6_dp,  15.8_dp,  16.0_dp,  15.8_dp, &
               16.0_dp,  16.0_dp,  15.9_dp,  16.5_dp,  17.1_dp,  17.2_dp,  21.2_dp,  18.1_dp/)
EWOL(:,2)  = (/39.4_dp,  40.1_dp,  41.0_dp,  40.9_dp,  42.3_dp,  41.9_dp,  41.6_dp,  41.2_dp, &
               42.1_dp,  42.9_dp,  43.5_dp,  43.2_dp,  44.1_dp,  45.8_dp,  50.7_dp,  47.1_dp/)
EWOL(:,3)  = (/72.4_dp,  72.7_dp,  74.1_dp,  75.5_dp,  73.8_dp,  73.9_dp,  74.6_dp,  74.6_dp, &
               75.0_dp,  75.2_dp,  77.5_dp,  78.4_dp,  78.8_dp,  80.0_dp,  85.6_dp,  82.4_dp/)
EWOL(:,4)  = (/73.9_dp,  76.6_dp,  74.4_dp,  75.1_dp,  75.5_dp,  75.4_dp,  74.8_dp,  76.0_dp, &
               76.1_dp,  76.2_dp,  77.2_dp,  77.4_dp,  77.7_dp,  80.3_dp,  80.8_dp,  81.2_dp/)
EWOL(:,5)  = (/127.8_dp, 131.1_dp, 130.3_dp, 129.1_dp, 130.9_dp, 129.8_dp, 130.9_dp, 130.4_dp, &
               130.6_dp, 130.7_dp, 134.9_dp, 132.8_dp, 136.4_dp, 137.4_dp, 138.6_dp, 140.2_dp/)
EWOL(:,6) = (/121.1_dp, 122.1_dp, 121.2_dp, 122.3_dp, 121.0_dp, 122.3_dp, 121.0_dp, 120.4_dp, &
              121.4_dp, 122.3_dp, 124.1_dp, 123.3_dp, 125.8_dp, 126.4_dp, 125.2_dp, 127.1_dp/)
EWOL(:,7) = (/5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp, &
              5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.7_dp,   5.7_dp/)
EWOL(:,8) = (/12.3_dp,  12.3_dp,  12.3_dp,  12.3_dp,  12.4_dp,  12.4_dp,  12.4_dp,  12.4_dp, &
              12.4_dp,  12.4_dp,  12.4_dp,  12.5_dp,  12.5_dp,  12.6_dp,  12.6_dp,  12.7_dp/)

 TAUW    =  0._dp
 ALW     =  0._dp
 DO 10  I = 1 , NLAY
   FLW(I) = 0._dp
   TW(I)  = 0._dp
   10 CONTINUE
 K1 = 0

 IF (.NOT. LCLOUD)       GOTO 100

 DO 20  I = 1 , NLAY
   PWZW =  PRESS(I)*1000._dp + 1.
   IF (TOP.LE.PWZW)      GOTO 200
   20 CONTINUE
  200 K1 = I-1

 ICLT = K1 + 1
 ICLB = ICLT + NCL
 IF(ICLB.GT.NLAY+1) THEN
   WRITE(*,*) ' cloud bottom has to be adjusted'
   ICLB = NLAY + 1
   NCL = ICLB - ICLT
 ENDIF

 TTOP = TEMPE(ICLT)
 TBOT = TEMPE(ICLB)
 EXPON = (7.5_dp*(TTOP-273.15_dp))/(TTOP-35.85_dp)
 ESAT = 6.11_dp*10._dp**EXPON
 ROTOP = 216.67_dp*ESAT/TTOP
 EXPON = (7.5_dp*(TBOT-273.15_dp))/(TBOT-35.85_dp)
 ESAT  = 6.11_dp*10._dp**EXPON
 ROBOT = 216.67_dp*ESAT/TBOT
 CLIM  = (ROBOT-ROTOP)*0.2_dp
 IF(CLIM.LT.CLOLWC) THEN
   IGO = 0
   RETURN
 ENDIF

 EXMASS  = EWOL(1,IWP)/FIXLWC(IWP)/1000._dp
 DO 30  IWO = 1 , NCL
   KWO = K1 + IWO
   FLW(KWO) = CLOLWC*DZ(KWO)*1000._dp
   TW(KWO)  = FLW(KWO)*EXMASS
   TAUW     = TAUW+TW(KWO)
   ALW = ALW+FLW(KWO)
   30 CONTINUE

  100 CONTINUE

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: WOLKE2
!
!> Determination of cloud properties.
!!
!! @param[in,out] NCL    INTEGER Number of cloud layers.
!! @param[in]     TOP    DOUBLE  Cloud top height [mbar].
!! @param[in]     TAUW   DOUBLE  Cloud optical thickness. [0-100] <br>
!!                               0.0:   no clouds <br>
!!                               100.0: completely cloudy
!! @param[in]     IWP    INTEGER Cloud droplet size distribution.
!! @param[in]     LCLOUD LOGICAL Calculate with or without clouds.
!! @param[out]    CLOLWC DOUBLE  Cloud liquid water content.
!! @param[in]     DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     TEMPE   DOUBLE  Temperature [C]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     PRESS  DOUBLE  Atmospheric pressure [mbar]. <br>
!!                       1D ARRAY with dimension 31 specified in STRPSRB.
!! @param[out]    TW     DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    K1     INTEGER ?
!!
!! @warning    Variable NCL is implicitly defined as a constant in D2STR and
!!             then changed in this subroutine.
!!
!! @see        D2STR STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE WOLKE2(NCL,TOP,TAUW,IWP,LCLOUD,CLOLWC,DZ,TEMPE,PRESS,TW,K1)
      ! use gradevars
       use Brasilsr_constants
!     determination of clouds properties, input is CLOLWC
!
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     IWP     : cloud droplet size distribution
!     TAUW    : cloud optical thickness
!    0.0 - no clouds
!    100.0 - with clouds
!     TOP     : cloud top (mbar)
!     NCL     : number of cloud layers
!     CLOLWC  : cloud liquid water content
!     DZ : thickness of each atmospheric layer
!     LCLOUD  : logical variable to calculate with or without clouds
!     PRESS   : atmospheric pressure (mbar)
!     TEMPE    : temperature (oC)
!     TW :
!     K1 :
!     ALW     :
!     FLW     :
!     PWZW    :
!     ICLT    :
!     ICLB    :
!     TTOP    :
!     TBOT    :
!     EXPON   :
!     ESAT    :
!     ROBOT   :
!     ROTOP   :
!     CLIM    :
!     EXMASS  :
!     EWOL    :
!     FIXLWC  :
!     IWO     :
!     KWO     :
!     ZTOTAL  :
!......................................................................

 IMPLICIT NONE

 DOUBLE PRECISION , DIMENSION(LEVMAX),INTENT(IN) :: TEMPE
 DOUBLE PRECISION , DIMENSION(LAYMAX),INTENT(IN)  :: DZ
 DOUBLE PRECISION , DIMENSION(NLEV),INTENT(IN) :: PRESS
 LOGICAL*1,INTENT(IN):: LCLOUD
 INTEGER,INTENT(IN):: IWP
 INTEGER,INTENT(INOUT):: NCL
 DOUBLE PRECISION,INTENT(IN) :: TOP, TAUW
 DOUBLE PRECISION , DIMENSION(LAYMAX),INTENT(OUT) :: TW
 DOUBLE PRECISION , INTENT(OUT) :: CLOLWC
 INTEGER,INTENT(OUT):: K1

 DOUBLE PRECISION , DIMENSION(16,8) :: EWOL
 DOUBLE PRECISION , DIMENSION(LAYMAX) :: FLW
 DOUBLE PRECISION , DIMENSION(8)::FIXLWC
 INTEGER :: I,ICLT,ICLB,KWO,IWO
 DOUBLE PRECISION ::  PWZW,EXMASS,ZTOTAL,ALW,TTOP,TBOT,EXPON,ESAT,ROTOP,ROBOT,CLIM

 FIXLWC = (/0.05_dp, 0.14_dp, 0.28_dp, 0.47_dp, 1.00_dp, 2.50_dp, 0.13_dp, 0.44_dp/)

 EWOL(:,1)  = (/15.2_dp,  15.4_dp,  15.7_dp,  15.3_dp,  15.6_dp,  15.8_dp,  16.0_dp,  15.8_dp, &
                16.0_dp,  16.0_dp,  15.9_dp,  16.5_dp,  17.1_dp,  17.2_dp,  21.2_dp,  18.1_dp/)
 EWOL(:,2)  = (/39.4_dp,  40.1_dp,  41.0_dp,  40.9_dp,  42.3_dp,  41.9_dp,  41.6_dp,  41.2_dp, &
                42.1_dp,  42.9_dp,  43.5_dp,  43.2_dp,  44.1_dp,  45.8_dp,  50.7_dp,  47.1_dp/)
 EWOL(:,3)  = (/72.4_dp,  72.7_dp,  74.1_dp,  75.5_dp,  73.8_dp,  73.9_dp,  74.6_dp,  74.6_dp, &
                75.0_dp,  75.2_dp,  77.5_dp,  78.4_dp,  78.8_dp,  80.0_dp,  85.6_dp,  82.4_dp/)
 EWOL(:,4)  = (/73.9_dp,  76.6_dp,  74.4_dp,  75.1_dp,  75.5_dp,  75.4_dp,  74.8_dp,  76.0_dp, &
                76.1_dp,  76.2_dp,  77.2_dp,  77.4_dp,  77.7_dp,  80.3_dp,  80.8_dp,  81.2_dp/)
 EWOL(:,5)  = (/127.8_dp, 131.1_dp, 130.3_dp, 129.1_dp, 130.9_dp, 129.8_dp, 130.9_dp, 130.4_dp, &
                130.6_dp, 130.7_dp, 134.9_dp, 132.8_dp, 136.4_dp, 137.4_dp, 138.6_dp, 140.2_dp/)
 EWOL(:,6) = (/121.1_dp, 122.1_dp, 121.2_dp, 122.3_dp, 121.0_dp, 122.3_dp, 121.0_dp, 120.4_dp, &
               121.4_dp, 122.3_dp, 124.1_dp, 123.3_dp, 125.8_dp, 126.4_dp, 125.2_dp, 127.1_dp/)
 EWOL(:,7) = (/5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp, &
               5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.7_dp,   5.7_dp/)
 EWOL(:,8) = (/12.3_dp,  12.3_dp,  12.3_dp,  12.3_dp,  12.4_dp,  12.4_dp,  12.4_dp,  12.4_dp, &
               12.4_dp,  12.4_dp,  12.4_dp,  12.5_dp,  12.5_dp,  12.6_dp,  12.6_dp,  12.7_dp/)

 DO 10  I = 1 , NLAY
   FLW(I) = 0._dp
   TW(I)  = 0._dp
   10 CONTINUE
 K1 = 0

 IF (.NOT. LCLOUD)       GOTO 100

 DO 20  I = 1 , NLAY
   PWZW =  PRESS(I)*1000._dp + 1.0_dp
   IF (TOP .LE. PWZW)    GOTO 200
   20 CONTINUE
  200 K1 = I-1

 ICLT = K1 + 1
 ICLB = ICLT + NCL
 IF(ICLB.GT.NLAY+1) THEN
   WRITE(*,*) ' cloud bottom has to be adjusted'
   ICLB = NLAY + 1
   NCL  = ICLB - ICLT
 ENDIF

 EXMASS  = EWOL(1,IWP)/FIXLWC(IWP)/1000._dp
 ZTOTAL = 0.0_dp

 DO 30  IWO = 1 , NCL
   KWO = K1 + IWO
   ZTOTAL = ZTOTAL + DZ(KWO)*1000._dp
   30 CONTINUE
 ALW = TAUW/EXMASS
 CLOLWC = ALW/ZTOTAL

 TTOP = TEMPE(ICLT)
 TBOT = TEMPE(ICLB)
 EXPON = (7.5_dp*(TTOP-273.15_dp))/(TTOP-35.85_dp)
 ESAT = 6.11_dp*10._dp**EXPON
 ROTOP = 216.67_dp*ESAT/TTOP
 EXPON = (7.5_dp*(TBOT-273.15_dp))/(TBOT-35.85_dp)
 ESAT = 6.11_dp*10._dp**EXPON
 ROBOT = 216.67_dp*ESAT/TBOT
 CLIM = (ROBOT-ROTOP)*0.2_dp
 IF(CLIM.LT.CLOLWC) THEN
 WRITE(*,*) ' WARNING: cloud LWC unrealistically large ',TEMPE
 read(*,*)
 ENDIF

 DO 40  IWO = 1 , NCL
   KWO = K1 + IWO
   FLW(KWO) = CLOLWC*DZ(KWO)*1000._dp
   TW(KWO)  = FLW(KWO)*EXMASS
   40 CONTINUE

  100 CONTINUE

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: VARIA
!
!> Changes atmospheric profiles.
!!
!! @param[in]     RAY    DOUBLE  ? <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     NCL    INTEGER Number of cloud layers.
!! @param[in]     TS     DOUBLE  The difference between the surface and
!!                               characteristical temperatures [K].
!! @param[in]     ROFF   DOUBLE  Difference in water vapor.
!! @param[in]     WH2O   DOUBLE  Precipitable water [cm].
!! @param[in]     K1     INTEGER ?
!! @param[in]     LCLOUD LOGICAL Calculate with or without clouds.
!! @param[in]     DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in,out] TEMPE   DOUBLE  Temperature [C]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     PRESS  DOUBLE  Atmospheric pressure [mbar]. <br>
!!                       1D ARRAY with dimension 31 specified in STRPSRB.
!! @param[in]     ROL    DOUBLE  Air density. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in,out] ROH    DOUBLE  Water content (humidity). <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     RO3    DOUBLE  Ozone content. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    WH2O1  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    WCO2SW DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    WO3SW  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    WO2SW  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    RAYM   DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!!
!! @warning    Variables ROFF and NCL, INTVAL are implicitly defined as
!!             constants in D2STR and then passed to this subroutine.
!! @warning    Variable RAYM depends on RAY for each level, but they have
!!             different dimensions (LAYMAX/LEVMAX respectively).
!!
!! @see        D2STR STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE VARIA(ZSTAT,RAY,NCL,TS,ROFF,WH2O,O3,K1,LCLOUD,DZ,TEMPE,PRESS,ROL,ROH,RO3,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM)
      ! use gradevars
       use Brasilsr_constants
!       changes atmospheric profiles

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     TS : difference between surface and characteristical temperatures
!     ROFF    : difference in water vapor
!     NCL     : number of cloud layers
!     WH2O    : precipitable water [cm]
!     DZ : thickness of each atmospheric layer
!     ROL     : air density
!     ROH     : water content (humidity)
!     RO3     : ozone content
!     LCLOUD  : logical variable to calculate with or without clouds
!     PRESS   : atmospheric pressure (mbar)
!     TEMPE    : temperature (oC)
!     RAY     :
!     K1 :
!     PPMCO2  :
!     CONZ    :
!     QP0     :
!     T0 :
!     DELT    :
!     DYOU    :
!     XCO2    :
!     SH2O    :
!     SH2ONR  :
!     SO3     :
!     SO3NR   :
!     SCO2    :
!     SCO2NR  :
!     KLEV    :
!     DIFT    :
!     NSTOP   :
!     TM :
!     WAS     :
!     POO     :
!     NGRZ    :
!     KGRZ    :
!     TGRZ    :
!     EXPON   :
!     ESAT    :
!     XR :
!     XP :
!     XH :
!     X3 :
!     XL :
!     RAYM    :
!     PP :
!     RO3M    :
!     DI :
!     TQ :
!     TT :
!     WCO2SW  :
!     WCO2    :
!     WO2SW   :
!     WO3SW   :
!     WO3     :
!     WH2O1   :
!     TB :
!......................................................................
IMPLICIT NONE
INTEGER, INTENT(IN) :: NCL,K1
LOGICAL*1, INTENT(IN) :: LCLOUD
DOUBLE PRECISION, INTENT(IN) :: TS,ROFF,WH2O,O3,ZSTAT
DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(IN) :: RAY,ROL,RO3
DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(IN) :: DZ
DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(INOUT) :: TEMPE,ROH
DOUBLE PRECISION, DIMENSION(NLEV), INTENT(IN) :: PRESS
DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT) :: WCO2SW,WO3SW,WO2SW,RAYM

DOUBLE PRECISION, DIMENSION(LAYMAX) :: TT,PP,TB,DI,TM,RM,WO3,WCO2,WH2O1
DOUBLE PRECISION, DIMENSION(LEVMAX) :: WAS,POO
DOUBLE PRECISION :: O3P, O3T, PPMCO2,CONZ,QP0,T0,DELT,DYOU,XCO2,SH2O,SH2ONR,SO3,SO3NR,SCO2
DOUBLE PRECISION :: SCO2NR,DIFT,TGRZ, EXPON,ESAT,XR,XP,XH,X3,XL,RO3M,TQ
INTEGER :: KLEV,NSTOP,IUP,I,NGRZ,KGRZ

 O3P = 0.36479_dp
 O3T = 0.0071908_dp

 ! PPMCO2 = 300.
 PPMCO2 = 401.01_dp
 CONZ=PPMCO2/330._dp
 QP0   = 1._dp/1.013_dp
 T0    = 273.16_dp
 DELT  = 0.0279_dp
 DYOU  = (6._dp+3._dp*DELT)/(6._dp-7._dp*DELT)
 XCO2  = CONZ*33._dp
 SH2O   = 0._dp
 SH2ONR = 0._dp
 SO3    = 0._dp
 SO3NR  = 0._dp
 SCO2   = 0._dp
 SCO2NR = 0._dp

!    surface temperature TS and water vapor adjustment +- 0-1

 KLEV = 5
 DIFT = TS/(KLEV-1)
 NSTOP=NLEV - KLEV - 2

 DO 10  IUP = NLEV,NSTOP,-1
    KLEV = KLEV - 1
    TEMPE(IUP) = TEMPE(IUP) + (DIFT*KLEV)
   10 CONTINUE

 DO 20  I=1,NLEV
    ROH(I) = ROH(I)*(1.0_dp+ROFF)
   20 CONTINUE

 DO 30  I = 1 , NLAY
    TM(I)  = (TEMPE(I)+TEMPE(I+1))*0.5_dp
    WAS(I) = ROH(I)/ROL(I)
    POO(I) = RO3(I)/ROL(I)
   30 CONTINUE
   WAS(NLEV) = ROH(NLEV)/ROL(NLEV)
   POO(NLEV) = RO3(NLEV)/ROL(NLEV)

 IF (.NOT. LCLOUD)  GOTO  100
   NGRZ = NCL +1
    DO 40 I = 1 , NGRZ
      KGRZ = K1 + I
      TGRZ = TEMPE(KGRZ)
      EXPON= (7.5_dp*(TGRZ-273.15_dp))/(TGRZ-35.85_dp)
      ESAT = 6.11_dp*10._dp**EXPON
      ROH(KGRZ) = 216.67_dp*ESAT/TGRZ
   40    CONTINUE
  100 CONTINUE

 DO I=1,NLAY
     XR = DLOG(RAY(I)/RAY(I+1))
     XP = DLOG(PRESS(I)/PRESS(I+1))
     XH = DLOG(ROH(I)/ROH(I+1))
     X3 = DLOG(RO3(I)/RO3(I+1))
     XL = DLOG(ROL(I)/ROL(I+1))

     RAYM(I) = RAY(I)*DEXP(-0.5_dp*XR) * DYOU
     PP(I)   = PRESS(I)*DEXP(-0.5_dp*XP) * QP0

     RM(I)   = ROH(I)*DEXP(-0.5_dp*XH)

     RO3M    = RO3(I)*DEXP(-0.5_dp*X3)
     DI(I)   = ROL(I)*DEXP(-0.5_dp*XL)
     TQ=TM(I)
     TT(I)=T0/TQ
     WCO2SW(I) = XCO2*PP(I)**1.75_dp*TT(I)**1.35_dp
     WCO2(I)   = CONZ*PP(I)**1.66_dp*TT(I)
     WO2SW(I)  = PP(I)**1.75_dp*TT(I)
     WO3SW(I)  = 46.667_dp*RO3M
     WO3(I)    = WO3SW(I)*PP(I)**O3P*TT(I)**O3T

     WH2O1(I)   = 0.4_dp*RM(I)*PP(I)**0.9*TT(I)**0.45_dp

     TB(I)     = 1745._dp * (1._dp/TQ - 1._dp/296._dp)

     SH2O   = SH2O   + RM(I)*0.1_dp * DZ(I)
     SH2ONR = SH2ONR + WH2O1(I)     * DZ(I)
     SO3    = SO3    + WO3SW(I)    * DZ(I)*1000 /(46.667_dp*48._dp)*(6.022E23/2.69E20)
     SO3NR  = SO3NR  + WO3(I) * DZ(I)
     SCO2   = SCO2   + WCO2SW(I)   * DZ(I)
     SCO2NR = SCO2NR + WCO2(I)     * DZ(I)
END DO

 DO I=1,NLAY
   ! WH2O1(I) = WH2O1(I)*WH2O/SH2O
   WH2O1(I) = WH2O1(I)*WH2O*DEXP(-ZSTAT*XH/1000._dp)/SH2O
 END DO

 DO I=1,NLAY
   WO3SW(I) = WO3SW(I)*O3/SO3
 END DO

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: AERO
!
!> Determination of aerosol profiles.
!!
!! @param[in]     SCA    DOUBLE  ? <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     ALT    DOUBLE  Altitude [m]. <br>
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    INAERO INTEGER  ? <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    ASIG   DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LEVMAX,MAXWEL.
!!                       Both parameters are specified in STRPSRB.
!!
!! @see        STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE AERO(BIOMA,SCA,DZ,ALT,INAERO,ASIG,EAngstrom)
      ! use gradevars
       use Brasilsr_constants
       use Brasilsr_commonspectral
!     determination of aerosol profiles

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     SCA     :
!     DZ : thickness of each atmospheric layer
!     ALT     : altitude (m)
!     INAERO  :
!     ASIG    :
!     K  :
!     XS :
!     S! :
!     EXAER   :
!     VIS1    :
!......................................................................
IMPLICIT NONE

 INTEGER, INTENT(in)::BIOMA
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(in)::DZ
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(in)::SCA,ALT
 INTEGER, DIMENSION(LAYMAX),INTENT(OUT)::INAERO
 DOUBLE PRECISION, DIMENSION(LEVMAX,MAXWEL), INTENT(OUT)::ASIG
 DOUBLE PRECISION, INTENT(IN),OPTIONAL::EAngstrom

 DOUBLE PRECISION, DIMENSION(MAXWEL,5)::EXAER
 INTEGER::aero_sup,I,J,K
 DOUBLE PRECISION EA,Sc,XS


EXAER(:,1)=init_exaer(1,EA=EAngstrom)
! EXAER(:,2)=init_exaer(2)
 IF (BIOMA.EQ.1) THEN
   aero_sup=4 !Amazonia
   EXAER(:,aero_sup)=init_exaer(aero_sup,EA=EAngstrom)
 ELSEIF (BIOMA.EQ.3) THEN
   aero_sup=5 !Cerrado
  EXAER(:,aero_sup)=init_exaer(aero_sup,EA=EAngstrom)
 ELSE
   aero_sup=2 !All others, use continental
   EXAER(:,aero_sup)=init_exaer(aero_sup,EA=EAngstrom)
 ENDIF

 INAERO  = 1
 DO 10  I = 1 , NLAY
   ! IF (ALT(I) .LT. 10.01_dp)    INAERO(I)=2
   IF (ALT(I) .LT. 10.01_dp)    INAERO(I)=aero_sup
10 CONTINUE

   EXAER(:,aero_sup)=init_exaer(aero_sup,EA=EAngstrom)
    DO 20  I = 1 , 37
    DO 30  J = 1 , NLAY
        K    = INAERO(J)
        ! XS   = DLOG(SCA(J)/SCA(J+1))
        ! Sc   = SCA(J)*DEXP(-0.5_dp*XS)
        ! ASIG(J,I) = EXAER(I,K)*SC * DZ(J)
        ! Kext and SCA are calculated from AOD assuming the value represents the whole layer, not the border, so the interpolation above is not correct in this context
        ASIG(J,I) = EXAER(I,K)*SCA(J+1) * DZ(J)
   30    CONTINUE
   20    CONTINUE
 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: ATMOS
!
!> Determination of absorption and scattering.
!!
!! @param[in]     DZ     DOUBLE  Thickness of each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     WH2O1  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     WCO2SW DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     WO3SW  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     WO2SW  DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     RAYM   DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    U      DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LAYMAX,MAXIND.
!!                       Both parameters are specified in STRPSRB.
!! @param[out]    V      DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LAYMAX,MAXIND.
!!                       Both parameters are specified in STRPSRB.
!!
!! @see        STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE ATMOS (DZ,WH2O1,WCO2SW,WO3SW,WO2SW,RAYM,U,V)
      ! use gradevars
       use Brasilsr_constants
       use Brasilsr_spectral_grid
!      determination of absorption and scattering

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     INDEX   : index of spectral intervals
!     DZ : thickness of each atmospheric layer
!     WH2O1   :
!     WCO2SW  :
!     WO3SW   :
!     WO2SW   :
!     RAYM    :
!     U  :
!     V  :
!     SIHO    :
!     SICO    :
!     SIO3    :
!     B! :
!     KB!     :
!     B  :
!     RB :
!     ANUE    :
!     RANUE   :
!     WE :
!     XWELSW  :
!     X1 :
!     AS :
!     X2 :
!     SR :
!......................................................................

 IMPLICIT NONE

 DOUBLE PRECISION, DIMENSION (LAYMAX), INTENT(IN) :: DZ, WH2O1,WCO2SW,WO3SW,WO2SW,RAYM
 DOUBLE PRECISION, DIMENSION (LAYMAX,MAXIND), INTENT(OUT) :: U,V

 DOUBLE PRECISION, DIMENSION(MAXWEL) :: XWELSW
 DOUBLE PRECISION, DIMENSION(LAYMAX) :: SICO,SIHO,SIO3,SR
 INTEGER :: I, IALL
 DOUBLE PRECISION :: BC,B,ANUE,AS,X1,X2,WE

   XWELSW = (/0.2025_dp, 0.210_dp, 0.220_dp, 0.230_dp, 0.240_dp, 0.250_dp, 0.260_dp, 0.270_dp, 0.280_dp, &
             0.290_dp, 0.3025_dp, 0.320_dp, 0.340_dp, 0.360_dp, 0.380_dp, 0.4075_dp, 0.450_dp, 0.500_dp, &
             0.550_dp, 0.600_dp, 0.650_dp, 0.6925_dp, 0.735_dp, 0.765_dp, 0.7775_dp, 0.826_dp, 0.877_dp, &
             0.9465_dp, 1.043_dp, 1.145_dp, 1.2435_dp, 1.426_dp, 1.6225_dp, 1.890_dp, 2.145_dp, 2.5945_dp, &
             3.285_dp/)

 DO 10  IALL = 1 , MAXIND
   DO 20   I = 1 , NLAY
      SIHO(I) = 0.0_dp
      SICO(I) = 0.0_dp
      SIO3(I) = 0.0_dp
   20   CONTINUE
   BC   =  KBC(IALL)*0.000001_dp
   B    =  RB(IALL)
   ANUE =  RANUE(IALL)
   WE   =  XWELSW(SPECTRAL_INDEX(IALL))

   IF  (IALL   .EQ. 1)   GOTO 100
   IF  (SPECTRAL_INDEX(IALL) .EQ. SPECTRAL_INDEX(IALL-1)) GOTO 200
  100   CONTINUE
  ! @cite Bodhaine1999 eq. 1
   X1   =  1._dp/(WE*WE)
   AS   =  (6432.8_dp+2949810._dp/(146._dp-X1)+25540._dp/(41._dp-X1))*1.0E-08+1._dp
   X2   =  (AS*AS - 1._dp)**2
   DO 30  I = 1 , NLAY
     SR(I)=1.352_dp*0.1_dp*X2*RAYM(I)/(WE**4)
   30   CONTINUE
  200   CONTINUE

   IF  (KBC(IALL) .EQ. 0)     GOTO 300
   DO 40  I = 1 , NLAY
     IF (WE .GT. 1.) SICO(I)=WCO2SW(I)*BC
     IF (WE .LT. 1.) SICO(I)=WO2SW(I) *BC
   40   CONTINUE
  300   CONTINUE

   IF  (B .LT. 0.000001_dp)      GOTO 400
   DO 50  I = 1 , NLAY
     SIHO(I) = WH2O1(I)*B
   50   CONTINUE
  400   CONTINUE

   IF  (ANUE .LT. 0.000001_dp)   GOTO 500
   DO 60  I = 1 , NLAY
     SIO3(I) = WO3SW(I)*ANUE
   60   CONTINUE
  500   CONTINUE

   DO 70  I = 1 , NLAY
     V(I,IALL) = (SIO3(I) + SIHO(I) + SICO(I)) * DZ(I)
     U(I,IALL) = V(I,IALL) + SR(I) * DZ(I)
   70   CONTINUE
   10 CONTINUE

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: BETAS
!
!> Determination of spectral absorption, extinction, and scattering for each
!! atmospheric layer.
!!
!! @param[in]     NCL    INTEGER Number of cloud layers.
!! @param[in]     IALL   INTEGER Counter of spectral interval.
!! @param[in]     COSZEN DOUBLE  Cossine of zenith angle.
!! @param[out]    BET    DOUBLE  Backscatter coefficient, for each atmospheric
!!                       layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    BET0   DOUBLE  Backscatter coefficient at zenith angle equal
!!                       to zero, for each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    OM     DOUBLE  Single-scattering albedo coefficient for each
!!                       atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    TAU    DOUBLE  Optical depth of atmospheric layers containing
!!                       clouds. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[out]    TOHNW  DOUBLE  Optical depth of atmospheric layers without
!!                       clouds. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     TW     DOUBLE  ?
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     K1     INTEGER ?
!! @param[in]     IWP    INTEGER Cloud droplet size distribution.
!! @param[in]     LCLOUD LOGICAL Calculate with or without clouds.
!! @param[in]     INAERO INTEGER  ? <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     ASIG   DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LEVMAX,MAXWEL.
!!                       Both parameters are specified in STRPSRB.
!! @param[in]     U      DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LAYMAX,MAXIND.
!!                       Both parameters are specified in STRPSRB.
!! @param[in]     V      DOUBLE  ? <br>
!!                       2D ARRAY with dimensions LAYMAX,MAXIND.
!!                       Both parameters are specified in STRPSRB.
!!
!! @see        STRPSRB
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE BETAS(NCL,IALL,COSZEN,TW,K1,IWP,LCLOUD,INAERO,ASIG,U,V,BET,BET0,OM,TAU,TOHNW,scaling)
      ! use gradevars
       use Brasilsr_constants
       use Brasilsr_spectral_grid, only: MAXIND, INWOSW, SPECTRAL_INDEX

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     IWP     : cloud droplet size distribution
!     NCL     : number of cloud layers
!     LCLOUD  : logical variable to calculate with or without clouds
!     TW :
!     K1 :
!     IALL    : counter of spectral interval
!     COSZEN  : cossine of zenith angle
!     BET     :
!     BET0    :
!     OM :
!     TAU     :
!     TOHNW   :
!     INAERO  :
!     ASIG    :
!     U  :
!     V  :
!     UMUE    :
!     KA :
!     KB :
!     K! :
!     XY :
!     T  :
!     X1 :
!     X2 :
!     OME     :
!     GV :
!     OMAER   :
!     GAER    :
!     EF :
!     GE :
!     LA :
!     OWO     :
!     AWOL    :
!     GWO     :
!     GWOL    :
!     IWO     :
!     TWO     :
!     EWOL    :
!     TGES    :
!     H1 :
!     H2 :
!     H3 :
!......................................................................

 IMPLICIT NONE
 LOGICAL, INTENT(IN),OPTIONAL::scaling
 LOGICAL*1, INTENT(IN):: LCLOUD
 INTEGER, INTENT(IN)::NCL, IALL, K1,IWP
 INTEGER, DIMENSION(LAYMAX), INTENT(IN)::INAERO
 DOUBLE PRECISION, INTENT(IN)::COSZEN
 DOUBLE PRECISION, DIMENSION(LEVMAX,MAXWEL), INTENT(IN)::ASIG
 DOUBLE PRECISION, DIMENSION(LAYMAX,MAXIND), INTENT(IN)::U,V
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(IN)::TW
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(OUT):: BET,BET0,OM,TAU,TOHNW


 DOUBLE PRECISION, DIMENSION(16,8):: AWOL,EWOL,GWOL
 DOUBLE PRECISION, DIMENSION(MAXWEL,5):: OMAER,GAER
 DOUBLE PRECISION, DIMENSION(LAYMAX)::EF
 INTEGER::I,KA,KB,LA,KC,IWO
 DOUBLE PRECISION::XY,T,X1,X2,OME,GV,GE,OWO,GWO,TWO,TGES,H1,H2,H3

 ! IF(.NOT.present(scaling)) THEN
 !   scaling=.TRUE.
 ! ENDIF

AWOL(:,1) = (/.000D+0,.602D-5,.110D-4,.900D-5,.140D-4,.220D-4,.195D-3,.121D-3,&
              .300D-3,.430D-3,.501D-2,.273D-2,.317D-2,.607D-2,.398D-1,.417D+0/)
AWOL(:,2) = (/.000D+0,.799D-5,.150D-4,.130D-4,.230D-4,.280D-4,.226D-3,.147D-3,&
              .332D-3,.513D-3,.608D-2,.332D-2,.404D-2,.793D-2,.553D-1,.438D+0/)
AWOL(:,3) = (/.000D+0,.900D-5,.180D-4,.150D-4,.230D-4,.330D-4,.234D-3,.165D-3,&
              .400D-3,.578D-3,.777D-2,.394D-2,.473D-2,.954D-2,.688D-1,.459D+0/)
AWOL(:,4) = (/.000D+0,.150D-4,.290D-4,.300D-4,.460D-4,.530D-4,.365D-3,.261D-3,&
              .585D-3,.909D-3,.113D-1,.607D-2,.728D-2,.157D-1,.114D+0,.481D+0/)
AWOL(:,5) = (/.000D+0,.170D-4,.310D-4,.270D-4,.410D-4,.610D-4,.450D-3,.319D-3,&
              .800D-3,.116D-2,.135D-1,.740D-2,.883D-2,.176D-1,.130D+0,.474D+0/)
AWOL(:,6) = (/.101D-5,.280D-4,.470D-4,.450D-4,.520D-4,.106D-3,.779D-3,.638D-3,&
              .150D-2,.232D-2,.296D-1,.175D-1,.215D-1,.424D-1,.255D+0,.468D+0/)
AWOL(:,7) = (/.256D-5,.269D-4,.349D-4,.574D-4,.785D-4,.168D-3,.278D-3,.732D-3,&
              .886D-3,.250D-2,.614D-2,.799D-1,.615D-1,.768D-1,.124D+0,.456D+0/)
AWOL(:,8) = (/.373D-5,.393D-4,.519D-4,.852D-4,.115D-3,.296D-3,.415D-3,.107D-2,&
              .132D-2,.366D-2,.905D-2,.113D+0,.875D-1,.108D+0,.170D+0,.452D+0/)
!-----------------------
GWOL(:,1) = (/.832_dp,  .830_dp,  .811_dp,  .829_dp,  .819_dp,  .812_dp,  .798_dp,  .803_dp, &
              .799_dp,  .795_dp,  .776_dp,  .771_dp,  .767_dp,  .796_dp,  .881_dp,  .820_dp/)
GWOL(:,2) = (/.836_dp,  .828_dp,  .813_dp,  .830_dp,  .820_dp,  .818_dp,  .797_dp,  .805_dp, &
              .803_dp,  .801_dp,  .789_dp,  .790_dp,  .773_dp,  .798_dp,  .868_dp,  .829_dp/)
GWOL(:,3) = (/.848_dp,  .819_dp,  .840_dp,  .836_dp,  .838_dp,  .830_dp,  .817_dp,  .829_dp, &
              .820_dp,  .812_dp,  .791_dp,  .799_dp,  .776_dp,  .788_dp,  .860_dp,  .852_dp/)
GWOL(:,4) = (/.862_dp,  .857_dp,  .825_dp,  .850_dp,  .849_dp,  .851_dp,  .849_dp,  .848_dp, &
              .847_dp,  .846_dp,  .836_dp,  .838_dp,  .829_dp,  .822_dp,  .845_dp,  .923_dp/)
GWOL(:,5) = (/.855_dp,  .834_dp,  .843_dp,  .839_dp,  .838_dp,  .837_dp,  .833_dp,  .837_dp, &
              .830_dp,  .826_dp,  .814_dp,  .818_dp,  .808_dp,  .814_dp,  .857_dp,  .887_dp/)
GWOL(:,6) = (/.860_dp,  .849_dp,  .850_dp,  .847_dp,  .847_dp,  .857_dp,  .843_dp,  .828_dp, &
              .835_dp,  .842_dp,  .834_dp,  .834_dp,  .824_dp,  .818_dp,  .872_dp,  .904_dp/)
GWOL(:,7) = (/.885_dp,  .885_dp,  .885_dp,  .885_dp,  .885_dp,  .885_dp,  .884_dp,  .884_dp, &
              .884_dp,  .884_dp,  .884_dp,  .897_dp,  .895_dp,  .900_dp,  .923_dp,  .938_dp/)
GWOL(:,8) = (/.887_dp,  .888_dp,  .888_dp,  .888_dp,  .888_dp,  .888_dp,  .888_dp,  .888_dp, &
              .887_dp,  .888_dp,  .889_dp,  .906_dp,  .904_dp,  .910_dp,  .935_dp,  .939_dp/)
!-----------------------
EWOL(:,1) =  (/15.2_dp,  15.4_dp,  15.7_dp,  15.3_dp,  15.6_dp,  15.8_dp,  16.0_dp,  15.8_dp, &
               16.0_dp,  16.0_dp,  15.9_dp,  16.5_dp,  17.1_dp,  17.2_dp,  21.2_dp,  18.1_dp/)
EWOL(:,2) =  (/39.4_dp,  40.1_dp,  41.0_dp,  40.9_dp,  42.3_dp,  41.9_dp,  41.6_dp,  41.2_dp, &
               42.1_dp,  42.9_dp,  43.5_dp,  43.2_dp,  44.1_dp,  45.8_dp,  50.7_dp,  47.1_dp/)
EWOL(:,3) =  (/72.4_dp,  72.7_dp,  74.1_dp,  75.5_dp,  73.8_dp,  73.9_dp,  74.6_dp,  74.6_dp, &
               75.0_dp,  75.2_dp,  77.5_dp,  78.4_dp,  78.8_dp,  80.0_dp,  85.6_dp,  82.4_dp/)
EWOL(:,4) =  (/73.9_dp,  76.6_dp,  74.4_dp,  75.1_dp,  75.5_dp,  75.4_dp,  74.8_dp,  76.0_dp, &
               76.1_dp,  76.2_dp,  77.2_dp,  77.4_dp,  77.7_dp,  80.3_dp,  80.8_dp,  81.2_dp/)
EWOL(:,5) = (/127.8_dp, 131.1_dp, 130.3_dp, 129.1_dp, 130.9_dp, 129.8_dp, 130.9_dp, 130.4_dp, &
              130.6_dp, 130.7_dp, 134.9_dp, 132.8_dp, 136.4_dp, 137.4_dp, 138.6_dp, 140.2_dp/)
EWOL(:,6) = (/121.1_dp, 122.1_dp, 121.2_dp, 122.3_dp, 121.0_dp, 122.3_dp, 121.0_dp, 120.4_dp, &
              121.4_dp, 122.3_dp, 124.1_dp, 123.3_dp, 125.8_dp, 126.4_dp, 125.2_dp, 127.1_dp/)
EWOL(:,7) =   (/5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp,   5.5_dp, &
                5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.6_dp,   5.7_dp,   5.7_dp/)
EWOL(:,8) =  (/12.3_dp,  12.3_dp,  12.3_dp,  12.3_dp,  12.4_dp,  12.4_dp,  12.4_dp,  12.4_dp, &
               12.4_dp,  12.4_dp,  12.4_dp,  12.5_dp,  12.5_dp,  12.6_dp,  12.6_dp,  12.7_dp/)
!-----------------------
 ! Stratospheric aerosol @cite Schmetz1993 Table1
OMAER(:,1) = (/1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, &
               1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, &
               1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, &
               1.000_dp, 1.000_dp, 1.000_dp, 1.000_dp, 0.999_dp, 0.995_dp, 0.980_dp, 0.952_dp, 0.775_dp, &
               0.072_dp/)
 ! Continental aerosol @cite Schmetz1993 Table1
OMAER(:,2) = (/0.661_dp, 0.679_dp, 0.703_dp, 0.727_dp, 0.751_dp, 0.775_dp, 0.793_dp, 0.811_dp, 0.828_dp, &
               0.846_dp, 0.867_dp, 0.885_dp, 0.902_dp, 0.902_dp, 0.901_dp, 0.901_dp, 0.899_dp, 0.898_dp, &
               0.891_dp, 0.889_dp, 0.885_dp, 0.879_dp, 0.869_dp, 0.863_dp, 0.860_dp, 0.849_dp, 0.839_dp, &
               0.826_dp, 0.807_dp, 0.793_dp, 0.781_dp, 0.761_dp, 0.751_dp, 0.767_dp, 0.769_dp, 0.652_dp, &
               0.758_dp/)
 ! Maritime aerosol @cite Schmetz1993 Table1
OMAER(:,3) = (/0.848_dp, 0.859_dp, 0.874_dp, 0.890_dp, 0.905_dp, 0.920_dp, 0.931_dp, 0.942_dp, 0.954_dp, &
               0.965_dp, 0.977_dp, 0.980_dp, 0.984_dp, 0.985_dp, 0.985_dp, 0.986_dp, 0.987_dp, 0.988_dp, &
               0.988_dp, 0.989_dp, 0.990_dp, 0.990_dp, 0.989_dp, 0.989_dp, 0.989_dp, 0.988_dp, 0.988_dp, &
               0.987_dp, 0.986_dp, 0.986_dp, 0.986_dp, 0.985_dp, 0.986_dp, 0.985_dp, 0.983_dp, 0.901_dp, &
               0.764_dp/)

! Amazonia
OMAER(:,4) = (/0.9220_dp, 0.9227_dp, 0.9254_dp, 0.9299_dp, 0.9309_dp, 0.9337_dp, 0.9354_dp, 0.9357_dp,&
              0.9354_dp, 0.9367_dp, 0.9387_dp, 0.9396_dp, 0.9400_dp, 0.9386_dp, 0.9383_dp, 0.9387_dp,&
              0.9371_dp, 0.9345_dp, 0.9334_dp, 0.9307_dp, 0.9262_dp, 0.9227_dp, 0.9189_dp, 0.9162_dp,&
              0.9150_dp, 0.9095_dp, 0.9038_dp, 0.8959_dp, 0.8845_dp, 0.8726_dp, 0.8619_dp, 0.8440_dp,&
              0.8282_dp, 0.8143_dp, 0.8071_dp, 0.8058_dp, 0.8157_dp/)
! Cerrado
OMAER(:,5) = (/0.9020_dp, 0.9026_dp, 0.9059_dp, 0.9107_dp, 0.9121_dp, 0.9151_dp, 0.9170_dp, 0.9171_dp,&
              0.9167_dp, 0.9182_dp, 0.9202_dp, 0.9212_dp, 0.9214_dp, 0.9195_dp, 0.9188_dp, 0.9189_dp,&
              0.9169_dp, 0.9149_dp, 0.9147_dp, 0.9125_dp, 0.9078_dp, 0.9036_dp, 0.8992_dp, 0.8961_dp,&
              0.8947_dp, 0.8886_dp, 0.8819_dp, 0.8726_dp, 0.8605_dp, 0.8476_dp, 0.8363_dp, 0.8199_dp,&
              0.8062_dp, 0.7977_dp, 0.7960_dp, 0.8019_dp, 0.8176_dp/)
!-----------------------
 ! Stratospheric aerosol @cite Schmetz1993 Table1
GAER(:,1) = (/0.681_dp, 0.683_dp, 0.685_dp, 0.688_dp, 0.690_dp, 0.693_dp, 0.697_dp, 0.701_dp, 0.705_dp, &
              0.709_dp, 0.714_dp, 0.719_dp, 0.726_dp, 0.730_dp, 0.735_dp, 0.739_dp, 0.737_dp, 0.734_dp, &
              0.726_dp, 0.716_dp, 0.705_dp, 0.694_dp, 0.682_dp, 0.675_dp, 0.672_dp, 0.659_dp, 0.647_dp, &
              0.626_dp, 0.597_dp, 0.566_dp, 0.535_dp, 0.480_dp, 0.422_dp, 0.350_dp, 0.289_dp, 0.208_dp, &
              0.137_dp/)
 ! Continental aerosol @cite Schmetz1993 Table1
GAER(:,2) = (/0.724_dp, 0.718_dp, 0.711_dp, 0.703_dp, 0.696_dp, 0.688_dp, 0.682_dp, 0.676_dp, 0.671_dp, &
              0.665_dp, 0.658_dp, 0.655_dp, 0.651_dp, 0.649_dp, 0.648_dp, 0.645_dp, 0.643_dp, 0.639_dp, &
              0.637_dp, 0.635_dp, 0.632_dp, 0.631_dp, 0.632_dp, 0.632_dp, 0.632_dp, 0.633_dp, 0.633_dp, &
              0.632_dp, 0.631_dp, 0.633_dp, 0.636_dp, 0.641_dp, 0.656_dp, 0.697_dp, 0.732_dp, 0.776_dp, &
              0.782_dp/)
 ! Maritime aerosol @cite Schmetz1993 Table1
GAER(:,3) = (/0.774_dp, 0.772_dp, 0.769_dp, 0.765_dp, 0.762_dp, 0.759_dp, 0.757_dp, 0.754_dp, 0.752_dp, &
              0.749_dp, 0.747_dp, 0.745_dp, 0.744_dp, 0.745_dp, 0.745_dp, 0.746_dp, 0.747_dp, 0.747_dp, &
              0.744_dp, 0.747_dp, 0.750_dp, 0.751_dp, 0.752_dp, 0.753_dp, 0.753_dp, 0.754_dp, 0.755_dp, &
              0.757_dp, 0.760_dp, 0.762_dp, 0.764_dp, 0.769_dp, 0.775_dp, 0.783_dp, 0.790_dp, 0.835_dp, &
              0.760_dp/)
! Amazonia
GAER(:,4) = (/0.7385_dp, 0.7329_dp, 0.7279_dp, 0.7354_dp, 0.7373_dp, 0.7377_dp, 0.7396_dp, 0.7363_dp,&
              0.7282_dp, 0.7234_dp, 0.7237_dp, 0.7233_dp, 0.7215_dp, 0.7114_dp, 0.6985_dp, 0.6905_dp,&
              0.6802_dp, 0.6528_dp, 0.6322_dp, 0.6151_dp, 0.5919_dp, 0.5746_dp, 0.5589_dp, 0.5495_dp,&
              0.5454_dp, 0.5285_dp, 0.5110_dp, 0.4929_dp, 0.4786_dp, 0.4619_dp, 0.4568_dp, 0.4650_dp,&
              0.4804_dp, 0.5179_dp, 0.5537_dp, 0.6009_dp, 0.6392_dp/)
! Cerrado
GAER(:,5) = (/0.7467_dp, 0.7413_dp, 0.7364_dp, 0.7422_dp, 0.7436_dp, 0.7437_dp, 0.7446_dp, 0.7410_dp,&
              0.7333_dp, 0.7284_dp, 0.7277_dp, 0.7267_dp, 0.7242_dp, 0.7143_dp, 0.7016_dp, 0.6930_dp,&
              0.6820_dp, 0.6545_dp, 0.6336_dp, 0.6168_dp, 0.5944_dp, 0.5778_dp, 0.5631_dp, 0.5548_dp,&
              0.5512_dp, 0.5369_dp, 0.5214_dp, 0.5061_dp, 0.4984_dp, 0.4870_dp, 0.4860_dp, 0.5039_dp,&
              0.5236_dp, 0.5635_dp, 0.5963_dp, 0.6345_dp, 0.6601_dp/)
!-----------------------

 KA    =  SPECTRAL_INDEX(IALL)
 DO 10  I = 1 , NLAY
    KB  = INAERO(I)
    XY  = ASIG(I,KA)
    T   = XY + U(I,IALL)
    X1  = XY*OMAER(KA,KB)
    X2  = U(I,IALL) - V(I,IALL)
    OME = (X1+X2)/T
    GV  = X1*GAER(KA,KB)/(X1+X2)
    EF(I)    = GV*GV
    OM(I)    = OME*(1._dp-EF(I))/(1._dp-OME*EF(I))

    TAU(I)   = (ASIG(I,KA)+U(I,IALL))*(1._dp-OME*EF(I))
    if (present(scaling)) then
      TAU(I)   =  ASIG(I,KA)+U(I,IALL)
    endif
    TOHNW(I) = TAU(I)
    GE  = (GV-EF(I))/(1._dp-EF(I))
    BET(I)   = (3._dp-3._dp*GE)/8._dp
    BET0(I)  = 0.5_dp-COSZEN*0.75_dp*GE
   10 CONTINUE

 IF (.NOT. LCLOUD)       GOTO 100

  LA  = INWOSW(IALL)
  OWO = 1. - AWOL(LA,IWP)
  GWO = GWOL(LA,IWP)

 DO 20  IWO = 1 , NCL

    KC  = K1 + IWO
    TWO = TW(KC)*EWOL(LA,IWP)/EWOL(1,IWP)
    TGES     = ASIG(KC,KA) + U(KC,IALL) + TWO
    H1  = U(KC,IALL) - V(KC,IALL)
    H2  = OWO*TWO
    KB  = INAERO(KC)
    H3  = OMAER(KA,KB)*ASIG(KC,KA)
    OME = (H1+H2+H3)/TGES
    GV  = (H3*GAER(KA,KB)+GWO*H2)/(H1+H2+H3)
    EF(KC)   = GV*GV
    OM(KC)   = OME*(1._dp-EF(KC))/(1._dp-OME*EF(KC))
    GE  = (GV-EF(KC))/(1._dp-EF(KC))
    TAU(KC)  = TGES*(1._dp-OME*EF(KC))
    BET(KC)  = (3._dp-3._dp*GE)/8._dp
    BET0(KC) = 0.5_dp-COSZEN*0.75_dp*GE
   20 CONTINUE

  100 CONTINUE
 !   write(18,*)OM(KC)
  !write(18,*)'111','111','111'
! 1005 FORMAT(1X,F30.25)
!  1005 FORMAT(1X,F30.10)
 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: MATBAU
!
!> Calculation of diffuse irradiation for each spectral interval.
!!
!! @param[in]     NLY    INTEGER Number of atmospheric layers minus 1 (NLAY-1).
!! @param[in]     ALDIF  DOUBLE  Surface albedo for cloudy sky.
!! @param[in]     ALDR   DOUBLE  Surface albedo for clear sky.
!! @param[in]     COSZEN DOUBLE  Cossine of zenith angle.
!! @param[in]     BET    DOUBLE  Backscatter coefficient, for each atmospheric
!!                       layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     BET0   DOUBLE  Backscatter coefficient at zenith angle equal
!!                       to zero, for each atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     OM     DOUBLE  Single-scattering albedo coefficient for each
!!                       atmospheric layer. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     TAU    DOUBLE  Optical depth of atmospheric layers containing
!!                       clouds. <br>
!!                       1D ARRAY with dimension LAYMAX, a parameter specified
!!                       in STRPSRB.
!! @param[in]     DIR    DOUBLE  Direct irradiation for each spectral interval
!!                       [Wh/m2].
!!                       1D ARRAY with dimension 31.
!! @param[out]    DIF    DOUBLE  Diffuse irradiation for each spectral interval
!!                       [Wh/m2].
!!                       1D ARRAY with dimension LEVMAX, a parameter specified
!!                       in STRPSRB.
!!
!! @warning    Input array DIR has a hardcoded dimension. It should be declared
!!             using a parameter (LEVMAX?).
!! @warning    Several arrays have hardcoded dimensions. Some are used as input
!!             and/or output for other subroutines (LINGAU).
!!
!! @see        STRPSRB LINGAU
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE MATBAU(NLY,ALDIF,ALDR,COSZEN,BET,BET0,OM,TAU,DIR,DIF)
      ! use gradevars
       use Brasilsr_constants
! calculation of diffuse irradiation for each spectral interval

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     DIR     : direct irradiation for each spectral interval (Wh/m2)
!     DIF     : diffuse irradiation for each spectral interval (Wh/m2)
!     NLY     : NLAY-1
!     ALDIF   : surface albedo for cloudy sky
!     ALDR    : surface albedo for clear sky
!     COSZEN  : cossine of zenith angle
!     BET     :
!     BET0    :
!     OM :
!     TAU     :
!     UMUE    :
!     ISS     :
!     WU :
!     BT :
!     BT0     :
!     OMEG    :
!     HH :
!     HMH     :
!     A1 :
!     A2 :
!     A3 :
!     A4 :
!     E  :
!     D1 :
!     D2 :
!     C! :
!     X  :
!     Y  :
!     D  :
!     F  :
!     DW :
!     XH :
!     G  :
!     N  :
!     AM3     :
!     AM4     :
!     AM1     :
!     AM2     :
!     S  :
!     UP :
!     HHH     :
!     HHD     :
!     XNET    :
!     DX1     :
!     DX :
!......................................................................

 IMPLICIT NONE

 INTEGER, INTENT(IN) :: NLY
 DOUBLE PRECISION, INTENT(IN) :: ALDIF,ALDR,COSZEN
 DOUBLE PRECISION, DIMENSION(LAYMAX), INTENT(IN) :: BET, BET0,TAU,OM
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(IN) :: DIR
 DOUBLE PRECISION, DIMENSION(LEVMAX), INTENT(OUT) :: DIF

 DOUBLE PRECISION, DIMENSION(LAYMAX2)::AM1,AM2,AM3,AM4,S
 DOUBLE PRECISION, DIMENSION(LAYMAX)::G,XH,X,Y,D,F
 DOUBLE PRECISION, DIMENSION(LEVMAX) :: UP
 DOUBLE PRECISION :: UMUE,WU,BT,BT0,OMEG,HMH,A1,A2,A3,A4,E,D1,D2,CC,DW,HH,HHH,HHD,XNET,DX1,DX
 INTEGER :: ISS,I,N,L,LA,LB

 ! DIMENSION AM1(100),AM2(100),AM3(100),AM4(100),S(100)
 ! DIMENSION DIR(31),DIF(LEVMAX),UP(LEVMAX)
 ! DIMENSION D(50),F(50),X(50),Y(50)
 ! DIMENSION BET(LAYMAX),BET0(LAYMAX)
 ! dimension TAU(LAYMAX), OM(LAYMAX), G(50), XH(50)
! real*8 TAU(LAYMAX)
! real*4 OM(LAYMAX)
! real*8 G(50)
! real*4 XH(50)

 UMUE  =  2._dp
 ISS  =   NLY + 1
 WU   =   1._dp / COSZEN

 DO 10  I = 1 , NLY
   BT   = BET (I)
   BT0  = BET0(I)
   OMEG = OM  (I)
   HH   = DBLE(0.9999999_dp)
   OMEG = DMIN1(OMEG,HH)
   HMH  = DBLE(0.0000001_dp)
   OMEG = DMAX1(OMEG,HMH)
   A1   = (1._dp-OMEG*(1._dp-BT)) * UMUE
   A2   = BT   * OMEG * UMUE
   A3   = OMEG * BT0
   A4   = OMEG * (1._dp-BT0)
   E    = SQRT(A1*A1-A2*A2)
   D1   = -A1*A3 - A2*A4 + A3/COSZEN
   D2   = -A1*A4 - A2*A3 - A4/COSZEN
   CC   = WU*WU - E*E
   X(I) = D1/CC
   Y(I) = D2/CC
   D(I) = A2/(A1+E)
   F(I) = A2/(A1-E)
   DW   = E * TAU(I)
   DW   = DMIN1(100._dp,DW)
   XH(I)= DEXP(-DW)
   G(I) = 1./XH(I)
   10 CONTINUE

 N =   2*NLY
 AM3(1) =   D(1)
 AM4(1) =   F(1)
 AM1(N) =   (1._dp-ALDIF*D(NLY)) * G (NLY)
 AM2(N) =   (1._dp-ALDIF*F(NLY)) * XH(NLY)
 S(N)   =   ((Y(NLY)*ALDIF-X(NLY)) * WU  +  ALDR) * DIR(ISS)
 S(1)   =  -Y(1)

 DO 20  L = 2 , NLY
   LA =   2*L-1
   LB =   2*L-2
   AM1(LB) =   D(L-1)*G(L-1)
   AM2(LB) =   F(L-1)*XH(L-1)
   AM3(LB) =  -D(L)
   AM4(LB) =  -F(L)
   AM1(LA) =   G(L-1)
   AM2(LA) =   XH(L-1)
   S(LB)   =   DIR(L)*WU*(Y(L)-Y(L-1))
   S(LA)   =   DIR(L)*WU*(X(L)-X(L-1))
   20 CONTINUE

!......................................................................

!     subroutine LINGAU -
!     input  - N,AM1,AM2,AM3,AM4,S
!     output - AM2,AM3,AM4,S

 CALL LINGAU(N,AM1,AM2,AM3,AM4,S)

!......................................................................

 UP (ISS) =  S(2*NLY-1)*G(NLY)+S(2*NLY)*XH(NLY)+X(NLY)*WU*DIR(ISS)
 DIF(ISS) =  S(2*NLY-1)*G(NLY)*D(NLY)+S(2*NLY)*XH(NLY)*F(NLY)+Y(NLY)*WU*DIR(ISS)
! write(18,1005)F(NLY),Y(NLY),DIR(ISS)
 ! 1005 FORMAT(3(1X,F30.10))

 DO 30  I = 1 , NLY
   DIF(I) = S(2*I-1)*D(I)+S(2*I)*F(I)+Y(I)*WU*DIR(I)
   UP (I) = S(2*I-1)+S(2*I)+X(I)* WU*DIR(I)
   30 CONTINUE

 HHH  =   DBLE(0._dp)
 HHD  =   DIR(1)
 DO 40  I = 2 , ISS
   DIF(I) = DMAX1(DIF(I),HHH)
   XNET = DIF(I) - UP(I)
   DX1  = DIF(I-1) + DIR(I-1)
   DX   = DIF(I)   + DIR(I)
    IF(DX.GT.DX1) THEN
      DIF(I) = DX1 - DIR(I)
      UP(I)  = DIF(I) - XNET
    ENDIF
!   write(18,*)DIF(I)
   IF (UP(I) .GT. HHD)     UP(I) = HHH
   UP(I) = DMAX1(UP(I),HHH)
   40 CONTINUE

 RETURN
 END

!-------------------------------------------------------------------------------
! SUBROUTINE: LINGAU
!
!> Solving the linear equation system ?? Made
!!
!! @param[in]     N      INTEGER Twice the number of atmospheric layers minus 1.
!!                       2*(NLAY-1).
!! @param[in,out] AM1    DOUBLE  ?
!!                       1D ARRAY with dimension 100.
!! @param[in,out] AM2    DOUBLE  ?
!!                       1D ARRAY with dimension 100.
!! @param[in,out] AM3    DOUBLE  ?
!!                       1D ARRAY with dimension 100.
!! @param[in,out] AM4    DOUBLE  ? <br>
!!                       1D ARRAY with dimension 100.
!! @param[in,out] S      DOUBLE  ? <br>
!!                       1D ARRAY with dimension 100.
!!
!! @warning    All arrays have hardcoded dimensions. They should be declared
!!             using a parameter.
!!
!! @see        STRPSRB MATBAU
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE LINGAU(N,AM1,AM2,AM3,AM4,S)
      ! use gradevars
      use Brasilsr_constants
!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     N  :
!     AM3     :
!     AM4     :
!     AM1     :
!     AM2     :
!     S  :
!     N3 :
!     L  :
!     L1 :
!     L2 :
!     A  :
!     N1 :
!     I  :
!     II :
!     I1 :
!     I2 :
!......................................................................

 IMPLICIT NONE
 INTEGER, INTENT(IN)::N
 DOUBLE PRECISION, DIMENSION(LAYMAX2), INTENT(INOUT)::AM1,AM2,AM3,AM4,S

 INTEGER:: N3, L, L1, L2, N1, II, I, I1, I2
 DOUBLE PRECISION :: A

 ! DIMENSION AM1(100),AM2(100),AM3(100),AM4(100),S(100)

 N3   =  N - 3

 DO 10  L  =  1 , N3 , 2
    L1 =  L  + 1
    L2 =  L1 + 1
    A  = -AM1(L1) / AM3(L)
    S(L1)   =  A * S(L) + S(L1)
    AM2(L1) =  AM2(L1) + A * AM4(L)
    A  = -AM1(L2) / AM3(L)
    S(L2)   =  A * S(L) + S(L2)
    AM2(L2) =  AM2(L2) + A * AM4(L)
    A  = -AM2(L2) / AM2(L1)
    S(L2)   =  S(L2) + A * S(L1)
    AM3(L2) =  A * AM3(L1) - 1._dp
    AM4(L2) =  A * AM4(L1) - 1._dp
   10 CONTINUE

 N1   =  N - 1
 A    = -AM1(N) / AM3(N1)
 S(N) =  (S(N) + A * S(N1))/ (AM2(N) + A * AM4(N1))

 DO 20  II =  1 , N3 , 2
    I  =  N1 - II
    I1 =  I  + 1
    I2 =  I1 + 1
    S(I1)   =  (S(I1) - AM4(I1) * S(I2)) / AM3(I1)
    S(I)    =  (S(I) - AM3(I) * S(I1) - AM4(I) * S(I2)) / AM2(I)
   20 CONTINUE

 S(1) = (S(1) - AM4(1) * S(2)) / AM3(1)

 RETURN
 END

!-------------------------------------------------------------------------------
! FUNCTION: PTZQ1
!
!> Calculates temperature for a given atmospheric level, based on data from the
!! model atmospheres described by McClatchey (1972) @cite mcclatchey1972optical.
!!
!! @param[in]     DZZP   DOUBLE  Altitude of the atmospheric layer [m].
!! @param[in]     LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                               1. tropical <br>
!!                               2. midlatitude summer <br>
!!                               3. midlatitude winter <br>
!!                               4. subartic summer <br>
!!                               5. subartic winter
!!
!! @return        DOUBLE Temperature at height DZZP for an atmosphere of the
!!                       type LATMOS, based on McClatchey (1972)
!!                       @cite mcclatchey1972optical.
!!
!! @see        PTZQD
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 DOUBLE PRECISION FUNCTION PTZQ1(DZZP,LATMOS)
use Brasilsr_constants, only:dp

 IMPLICIT NONE

 INTEGER, INTENT(IN)::LATMOS
 DOUBLE PRECISION, INTENT(IN)::DZZP
 DOUBLE PRECISION, DIMENSION(10,6) :: DZB,DELTA
 DOUBLE PRECISION, DIMENSION(11,6) :: C
 DOUBLE PRECISION, DIMENSION(6) :: TSTAR

 INTEGER :: NLAST, N
 DOUBLE PRECISION :: DTEMP,DEXPO,DEXPP,DFAC,DX,DY,DZLOG,DFACLG

 !----->>> tropical sounding <<<
 !DATA (DZB(N,1),N=1,10)
 DZB(:,1) = (/2._dp,3._dp,16.5_dp,21.5_dp,45._dp,51._dp,70._dp,100._dp,200._dp,300._dp/)
 !DATA (C(N,1),N=1,11)
  C(:,1)  = (/-6._dp,-4._dp,-6.7_dp,4._dp,2.2_dp,1._dp,-2.8_dp,-0.27_dp,0._dp,0._dp,0._dp/)
 !DATA (DELTA(N,1),N=1,10)
  DELTA(:,1)  = (/0.5_dp,0.5_dp,0.3_dp,0.5_dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp/)

 !----->>> midlatitude summer <<<
 !DATA (DZB(N,2),N=1,10)
  DZB(:,2) = (/1.5_dp,6.5_dp,13._dp,18._dp,26._dp,36._dp,48._dp,50._dp,70._dp,100._dp/)
  !DATA (C(N,2),N=1,11)
  C(:,2) = (/-4._dp,-6._dp,-6.5_dp,0._dp,1.2_dp,2.2_dp,2.5_dp,0._dp,-3._dp,-0.25_dp,0._dp/)
  !DATA(DELTA(N,2),N=1,10)
  DELTA(:,2) = (/0.5_dp,1._dp,0.5_dp,0.5_dp,1._dp,1._dp,2.5_dp,0.5_dp,1._dp,1._dp/)

 !----->>> midlatitude winter <<<
 !DATA (DZB(N,3),N=1,10)
  DZB(:,3) = (/3._dp,10._dp,19._dp,25._dp,32._dp,44.5_dp,50._dp,71._dp,98._dp,200._dp/)
  !DATA (C(N,3),N=1,11)
  C(:,3) = (/-3.5_dp,-6._dp,-0.5_dp,0._dp,0.4_dp,3.2_dp,1.6_dp,-1.8_dp,-0.7_dp,0._dp,0._dp/)
 !DATA (DELTA(N,3),N=1,10)
  DELTA(:,3) = (/0.5_dp,0.5_dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp/)

 !----->>> subarctic summer <<<
 !DATA (DZB(N,4),N=1,10)
  DZB(:,4) = (/4.7_dp,10._dp,23._dp,31.8_dp,44._dp,50.2_dp,69.2_dp,100._dp,102._dp,103._dp/)
  !DATA (C(N,4),N=1,11)
  C(:,4) = (/-5.3_dp,-7._dp,0._dp,1.4_dp,3._dp,0.7_dp,-3.3_dp,-0.2_dp,0._dp,0._dp,0._dp/)
  !DATA(DELTA(N,4),N=1,10)
  DELTA(:,4) = (/0.5_dp,0.3_dp,1._dp,1._dp,2._dp,1._dp,1.5_dp,1._dp,1._dp,1._dp/)

 !----->>> subarctic winter <<<
 !DATA (DZB(N,5),N=1,10)
  DZB(:,5) = (/1._dp,3.2_dp,8.5_dp,15.5_dp,25._dp,30._dp,35._dp,50._dp,70._dp,100._dp/)
  !DATA (C(N,5),N=1,11)
  C(:,5) = (/3._dp,-3.2_dp,-6.8_dp,0._dp,-0.6_dp,1._dp,1.2_dp,2.5_dp,-0.7_dp,-1.2_dp,0._dp/)
  !DATA (DELTA(N,5),N=1,10)
  DELTA(:,5) = (/0.4_dp,1.5_dp,0.3_dp,0.5_dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp/)

 !----->>> us standard 1976 <<<
 !DATA (DZB(N,6),N=1,10)
  DZB(:,6) = (/11._dp,20.1_dp,32.1_dp,47.4_dp,51.4_dp,71.7_dp,85.7_dp,90._dp,91._dp,92._dp/)
  !DATA (C(N,6),N=1,11)
  C(:,6) = (/-6.5_dp,0._dp,1._dp,2.75_dp,0._dp,-2.75_dp,-1.97_dp,0._dp,0._dp,0._dp,0._dp/)
  !DATA (DELTA(N,6),N=1,10)
  DELTA(:,6) = (/0.3_dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp,1._dp/)

  !> Array TSTAR contains the temperature [in K] at height H=0 for each type
  !! of atmosphere described in McClatchey (1972) @cite mcclatchey1972optical,
  !! Tables 1 and B1.
  TSTAR(:) = (/300._dp, 294._dp, 272.2_dp, 287._dp, 257.1_dp, 288.15_dp/)

 NLAST=10
 DTEMP=TSTAR(LATMOS)+C(1,LATMOS)*DZZP

 DO 10 N=1,NLAST
 DEXPO=(DZZP-DZB(N,LATMOS))/DELTA(N,LATMOS)
 DEXPP=DZB(N,LATMOS)/DELTA(N,LATMOS)
 DFAC=DEXP(DEXPP)+DEXP(-DEXPP)

 IF((DABS(DEXPO)-100._dp).LE.0.0_dp) THEN
 DX=DEXP(DEXPO)
 DY=DX+1./DX
 DZLOG=DLOG(DY)
 ELSE
 DZLOG=DABS(DEXPO)
 END IF

 IF((DEXPP-100._dp).LT.0.0_dp) THEN
 DFACLG=DLOG(DFAC)
 ELSE
 DFACLG=DEXPP
 END IF

 DTEMP=DTEMP+(C(N+1,LATMOS)-C(N,LATMOS))*0.5_dp*(DZZP+DELTA(N,LATMOS)*(DZLOG-DFACLG))
   10 CONTINUE

 PTZQ1=DTEMP
 RETURN
 END

!-------------------------------------------------------------------------------
! FUNCTION: PTZQ2
!
!> Calculates water content for a given atmospheric level, based on data
!! from the model atmospheres described by McClatchey (1972) @cite
!! mcclatchey1972optical.
!!
!! @param[in]     P      DOUBLE  Pressure at the atmospheric layer [mb].
!! @param[in]     LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                               1. tropical <br>
!!                               2. midlatitude summer <br>
!!                               3. midlatitude winter <br>
!!                               4. subartic summer <br>
!!                               5. subartic winter
!! @param[in]     QL      DOUBLE Air density, computed as:
!!                               \f[
!!                                 \rho = 348.36764 \frac{P}{T}
!!                               \f]
!!
!! @return        DOUBLE Water content (humidity) at atmospheric level of the
!!                       type LATMOS, based on McClatchey (1972) @cite
!!                       mcclatchey1972optical.
!!
!! @see        PTZQD
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 DOUBLE PRECISION FUNCTION PTZQ2(P,LATMOS,QL)
      ! use gradevars
      use Brasilsr_constants, only:dp
 IMPLICIT NONE

 DOUBLE PRECISION, INTENT(IN) ::QL
 DOUBLE PRECISION, INTENT(IN) ::P
 INTEGER, INTENT(IN)::LATMOS

 DOUBLE PRECISION, DIMENSION(6,6) :: A,B,PRAT
 DOUBLE PRECISION, DIMENSION(6) :: QS,PS,PMIN, QCONST
 DOUBLE PRECISION :: PR,ETA, Pcopy
 INTEGER :: N
 ! DIMENSION A(6,6),B(6,6),QS(6),PRAT(6,6),PS(6)
 ! DIMENSION PMIN(6),QCONST(6)
  !> Variable QS contains water vapor data at height H=0 from model atmospheres
  !! described in McClatchey (1972) @cite mcclatchey1972optical, Tables 1
  !! (pp. 3-7) and B1 (pp. 94).
  !! Units: \f$ [g/m^3] \f$
  QS(:) = (/19._dp,14._dp,3.5_dp,9.1_dp,1.2_dp,14._dp/)

 ! DATA ((A(I,J),I=1,6),J=1,6)
  A(:,1) = (/3.55_dp,1.37_dp,3.99_dp,2.79_dp,8.66_dp,4.66_dp/)
  A(:,2) = (/3.31_dp,4.77_dp,4.19_dp,1.3_dp,8.21_dp,4.52_dp/)
  A(:,3) = (/2.72_dp,2.11_dp,3.41_dp,5.44_dp,3.91_dp,0.0_dp/)
  A(:,4) = (/3.76_dp,2.95_dp,0.83_dp,4.71_dp,7.43_dp,4.28_dp/)
  A(:,5) = (/0.0_dp,-0.59_dp,4.99_dp,4.18_dp,3.3_dp,0.0_dp/)
  A(:,6) = (/3.31_dp,4.77_dp,4.19_dp,1.3_dp,8.21_dp,4.52_dp/)

 ! DATA ((B(I,J),I=1,6),J=1,6)
  B(:,1) = (/1.91_dp,-7.58_dp,-0.41_dp,-1.52_dp,1.81_dp,0.0_dp/)
  B(:,2) = (/-1.83_dp,0.67_dp,0.0_dp,-2.26_dp,1.67_dp,0.0_dp/)
  B(:,3) = (/0.39_dp,-2.02_dp,-0.81_dp,0.71_dp,0.0_dp,0.0_dp/)
  B(:,4) = (/2.37_dp,-1.01_dp,-3.17_dp,0.0_dp,1.41_dp,0.0_dp/)
  B(:,5) = (/0.0_dp,-4.46_dp,0.86_dp,0.38_dp,0.0_dp,0.0_dp/)
  B(:,6) = (/-1.83_dp,0.67_dp,0.0_dp,-2.26_dp,1.67_dp,0.0_dp/)

 !DATA ((PRAT(I,J),I=1,6),J=1,6)
  PRAT(:,1) = (/0.795_dp,0.694_dp,0.339_dp,0.172_dp,0.11_dp,0._dp/)
  PRAT(:,2) = (/0.558_dp,0.421_dp,0.278_dp,0.172_dp,0.11_dp,0._dp/)
  PRAT(:,3) = (/0.776_dp,0.342_dp,0.263_dp,0.10_dp,0._dp,0._dp/)
  PRAT(:,4) = (/0.787_dp,0.375_dp,0.294_dp,0.145_dp,0.107_dp,0._dp/)
  PRAT(:,5) = (/0.876_dp,0.35_dp,0.185_dp,0.10_dp,0.0_dp,0.0_dp/)
  PRAT(:,6) = (/0.558_dp,0.421_dp,0.278_dp,0.172_dp,0.11_dp,0._dp/)

  !> Variable PS contains pressure data at height H=0 from model atmospheres
  !Variable Pmin contains pressure data at height H=15 km from model atmospheres
  !! described in McClatchey (1972) @cite mcclatchey1972optical, Tables 1
  !! (pp. 3-7) and B1 (pp. 94).
  !! Units:  \f$ [mb] \f$

 ! DATA (PS(I),I=1,6)
  PS(:) = (/1013._dp,1013._dp,1018._dp,1010._dp,1013._dp,1013._dp/)
  !DATA (PMIN(I),I=1,6)
  PMIN(:) = (/131.99_dp,129.99_dp,117.79_dp,124.99_dp,110.29_dp,120.99_dp/)
  ! PMIN(:) = (/110.99_dp,152.99_dp,117.79_dp,124.99_dp,110.29_dp,152.99_dp/)
 ! DATA (QCONST(I),I=1,6)
  QCONST(:) = (/3.25D-06,4.0D-06,4.0D-06,4.0D-06,4.0D-06,4.0D-06/)

 Pcopy=P
 IF(P.GT.PS(LATMOS)) Pcopy=PS(LATMOS)
 PR=Pcopy/PS(LATMOS)
 N=1
  300 IF(PR.GE.PRAT(N,LATMOS)) GOTO 100
 IF(Pcopy.LE.PMIN(LATMOS)) GOTO 200
 N=N+1
 GOTO 300
  100 ETA=A(N,LATMOS)+B(N,LATMOS)*DLOG(PR)
 PTZQ2=QS(LATMOS)*PR**ETA
! print*,'dados',PR,P,QL, LATMOS,N,ETA,PTZQ2
 RETURN
  200 PTZQ2=QCONST(LATMOS)*QL
 RETURN

 END

!-------------------------------------------------------------------------------
! FUNCTION: PTZQ3
!
!> Calculates ozone content for a given atmospheric level, based on data
!! from the model atmospheres described by McClatchey (1972) @cite
!! mcclatchey1972optical.
!!
!! @param[in]     PPP    DOUBLE  Pressure at the atmospheric layer [mb].
!! @param[in]     LATMOS INTEGER Atmosphere index. Possible values: <br>
!!                               1. tropical <br>
!!                               2. midlatitude summer <br>
!!                               3. midlatitude winter <br>
!!                               4. subartic summer <br>
!!                               5. subartic winter
!!
!! @return        DOUBLE Pressure in [mb] at atmospheric level of the type
!!                       LATMOS, based on McClatchey (1972) @cite
!!                       mcclatchey1972optical.
!!
!! @see        PTZQD
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 DOUBLE PRECISION FUNCTION PTZQ3(PPP,LATMOS)
      use gradevars

 IMPLICIT NONE

 DOUBLE PRECISION, INTENT(IN):: PPP
 INTEGER,INTENT(IN):: LATMOS
 DOUBLE PRECISION, DIMENSION (33,6):: P,O

 DOUBLE PRECISION :: PP,A,B
 INTEGER :: N

 ! DIMENSION P(33,6),O(33,6)
 !> Variable P contains pressure data from model atmospheres described in
 !! McClatchey (1972) @cite mcclatchey1972optical, Tables 1 (pp. 3-7) and
 !! B1 (pp. 94).
 !! Original units: \f$ [mb] \f$
 !! Table units: \f$ [Pa] \f$
 ! P(:,1) - Tropical
 ! DATA (P(I,1),I=1,33)
 P(:,1) = (/1.013D+05,9.04D+04,8.05D+04,7.15D+04,6.33D+04,5.59D+04,&
             4.92D+04,4.32D+04,3.78D+04,3.29D+04,2.86D+04,2.47D+04,&
             2.13D+04,1.82D+04,1.56D+04,1.32D+04,1.11D+04,9.37D+03,&
             7.89D+03,6.66D+03,5.65D+03,4.80D+03,4.09D+03,3.50D+03,&
             3.00D+03,2.57D+03,1.22D+03,6.00D+02,3.05D+02,1.59D+02,&
             8.54D+01,5.79D+00,3.00D-02/)
 ! P(:,2) - Midlatitude Summer
 ! DATA (P(I,2),I=1,33)
 P(:,2) = (/1.013D+05,9.02D+04,8.02D+04,7.10D+04,6.28D+04,5.54D+04, &
             4.87D+04,4.26D+04,3.72D+04,3.24D+04,2.81D+04,2.43D+04, &
             2.09D+04,1.79D+04,1.53D+04,1.30D+04,1.11D+04,9.50D+03, &
             8.12D+03,6.95D+03,5.95D+03,5.10D+03,4.37D+03,3.76D+03, &
             3.22D+03,2.77D+03,1.32D+03,6.52D+02,3.33D+02,1.76D+02, &
             9.51D+01,6.71D+00,3.00D-02/)
 ! P(:,3) - Midlatitude Winter
 ! DATA (P(I,3),I=1,33)
 P(:,3) = (/1.018D+05,8.973D+04,7.897D+04,6.938D+04,6.081D+04,5.313D+04, &
            4.627D+04,4.016D+04,3.473D+04,2.992D+04,2.568D+04,2.199D+04, &
            1.882D+04,1.610D+04,1.378D+04,1.178D+04,1.007D+04,8.610D+03, &
            7.350D+03,6.280D+03,5.370D+03,4.580D+03,3.910D+03,3.340D+03, &
            2.860D+03,2.430D+03,1.110D+03,5.180D+02,2.530D+02,1.290D+02, &
            6.820D+01,4.670D+00,3.000D-02/)
 ! P(:,4) - Subartic Summer
 ! DATA (P(I,4),I=1,33)
 P(:,4) = (/1.010D+05,8.960D+04,7.929D+04,7.000D+04,6.160D+04,5.410D+04, &
            4.730D+04,4.130D+04,3.590D+04,3.107D+04,2.677D+04,2.300D+04, &
            1.977D+04,1.700D+04,1.460D+04,1.250D+04,1.080D+04,9.280D+03, &
            7.980D+03,6.860D+03,5.890D+03,5.070D+03,4.360D+03,3.750D+03, &
            3.227D+03,2.780D+03,1.340D+03,6.610D+02,3.400D+02,1.810D+02, &
            9.870D+01,7.070D+00,3.000D-02/)
 ! P(:,5) - Subartic Winter
 ! DATA (P(I,5),I=1,33)
 P(:,5) = (/1.013D+05,8.878D+04,7.775D+04,6.798D+04,5.932D+04,5.158D+04, &
            4.467D+04,3.853D+04,3.308D+04,2.829D+04,2.418D+04,2.067D+04, &
            1.766D+04,1.510D+04,1.291D+04,1.103D+04,9.431D+03,8.058D+03, &
            6.882D+03,5.875D+03,5.014D+03,4.277D+03,3.647D+03,3.109D+03, &
            2.649D+03,2.256D+03,1.020D+03,4.701D+02,2.243D+02,1.113D+02, &
            5.719D+01,4.016D+00,3.000D-02/)
 ! P(:,6) - US Standard Atmosphere 1962: Table B1, Apprendix B, pp. 94.
! DATA (P(I,6),I=1,33)
 P(:,6) = (/1.013D+05,8.988D+04,7.950D+04,7.012D+04,6.166D+04,5.405D+04, &
           4.722D+04,4.111D+04,3.565D+04,3.080D+04,2.650D+04,2.270D+04, &
           1.940D+04,1.658D+04,1.417D+04,1.211D+04,1.035D+04,8.850D+03, &
           7.565D+03,6.467D+03,5.529D+03,4.729D+03,4.048D+03,3.467D+03, &
           2.972D+03,2.549D+03,1.197D+03,5.746D+02,2.871D+02,1.491D+02, &
           7.978D+01,5.220D+00,3.008D-02/)
 !> Variable O contains ozone data from model atmospheres described in
 !! McClatchey (1972) @cite mcclatchey1972optical, Tables 1 (pp. 3-7) and
 !! B1 (pp. 94).
 !! Original units: \f$ [g/m^3] \f$
 !! Table units: \f$ [kg/m^3] \f$
!  ! O(:,1) - Tropical
! ! DATA (O(I,1),I=1,33)
 ! O(:,1) = (/5.6D-08,5.6D-08,5.4D-08,5.1D-08,4.7D-08,4.5D-08,4.3D-08,4.1D-08, &
 !            3.9D-08,3.9D-08,3.9D-08,4.1D-08,4.3D-08,4.5D-08,4.5D-08,4.7D-08, &
 !            4.7D-08,6.9D-08,9.0D-08,1.4D-07,1.9D-07,2.4D-07,2.8D-07,3.2D-07, &
 !            3.4D-07,3.4D-07,2.4D-07,9.2D-08,4.1D-08,1.3D-08,4.3D-09,8.6D-11, &
 !            4.3D-14/)
 ! ! O(:,2) - Midlatitude Summer
 ! ! DATA (O(I,2),I=1,33)
 ! O(:,2) = (/6.0D-08,6.0D-08,6.0D-08,6.2D-08,6.4D-08,6.6D-08,6.9D-08,7.5D-08, &
 !            7.9D-08,8.6D-08,9.0D-08,1.1D-07,1.2D-07,1.5D-07,1.8D-07,1.9D-07, &
 !            2.1D-07,2.4D-07,2.8D-07,3.2D-07,3.4D-07,3.6D-07,3.6D-07,3.4D-07, &
 !            3.2D-07,3.0D-07,2.0D-07,9.2D-08,4.1D-08,1.3D-08,4.3D-09,8.6D-11, &
 !            4.3D-14/)
 ! ! O(:,3) - Midlatitude Winter
 ! O(:,3) = (/6.0D-08,5.4D-08,4.9D-08,4.9D-08,4.9D-08,5.8D-08,6.4D-08,7.7D-08, &
 !            9.0D-08,1.2D-07,1.6D-07,2.1D-07,2.6D-07,3.0D-07,3.2D-07,3.4D-07, &
 !            3.6D-07,3.9D-07,4.1D-07,4.3D-07,4.5D-07,4.3D-07,4.3D-07,3.9D-07, &
 !            3.6D-07,3.4D-07,1.9D-07,9.2D-08,4.1D-08,1.3D-08,4.3D-09,8.6D-11, &
 !            4.3D-14/)
 ! ! O(:,4) - Subartic Summer
 ! O(:,4) = (/4.9D-08,5.4D-08,5.6D-08,5.8D-08,6.0D-08,6.4D-08,7.1D-08,7.5D-08, &
 !            7.9D-08,1.1D-07,1.3D-07,1.8D-07,2.1D-07,2.6D-07,2.8D-07,3.2D-07, &
 !            3.4D-07,3.9D-07,4.1D-07,4.1D-07,3.9D-07,3.6D-07,3.2D-07,3.0D-07, &
 !            2.8D-07,2.6D-07,1.4D-07,9.2D-08,4.1D-08,1.3D-08,4.3D-09,8.6D-11, &
 !            4.3D-14/)
 ! ! O(:,5) - Subartic Winter
 ! O(:,5) = (/4.1D-08,4.1D-08,4.1D-08,4.3D-08,4.5D-08,4.7D-08,4.9D-08,7.1D-08, &
 !            9.0D-08,1.6D-07,2.4D-07,3.2D-07,4.3D-07,4.7D-07,4.9D-07,5.6D-07, &
 !            6.2D-07,6.2D-07,6.2D-07,6.0D-07,5.6D-07,5.1D-07,4.7D-07,4.3D-07, &
 !            3.6D-07,3.2D-07,1.5D-07,9.2D-08,4.1D-08,1.3D-08,4.3D-09,8.6D-11, &
 !            4.3D-14/)
 ! ! O(:,6) - US Standard Atmosphere 1962: Table B1, Apprendix B, pp. 94.
 ! O(:,6) = (/5.4D-08,5.4D-08,5.4D-08,5.0D-08,4.6D-08,4.6D-08,4.5D-08,4.9D-08, &
 !            5.2D-08,7.1D-08,9.0D-08,1.3D-07,1.6D-07,1.7D-07,1.9D-07,2.1D-07, &
 !            2.4D-07,2.8D-07,3.2D-07,3.5D-07,3.8D-07,3.8D-07,3.9D-07,3.8D-07, &
 !            3.6D-07,3.4D-07,2.0D-07,1.1D-07,4.9D-08,1.7D-08,5.3D-09,4.3D-11, &
 !            4.3D-14/)
!! Variable O contains ozone data from model atmospheres described in
!! Anderson (1986) @cite Anderson1986, Tables 1 (pp. 4-9)
!! Original units: \f$ [ppmV] \f$
!! Table units: \f$ [kg/m^3] \f$
!! DATA (O(I,1),I=1,33)
 O(:,1) = (/5.600D-08,5.599D-08,5.400D-08,5.100D-08,4.700D-08,4.499D-08,4.300D-08,4.100D-08, &
            3.899D-08,3.900D-08,3.899D-08,4.100D-08,4.299D-08,4.499D-08,4.498D-08,4.700D-08, &
            4.699D-08,6.945D-08,1.146D-07,1.803D-07,2.210D-07,2.368D-07,2.642D-07,3.167D-07, &
            3.399D-07,3.620D-07,2.821D-07,1.383D-07,5.201D-08,1.560D-08,5.111D-09,4.591D-11, &
            3.501D-13/)
! O(:,2) - Midlatitude Summer
! DATA (O(I,2),I=1,33)
O(:,2) = (/5.999D-08,6.000D-08,5.999D-08,6.200D-08,6.400D-08,6.600D-08,6.900D-08,7.499D-08, &
           7.899D-08,8.601D-08,8.993D-08,1.100D-07,1.211D-07,1.437D-07,1.802D-07,1.740D-07, &
           1.783D-07,1.780D-07,2.163D-07,2.763D-07,3.135D-07,3.207D-07,3.303D-07,3.314D-07, &
           3.322D-07,3.411D-07,2.283D-07,1.367D-07,5.639D-08,1.695D-08,5.578D-09,7.096D-11, &
           3.129D-13/)
! ! O(:,3) - Midlatitude Winter
O(:,3) = (/6.000D-08,5.400D-08,4.899D-08,4.899D-08,4.899D-08,5.800D-08,6.400D-08,7.699D-08, &
           8.994D-08,1.200D-07,1.600D-07,2.100D-07,2.600D-07,3.000D-07,2.924D-07,2.819D-07, &
           2.952D-07,3.220D-07,3.547D-07,3.876D-07,4.179D-07,4.302D-07,4.092D-07,3.854D-07, &
           3.607D-07,3.340D-07,1.7999D-07,9.320D-08,4.145D-08,1.326D-08,4.082D-09,3.765D-11, &
           4.305D-14/)
! O(:,4) - Subartic Summer
! DATA (O(I,2),I=1,33)
O(:,4) = (/4.899D-08,5.400D-08,5.600D-08,5.801D-08,6.000D-08,6.400D-08,7.099D-08,7.499D-08, &
           7.900D-08,1.098D-07,1.297D-07,1.799D-07,2.079D-07,2.180D-07,2.246D-07,2.262D-07, &
           2.354D-07,2.380D-07,2.660D-07,2.991D-07,3.177D-07,3.510D-07,3.690D-07,3.558D-07, &
           3.455D-07,3.167D-07,1.876D-07,1.189D-07,5.244D-08,1.613D-08,5.141D-09,7.572D-11, &
           3.009D-13/)
! O(:,5) - Subartic Winter
O(:,5) = (/4.099D-08,4.100D-08,4.099D-08,4.299D-08,4.499D-08,4.700D-08,4.899D-08,7.099D-08, &
           9.007D-08,1.580D-07,1.929D-07,1.924D-07,1.878D-07,2.610D-07,3.089D-07,3.519D-07, &
           3.772D-07,4.093D-07,4.521D-07,4.897D-07,5.002D-07,4.626D-07,4.153D-07,3.804D-07, &
           3.323D-07,2.899D-07,1.473D-07,7.572D-08,3.256D-08,1.067D-08,3.312D-09,4.707D-11, &
           4.472D-13/)
! ! O(:,6) - US Standard Atmosphere 1962
O(:,6) = (/5.400D-08,5.401D-08,5.400D-08,5.001D-08,4.600D-08,4.600D-08,4.500D-08,4.900D-08, &
           5.200D-08,7.100D-08,8.978D-08,1.300D-07,1.603D-07,1.701D-07,1.900D-07,2.098D-07, &
           2.400D-07,2.807D-07,3.206D-07,3.499D-07,3.802D-07,3.803D-07,3.902D-07,3.802D-07, &
           3.602D-07,3.401D-07,1.999D-07,1.100D-07,4.834D-08,1.711D-08,5.276D-09,4.118D-11, &
           3.789D-13/)

 PP=PPP
 PP=PP*100.
 N=2
  200 IF(PP.GT.P(N,LATMOS)) GOTO 100
 N=N+1
 GOTO 200
  100 IF(PP.EQ.P(N-1,LATMOS)) GOTO 300
 B=(O(N-1,LATMOS)-O(N,LATMOS))/(P(N-1,LATMOS)-P(N,LATMOS))
 A=O(N,LATMOS)-B*P(N,LATMOS)
 PTZQ3=A+B*PP
 RETURN
  300 PTZQ3=O(N-1,LATMOS)
 RETURN
 END
