!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! SUBROUTINE: D2STR
!
!> Makes calls to routines that alculates albedo, solar position parameters,
!  type of atmosphere and radiative transfer calculations
!!
!! @param[in]  JDAY   INTEGER     day of year
!! @param[in]  XLON   DOUBLE      Longitude of the ground station [degrees].
!! @param[in]  YLAT   DOUBLE      Latitude of the ground station [degrees].
!! @param[in]  BIOMA INTEGER Brazilian biome:<br>
!!                            1: Amazonia -> aerosol type 4
!!                            3: Cerrado -> aerosol type 5
!!                            all other values -> aerosol type 3 (continental polluted)
!! @param[in]  TSTAT  DOUBLE      Temperature at the ground station [K].
!! @param[in]  fiso   DOUBLE   isotropic kernel coeficient
!! @param[in]  fgeo   DOUBLE   geometric kernel coeficient   (LiSparse-Reciprocal)
!! @param[in]  fvol   DOUBLE   volumetric kernel coeficient (RossThick)
!! @param[in]  RFSURF     DOUBLE      Relative humidity [0-1].
!! @param[in]  PWV   DOUBLE   Precipitable water vapor (cm)
!! @param[in]  O3   DOUBLE   Column ozone (Dobson)
!! @param[in]  ZSTAT  DOUBLE      Surface elevation [m].
!! @param[in]  Kext   DOUBLE, DIMENSION(5) Extinction coeficient from 0 to 5 km.
!! @param[in]  XHOR   DOUBLE      Time the sattelite image was generated [h].
!! @param[in]  ICLOUD INTEGER     Cloud parameter. <br>
!!                                0: no clouds <br>
!!                                1: calculation with clouds
!! @param[in]  TAUW   DOUBLE      Cloud optical thickness. [0-100] <br>
!!                                0.0:   no clouds <br>
!!                                100.0: completely cloudy with maximum optical thickness clouds
!! @param[out] TRANS  DOUBLE      Parameterized sky transmittance for broadband
!!                                solar radiation in the atmosphere.
!! @param[out] TDIR   DOUBLE      Downward direct flux.
!! @param[in] EAngstrom   DOUBLE, optional  Angstrom exponent. If not entered, a climatological value is used.
!! @param[in] scaling LOGICAL, optional whether use delta-Eddington scaling or not.
!! @param[in] IP integer, index representing grid point in spectral variables.
!!
!!@see BRASILSR
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE D2STR(JDAY,XLON,YLAT,BIOMA,TSTAT,fiso,fgeo,fvol,RFSURF,PWV,O3,ZSTAT,Kext,XHOR,ICLOUD,TAUW,TRANS,TDIR,COSZ,EAngstrom,scaling,ip)
      use gradevars
      use Brasilsr_interface
      use Brasilsr_constants

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!     XLON    : longitude of the ground station (o)
!     YLAT    : latitude of the ground station (o)
!     TSTAT   : temperature at ground station (K)
!     WH2O    : precipitable water (cm)
!     SFCALB  : surface albedo (0/1)
!     Kext(5) : Extinction coeficient from 0 to 5 km
!     JDAY    : julian day
!     PI      : pi
!     CDR     : change degree to radians
!     SC0     : solar constant (W/m2)
!     E0      : eccentricity correction factor of the earth's orbit
!     DEC     : solar declination (degrees)
!     DECR    : solar declination (radians)
!     YLATR   : latitude of the ground station (radians)
!     CODEC   : cossine of solar declination
!     SIDEC   : sine of solar declination
!     SILAT   : sine of latitude of ground station
!     ET      : equation of time (minutes)
!     ZN      : time zone number (3.0 - Florianopolis) (hours)
!     TIMCOR  : time correction (hours)
!     T1S     : standard time at begin of integration interval (hours)
!     T2S     : standard time at end of integration interval (hours)
!     T1A     : apparent time at begin of integration interval (hours)
!     T2A     : apparent time at end of integration interval (hours)
!     IT      : hour of sattelite image
!     COWI    : cossine of hour angle at middle of integration interval
!     COSZEN  : cossine of zenith angle
!     THETA   : zenith angle (degrees)
!     ROFF    : difference in water vapor
!     IWP     : cloud droplet size distribution (possible values: 1 to 8)
!     ISUB    : use subroutine WOLKE1 (ISUB=1) or WOLKE2 (ISUB=2)
!     TOP     : cloud top (mbar)
!     NCL     : number of cloud layers
!     INTVAL  : spectral interval
!     CLOLWC  : cloud liquid water content
!     W1      : hour angle at begin of interval (degrees)
!     W2      : hour angle at end of interval (degrees)
!     WI      : characteristic hour angle (degrees)
!     LATMOS  : atmosphere type
!               1 - tropical
!               2 - midlatitude summer
!               3 - midlatitude winter
!               4 - subartic summer
!               5 - subartic winter
!     TS      : difference between characteristical temperature of LATMOS
!              and temperature at gound station (oC)
!     ICLOUD  : cloud parameter
!              0 - no clouds
!              1 - calculation with clouds
!     TAUW    : cloud optical thickness
!              0.0 - no clouds
!              100.0 - with clouds
!     TRANS   : parameterized sky transmittance for broadband solar radiation
!              in the atmosphere
!......................................................................

      ! IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT NONE
      INTEGER, INTENT(IN)::JDAY,BIOMA
      DOUBLE PRECISION, INTENT(IN)::XLON,YLAT,TSTAT,RFSURF,ZSTAT,PWV,O3
      DOUBLE PRECISION, dimension(:), INTENT(IN) :: fiso,fgeo,fvol
      DOUBLE PRECISION, dimension(:), intent(in) :: Kext
      DOUBLE PRECISION, intent(in), optional :: EAngstrom
      DOUBLE PRECISION, intent(in) :: XHOR
      INTEGER, intent(in) :: ICLOUD
      DOUBLE PRECISION, intent(in) ::TAUW
      DOUBLE PRECISION, intent(out) ::TRANS,TDIR, COSZ
      INTEGER, INTENT(IN),OPTIONAL::ip
      LOGICAL, intent(in):: scaling
      DOUBLE PRECISION::RF,PI,CDR,E0,DEC,ET,DECR,YLATR,CODEC,COLAT,SIDEC,SILAT,ZN,TIMCOR
      DOUBLE PRECISION::ROFF,TOP,CLOLWC,TS,TSOLAR,WSOLAR,COWI,COSZEN,THETA,THETARAD
      INTEGER :: IWP,ISUB,NCL,INTVAL,LATMOS,k

      DOUBLE PRECISION, DIMENSION(MAXWEL)::ALDIF,ALDR
      INTEGER, DIMENSION(MAXWEL):: previous_modis, next_modis
      DOUBLE PRECISION, dimension(MAXWEL) :: brfiso,brfgeo,brfvol, XWSA, XBSA

      DOUBLE PRECISION, dimension(37)::weightmodis
      CHARACTER(256) :: my_errmsg

!    Weight used to interpolate between two MODIS wavelenghts to BRASIL-SR model wavelenghts,
!    used in conjunction with previous_modis and next_modis variables, that determine which two modis will be used
!    An unexistent eight band with fiso=fgeo=fvol=0 was conceived for interpolation purposes
      weightmodis = (/0.000000, 0.030303, 0.0707071, 0.1111111, 0.1515151, 0.1919192, 0.2323232, 0.2727273, 0.3131313, 0.3535353,&
                & 0.4040405,0.4747475,0.5555556,0.6363637,0.7373737,0.8484849,1.000000,&
                & 0.5000000, 1.000000,&
                & 0.5000000, 1.000000,&
                & 0.1872248,0.3744495,0.506608,0.561674,0.7753304,1.000000,&
                & 0.1896317,0.4529332,0.7312415,1.000000,&
                & 0.2822893,0.5862335,1.000000,&
                & 1.000000, &
                & 0.3892083, 0.000000/)
      previous_modis=(/8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,&
                  & 3,3,4,4,1,1,1,1,1,1, &
                  & 2,2,2,2,5,5,5,6,7,8/)
      next_modis=(/3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, &
                  & 4,4,1,1,2,2,2,2,2,2, &
                  & 5,5,5,5,6,6,6,7,8,8/)

      ! PARAMETER (NHOUR=24)
      ! DOUBLE PRECISION, DIMENSION(5), INTENT(IN) :: Kext
!     definition of constants bellow

      PI    = DACOS(-1.0_dp)
      CDR  = PI/180.0_dp

!......................................................................
!     subroutine ASTRO - calculation of eccentricity correction,
!     declination an equation of time
!     input  - JDAY
!     output - E0,DEC,ET
!
      ! SFCALB = SFALB/100.0_dp
      RF     = RFSURF/100.0_dp

      CALL ASTRO(JDAY,E0,DEC,ET)



!......................................................................

!     change degrees to radians and calculate sines and cossines


      DECR = DEC*CDR
      YLATR= YLAT*CDR
      CODEC = DCOS(DECR)
      COLAT = DCOS(YLATR)
      SIDEC = DSIN(DECR)
      SILAT = DSIN(YLATR)


!     calculate time correction

      ZN = 0.0_dp
      TIMCOR = (4.0_dp*(15.0_dp*ZN+XLON)+ET)/60.0_dp

      !write(*,*) 'd2str'



!......................................................................
!     calculate transmittance for characteristic day

!     fixed input parameters for subroutine strpsrb

      ROFF   = 0.0_dp
      IWP    = 3  !There are 8 possible cloud droplet size distributions
      ISUB   = 2  !This value means WOLKE2 is called always, since WOLKE1 requires ISUB   = 1
      TOP    = 500.0_dp
      NCL    = 2  !Initial value, recalculated in WOLKE2 (or WOLKE1, if called)
      INTVAL = 3
      CLOLWC = 0.0_dp

!......................................................................
!     subroutine ATMOSPHERE - choose atmosphere by surface temperature
!     input  - TSTAT
!     output - LATMOS,TS

      CALL ATMOSPHERE(TSTAT,zstat,LATMOS,TS)

!......................................................................
!     construction of matrix TRANS

      TSOLAR = XHOR+TIMCOR    !para entrada com horario em UTC
!      TSOLAR = XHOR+ET        !para entrada com horario local
!      T1S = XHOR-1.5
!      T1A = T1S+TIMCOR
!      T2S = XHOR+1.5
!      T2A = T2S+TIMCOR

!     characteristic hour angle WI corresponding to the W1-W2 interval

      WSOLAR    = (12.00_dp - TSOLAR)*15.0_dp
      COWI  = DCOS(WSOLAR*CDR)
!      W1    = (12.00 - T1A)*15.  !para entrada com horario local
!      W2    = (12.00 - T2A)*15.
!      WI    = (W1 + W2)*0.5
!      COWI  = COS(WI*CDR)

!     characteristic solar zenith angle THETA (o)

      COSZEN = SIDEC*SILAT + CODEC*COLAT*COWI
      COSZ=COSZEN
      ! THETA  = DACOS(COSZEN)/CDR
      THETA  = DACOSD(COSZEN)
      THETA = DMAX1(THETA,0.1_dp)
      THETARAD = DACOS(COSZEN)


     IF ((USE_SPECTRAL_ALB).AND.(THETA.LE.90)) THEN

          brfiso(1)=0.D0
          brfgeo(1)=0.D0
          brfvol(1)=0.D0
          brfiso(37)=0.D0
          brfgeo(37)=0.D0
          brfvol(37)=0.D0
          DO k=2, 36
            brfiso(k)= fiso(next_modis(k))*weightmodis(k)+fiso(previous_modis(k))*(1-weightmodis(k))
            brfvol(k)= fvol(next_modis(k))*weightmodis(k)+fvol(previous_modis(k))*(1-weightmodis(k))
            brfgeo(k)= fgeo(next_modis(k))*weightmodis(k)+fgeo(previous_modis(k))*(1-weightmodis(k))
          ENDDO
          DO k=1, 37
            ALDIF(k) = WSA(brfiso(k), brfgeo(k), brfvol(k))
            ALDR(k) = BSA(brfiso(k), brfgeo(k), brfvol(k), THETARAD)
          ENDDO

     ELSE
          ALDIF(:) = WSA(fiso(1), fgeo(1), fvol(1))
          ALDR(:)  = BSA(fiso(1), fgeo(1), fvol(1), THETARAD)
     ENDIF

!......................................................................
!     subroutine STRPSRB - radiative transfer calculations

      IF(THETA.LE.90) THEN
          CALL STRPSRB(BIOMA,LATMOS,TS,ROFF,ALDIF(:),ALDR(:),PWV,O3,Kext(:),THETA,E0,ICLOUD,IWP,ISUB,TAUW,TOP,NCL,INTVAL,tstat,rf,zstat,CLOLWC,CDR,TRANS,TDIR,EAngstrom=EAngstrom,scaling=scaling,ip=ip)
      ELSE
        TRANS=-2.0_dp
        TDIR=-2.0_dp
      ENDIF
!......................................................................

      RETURN
      END

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-------------------------------------------------------------------------------
! FUNCTION: BSA
!
!> Determination of directional-hemispherical reflectance (black albedo).
!!
!! @param[in]  fiso   DOUBLE    isotropic kernel coeficient
!! @param[in]  fgeo   DOUBLE    geometric kernel coeficient   (LiSparse-Reciprocal)
!! @param[in]  fvol   DOUBLE    volumetric kernel coeficient (RossThick)
!! @param[in]  Z      DOUBLE    solar zenith angle [radians]
!!
!! @see        STRPSRB
!!
!! @author     LABREN
!! @version    1.0
!! @date       2021
!-------------------------------------------------------------------------------
 DOUBLE PRECISION function BSA(fiso, fgeo,fvol,Z)
  use Brasilsr_constants
   DOUBLE PRECISION, intent (in) ::fiso, fgeo,fvol,Z
   DOUBLE PRECISION, PARAMETER :: g0vol = 0.007574_dp, g1vol = 0.070987_dp, g2vol = 0.307588_dp
   DOUBLE PRECISION, parameter :: g0geo = 1.284909_dp, g1geo = 0.166314_dp, g2geo = 0.041840_dp

   BSA=fiso+fvol*(-g0vol-g1vol*Z**2+g2vol*Z**3)+fgeo*(-g0geo-g1geo*Z**2+g2geo*Z**3)

 end function BSA
 !-------------------------------------------------------------------------------
 ! FUNCTION: WSA
 !
 !> Determination of bi-hemispherical reflectance (white albedo).
 !!
 !! @param[in]  DOUBLE   fiso   isotropic kernel coeficient
 !! @param[in]  DOUBLE   fgeo   geometric kernel coeficient   (LiSparse-Reciprocal)
 !! @param[in]  DOUBLE   fvol   volumetric kernel coeficient (RossThick)
 !!
 !! @see        STRPSRB
 !!
 !! @author     LABREN
 !! @version    1.0
 !! @date       2020
 !-------------------------------------------------------------------------------
 DOUBLE PRECISION function WSA(fiso, fgeo,fvol)
   use Brasilsr_constants
   DOUBLE PRECISION, intent (in) ::fiso, fgeo,fvol
   DOUBLE PRECISION, PARAMETER :: wsvol=0.189184_dp, wsgeo=1.377622_dp

   WSA=fiso+fvol*wsvol-fgeo*wsgeo

 end function WSA
